@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-#   Copyright 2009-2021 Oli Schacher, Fumail Project
+#   Copyright 2009-2022 Oli Schacher, Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -240,8 +240,12 @@ class Bounce(object):
         targethost = self._get_targethost()
         outgoingport = self.config.getint('main', 'outgoingport')
         helostring = get_outgoing_helo(self.config)
-        smtp_server = FugluSMTPClient(host=targethost, port=outgoingport, local_hostname=helostring)
-        smtp_server.sendmail(fromaddress, toaddress, message)
+        try:
+            smtp_server = FugluSMTPClient(host=targethost, port=outgoingport, local_hostname=helostring)
+            smtp_server.sendmail(fromaddress, toaddress, message)
+        except smtplib.SMTPException as e:
+            self.logger.error(f'failed to send mail from={fromaddress} to={toaddress} via={targethost} port={outgoingport} due to {e.__class__.__name__} {str(e)}')
+            raise
         smtp_server.quit()
         return smtp_server.queueid
     
@@ -250,9 +254,13 @@ class Bounce(object):
         targethost = self._get_targethost()
         outgoingport = self.config.getint('main', 'outgoingport')
         helostring = get_outgoing_helo(self.config)
-        smtp_server = FugluAioSMTPClient(hostname=targethost, port=outgoingport, source_address=helostring)
-        smtp_server.connect()
-        send_resp = smtp_server.sendmail(fromaddress, toaddress, message)
+        try:
+            smtp_server = FugluAioSMTPClient(hostname=targethost, port=outgoingport, source_address=helostring)
+            smtp_server.connect()
+            send_resp = smtp_server.sendmail(fromaddress, toaddress, message)
+        except SMTPException as e:
+            self.logger.error(f'failed to send mail from={fromaddress} to={toaddress} via={targethost} port={outgoingport} due to {e.__class__.__name__} {str(e)}')
+            raise
         smtp_server.quit()
         return smtp_server.parse_postfixreply(send_resp[1])
     
