@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-#   Copyright 2009-2021 Fumail Project
+#   Copyright 2009-2022 Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -157,19 +157,20 @@ class MilterHandler(TrackTimings):
             self.log("milter_mode: setting to default value: 'tags'")
             configstring = "tags"
 
-        if configstring not in ["auto", "readonly", "tags", "replace_demo", "manual"]:
+        if configstring not in ["auto", "readonly", "tags", "replace_demo", "manual", "autoheaders"]:
             self.logger.warning("milter_mode: '%s' not recognised, resetting to 'tags'" % configstring)
 
-        self.enable_mode_manual = ("manual" in configstring)
-        self.enable_mode_auto = ("auto" in configstring)
-        self.enable_mode_readonly = ("readonly" in configstring)
-        self.enable_mode_tags = ("tags" in configstring)
-        self.replace_demo = ("replace_demo" in configstring)
+        self.enable_mode_manual = ("manual" == configstring)
+        self.enable_mode_auto = ("auto" == configstring)
+        self.enable_mode_autoheaders = ("autoheaders" == configstring)
+        self.enable_mode_readonly = ("readonly" == configstring)
+        self.enable_mode_tags = ("tags" == configstring)
+        self.replace_demo = ("replace_demo" == configstring)
 
         self.sess_options = 0x00 if self.enable_mode_readonly else lm.SMFIF_ALLOPTS
 
         self.logger.debug(f"{createPIDinfo()}: new MilterHandler (asyncid: {asyncid})")
-        self.log(f"Milter mode: auto={self.enable_mode_auto}, "
+        self.log(f"Milter mode: auto={self.enable_mode_auto}, autoheaders={self.enable_mode_autoheaders}"
                  f"readonly={self.enable_mode_readonly}, "
                  f"tags={self.enable_mode_tags}")
 
@@ -325,7 +326,7 @@ class MilterHandler(TrackTimings):
             except Exception as e:
                 CrashStore.store_exception()
                 exc = traceback.format_exc()
-                self.logger.error(f'{suspect.id}Plugin %s failed: %s' % (str(plugin), exc))
+                self.logger.error(f'{suspect.id} Plugin %s failed: %s' % (str(plugin), exc))
                 suspect.debug('Plugin failed : %s . Please check fuglu log for more details' % e)
                 ptag = suspect.get_tag("processingerrors", defaultvalue=[])
                 ptag.append("Plugin %s failed: %s" % (str(plugin), str(e)))
@@ -505,6 +506,11 @@ class MilterSession(lmp.ASYNCMilterProtocol):
         except Exception:
             self.async_conlost_timeout = None
 
+        try:
+            self.ignoreclient = config.getboolean('milter', 'ignoreclient')
+        except Exception:
+            self.ignoreclient = False
+
         self.be_verbose = BE_VERBOSE
 
         if self.be_verbose:
@@ -545,10 +551,10 @@ class MilterSession(lmp.ASYNCMilterProtocol):
         self.addheaders = {}
 
         # tags (will be passed to Suspect)
-        self.tags = {}
+        self.tags = copy.deepcopy(Suspect._default_tags)
 
         # tags backup to reset to connect
-        self.tags_backup = {}
+        self.tags_backup = copy.deepcopy(self.tags)
 
         self.logger.debug(f"{createPIDinfo()}: new MilterSession")
 
@@ -659,17 +665,23 @@ class MilterSession(lmp.ASYNCMilterProtocol):
     def reset_connection(self):
         """Reset all variables except to prepare for a second mail through the same connection.
         keep helo (heloname), ip address (addr) and hostname (fcrdns)"""
-        if self.id and self.mhandler:
-            self.mhandler.report_timings(suspectid=self.id, withrealtime=True)
+        if self.id:
+            if self.mhandler:
+                self.logger.debug(f"{self.id} (reset_connection) - report timings")
+                self.mhandler.report_timings(suspectid=self.id, withrealtime=True)
+            else:
+                self.logger.debug(f"{self.id} (reset_connection) - can't report timings because there's no mhandler object")
+        else:
+            self.logger.debug(f"(reset_connection) - can't report timings because there's no milter id (id={self.id})")
         self.recipients = []
         self.original_headers = []
         self._buffer = None
         if self.tempfilename and os.path.exists(self.tempfilename):
             try:
                 os.remove(self.tempfilename)
-                self.logger.info("Abort -> removed temp file: %s" % self.tempfilename)
+                self.logger.info(f"{self.id } Abort -> removed temp file: {self.tempfilename}")
             except OSError:
-                self.logger.error("Could not remove tmp file: %s" % self.tempfilename)
+                self.logger.error(f"{self.id} Could not remove tmp file: {self.tempfilename}")
                 pass
         self.tempfilename = None
         # postfix queue id
@@ -853,7 +865,7 @@ class MilterSession(lmp.ASYNCMilterProtocol):
             returncode = res
             message = ""
         except Exception as e:
-            self.logger.error(f"handle_milter_plugin_reply exception: {str(e)}", exc_info=e)
+            self.logger.error(f"handle_milter_plugin_reply exception (input:{res}): {str(e)}", exc_info=e)
             outres = res
             outres, returncode = MilterSession.milter_return_code(incode=outres)
             returncode = res
@@ -1558,7 +1570,7 @@ class MilterSession(lmp.ASYNCMilterProtocol):
             if self.tags:
                 suspect.tags.update(self.tags)
 
-            if self.heloname is not None and self.addr is not None and self.fcrdns is not None:
+            if self.heloname is not None and self.addr is not None and self.fcrdns is not None and not self.ignoreclient:
                 suspect.clientinfo = force_uString(self.heloname), force_uString(self.addr), force_uString(self.fcrdns)
 
             suspect.tags['incomingport'] = self.mhandler.port
@@ -1698,7 +1710,7 @@ class MilterSession(lmp.ASYNCMilterProtocol):
             pass
 
     async def abort(self):
-        self.logger.debug('Abort has been called')
+        self.logger.debug(f'{self.id} Abort has been called -> call reset_connection')
         self.reset_connection()
 
     async def replacebody(self, newbody):
@@ -1907,13 +1919,9 @@ class MilterSession(lmp.ASYNCMilterProtocol):
             for recipient in suspect.recipients:
                 await self.add_rcpt(recipient)
 
-        if self.mhandler.enable_mode_auto and not replace_headers:
-            self.logger.warning(f"{suspect.id} Modify({len(suspect.added_headers)})/add({len(suspect.modified_headers)}"
+        if (self.mhandler.enable_mode_auto or self.mhandler.enable_mode_autoheaders) and not replace_headers:
+            self.logger.warning(f"{suspect.id} Modify {len(suspect.modified_headers)}"
                                 f" headers according to modification track in suspect")
-            for key, val in iter(suspect.added_headers.items()):
-                hdr = Header(val, header_name=key, continuation_ws=' ')
-                await self.addheader(key, hdr.encode())
-
             for key, val in iter(suspect.modified_headers.items()):
                 hdr = Header(val, header_name=key, continuation_ws=' ')
                 await self.changeheader(key, hdr.encode())
@@ -2035,9 +2043,9 @@ class ProcLocalDict(object):
     @classmethod
     def instance(cls) -> tp.Dict:
         pid = os.getpid()
-        logger = logging.getLogger("%s.CacheSingleton" % __package__)
+        logger = logging.getLogger("%s.%s" % (__package__, cls.__class__.__name__))
         if pid == ProcLocalDict.procPID and ProcLocalDict.instance is not None:
-            logger.debug("Return existing Cache Singleton for process with pid: %u"%pid)
+            logger.debug("Return existing ProcLocalDict Singleton for process with pid: %u"%pid)
         else:
             if ProcLocalDict.instance is None:
                 logger.info("Create ProcLocalDict for process with pid: %u"%pid)
