@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-#   Copyright 2009-2021 Oli Schacher, Fumail Project #
+#   Copyright 2009-2022 Oli Schacher, Fumail Project #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
@@ -65,7 +65,7 @@ class ESMTPHandler(ProtocolHandler):
         if suspect.get_tag('noreinject'):
             # in esmtp sessions we don't want to provide info to the connecting
             # client
-            return 250, 'OK'
+            return 250, 'OK', '<>'
         if suspect.get_tag('reinjectoriginal'):
             self.logger.info('Injecting original message source without modifications')
             msgcontent = suspect.get_original_source()
@@ -408,7 +408,7 @@ class ESMTPPassthroughSession(object):
         if len(self.dataAccum) > 4 and self.dataAccum[-5:] == b'\r\n.\r\n':
             # check if there is more data to write to the file
             if len(data) > 4:
-                self.tempfile.write(data[0:-5])
+                self.tempfile.write(data[0:-3])
 
             self._close_tempfile()
 
@@ -420,7 +420,7 @@ class ESMTPPassthroughSession(object):
 
     def unquoteData(self, data):
         """two leading dots at the beginning of a line must be unquoted to a single dot"""
-        return re.sub(b'(?m)^\.\.', b'.', data)
+        return re.sub(rb'(?m)^\.\.', b'.', data)
 
     def stripAddress(self, address):
         """
@@ -432,6 +432,6 @@ class ESMTPPassthroughSession(object):
         retaddr, remaining = smtp_strip_address(address)
 
         # keeping this check as it was in the previous code
-        if retaddr != '' and re.match("^[^@]+@[^@]+\.[^@]+$", retaddr) is None:
+        if retaddr != '' and re.match(r"^[^@]+@[^@]+\.[^@]+$", retaddr) is None:
             raise ValueError("Could not parse address %s" % address)
         return retaddr
