@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-#   Copyright 2009-2021 Fumail Project
+#   Copyright 2009-2022 Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,6 +21,7 @@ import tempfile
 import os
 import time
 import typing as tp
+import copy
 
 from typing import Tuple, Union
 from email.header import Header
@@ -94,8 +95,6 @@ HEADER = "header"
 EOH = "eoh"
 EOB = "eob"
 
-from asyncore import dispatcher
-
 class MilterHandler(ProtocolHandler):
     protoname = 'MILTER V6'
 
@@ -122,17 +121,19 @@ class MilterHandler(ProtocolHandler):
         if configstring not in ["auto", "readonly", "tags", "replace_demo", "manual"]:
             self.logger.warning("milter_mode: '%s' not recognised, resetting to 'tags'" % configstring)
 
-        self.enable_mode_manual = ("manual" in configstring)
-        self.enable_mode_auto = ("auto" in configstring)
-        self.enable_mode_readonly = ("readonly" in configstring)
-        self.enable_mode_tags = ("tags" in configstring)
-        self.replace_demo = ("replace_demo" in configstring)
+        self.enable_mode_manual = ("manual" == configstring)
+        self.enable_mode_auto = ("auto" == configstring)
+        self.enable_mode_autoheaders = ("autoheaders" == configstring)
+        self.enable_mode_readonly = ("readonly" == configstring)
+        self.enable_mode_tags = ("tags" == configstring)
+        self.replace_demo = ("replace_demo" == configstring)
 
         sess_options = 0x00 if self.enable_mode_readonly else lm.SMFIF_ALLOPTS
         self.sess = MilterSession(sock, config, options=sess_options)
 
-        self.logger.debug("Milter mode: auto=%s, readonly=%s, tags=%s" %
-                         (self.enable_mode_auto, self.enable_mode_readonly, self.enable_mode_tags))
+        self.logger.debug("Milter mode: auto=%s, readonly=%s, tags=%s, autoheaders=%s" %
+                         (self.enable_mode_auto, self.enable_mode_readonly, self.enable_mode_tags,
+                          self.enable_mode_autoheaders))
 
         # options (can be combined into a string): "all" "body" "headers" "from" "to"
         try:
@@ -416,12 +417,9 @@ class MilterHandler(ProtocolHandler):
             for recipient in suspect.recipients:
                 self.add_rcpt(recipient)
 
-        if self.enable_mode_auto and not replace_headers:
+        if (self.enable_mode_auto or self.enable_mode_autoheaders) and not replace_headers:
             self.logger.warning(f"{suspect.id} Modify({len(suspect.added_headers)})/add({len(suspect.modified_headers)}"
                                 f" headers according to modification track in suspect")
-            for key, val in iter(suspect.added_headers.items()):
-                hdr = Header(val, header_name=key, continuation_ws=' ')
-                self.addheader(key, hdr.encode())
 
             for key, val in iter(suspect.modified_headers.items()):
                 hdr = Header(val, header_name=key, continuation_ws=' ')
@@ -588,7 +586,7 @@ class MilterSession(lm.MilterProtocol):
         self.milterplugins = {}
         self.addheaders = {}
         # tags (will be passed to Suspect)
-        self.tags = {}
+        self.tags = copy.deepcopy(Suspect._default_tags)
 
     @property
     def sasl_user(self):
@@ -686,7 +684,7 @@ class MilterSession(lm.MilterProtocol):
         self.sasl_sender = None
         self.sasl_method = None
         self.addheaders = {}
-        self.tags = {}
+        self.tags = copy.deepcopy(Suspect._default_tags)
         self.id = Suspect.generate_id()
 
     def _clean_address(self, address: tp.Optional[bytes]) -> tp.Optional[bytes]:
