@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-#   Copyright 2009-2021 Oli Schacher, Fumail Project
+#   Copyright 2009-2022 Oli Schacher, Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -110,7 +110,7 @@ class SMTPHandler(ProtocolHandler):
     def re_inject(self, suspect):
         """Send message back to postfix"""
         if suspect.get_tag('noreinject'):
-            return 250, 'message not re-injected by plugin request'
+            return 250, 'message not re-injected by plugin request', '<>'
 
         if suspect.get_tag('reinjectoriginal'):
             self.logger.info('%s: Injecting original message source without modifications' % suspect.id)
@@ -502,7 +502,7 @@ class SMTPSession(object):
     
     def unquoteData(self, data):
         """two leading dots at the beginning of a line must be unquoted to a single dot"""
-        return re.sub(b'(?m)^\.\.', b'.', force_bString(data))
+        return re.sub(rb'(?m)^\.\.', b'.', force_bString(data))
     
     
     def stripAddress(self, address):
