@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-#   Copyright 2009-2021 Oli Schacher, Fumail Project
+#   Copyright 2009-2022 Oli Schacher, Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -196,7 +196,7 @@ class MainController(DefConfigMixin):
             'group': {
                 'section': 'main',
                 'description': "run as group",
-                'default': "nobody",
+                'default': "nobody, nogroup",
             },
 
             'plugindir': {
@@ -451,24 +451,36 @@ class MainController(DefConfigMixin):
             },
 
             # databaseconfig
+            'dbpriority': {
+                'default': "sql",
+                'section': 'databaseconfig',
+                'description': """read runtime configuration values from database types specified in this order. currently supported db types: sql, rest""",
+            },
+
             'dbconnectstring': {
                 'default': "",
                 'section': 'databaseconfig',
-                'description': """read runtime configuration values from a database. requires sqlalchemy to be installed""",
+                'description': """read runtime configuration values from SQL database. requires sqlalchemy to be installed""",
                 'confidential': True,
             },
             
             'sql': {
-                'default': """SELECT value FROM fugluconfig WHERE `section`=:section AND `scope` IN ('$GLOBAL',CONCAT('%',:to_domain),:to_address) AND `option`=:option ORDER BY `scope` DESC LIMIT 1""",
+                'default': """SELECT value FROM fugluconfig WHERE `section`=:section AND `scope` IN (:globalscope,CONCAT('%',:to_domain),:to_address) AND `option`=:option ORDER BY `scope` DESC LIMIT 1""",
                 'section': 'databaseconfig',
                 'description': """sql query that returns a configuration value override. sql placeholders are ':section',':option' in addition the usual suspect filter default values like ':to_domain', ':to_address' etc\nif the statement returns more than one row/value only the first value in the first row is used""",
             },
             
             'sqlsection': {
-                'default': """SELECT value, option, scope FROM fugluconfig WHERE `section`=:section AND `scope` IN ('$GLOBAL',CONCAT('%',:to_domain),:to_address) ORDER BY `scope` DESC""",
+                'default': """SELECT value, option, scope FROM fugluconfig WHERE `section`=:section AND `scope` IN (:globalscope,CONCAT('%',:to_domain),:to_address) ORDER BY `scope` DESC""",
                 'section': 'databaseconfig',
                 'description': """sql query that returns all values, options and scopes (relevant for one recipient) of one section. sql placeholders ':section' available in addition the usual suspect filter default values like ':to_domain', ':to_address' etc""",
             },
+
+            'globalscope': {
+                'default': "$GLOBAL",
+                'section': 'databaseconfig',
+                'description': """name of global scope""",
+            },
     
             'option_field': {
                 'default': "option",
@@ -485,7 +497,33 @@ class MainController(DefConfigMixin):
                 'section': 'databaseconfig',
                 'description': """what is the name of the "value" field in the database""",
             },
+
+            'restapi_uri': {
+                'default': "",
+                'section': 'databaseconfig',
+                'description': """REST API base URI""",
+                'confidential': True,
+            },
             
+            'restapi_endpoint': {
+                'default': "",
+                'section': 'databaseconfig',
+                'description': """REST API endpoint path to config overrides""",
+                'confidential': True,
+            },
+
+            'restapi_headers': {
+                'default': "",
+                'section': 'databaseconfig',
+                'description': """list of additional headers, e.g. for API authentication""",
+                'confidential': True,
+            },
+            
+            'restapi_verify': {
+                'default': "True",
+                'section': 'databaseconfig',
+                'description': """verify REST API server's SSL certificate""",
+            },
             
             # environment
             'boundarydistance': {
@@ -550,7 +588,7 @@ class MainController(DefConfigMixin):
                 'section': 'PluginAlias',
             },
 
-            'm_ca': {
+            'm_callahead': {
                 'default': "fuglu.plugins.call_ahead.AddressCheck",
                 'section': 'PluginAlias',
             },
@@ -769,6 +807,11 @@ class MainController(DefConfigMixin):
                 'default': "fuglu.plugins.antivirus.FprotPlugin",
                 'section': 'PluginAlias',
             },
+            
+            'cyren': {
+                'default': "fuglu.plugins.antivirus.CyrenPlugin",
+                'section': 'PluginAlias',
+            },
 
             'scriptfilter': {
                 'default': "fuglu.plugins.script.ScriptFilter",
@@ -968,8 +1011,8 @@ class MainController(DefConfigMixin):
             numprocs = int(numprocs)
             self.logger.info(f"Set number of worker processors for procpool according to input to {numprocs}")
         elif -1.0e-6 < numprocs < 1.0e-6:
-            self.logger.info(f"Set number of worker processors for procpool automatically (2*#procs) to {numprocs}")
             numprocs = multiprocessing.cpu_count() * 2
+            self.logger.info(f"Set number of worker processors for procpool automatically (2*#procs) to {numprocs}")
         else:
             numprocs_in = numprocs
             numprocs = max(int(multiprocessing.cpu_count() * abs(numprocs)), 1)
@@ -985,8 +1028,8 @@ class MainController(DefConfigMixin):
             numprocs = int(numprocs)
             self.logger.info(f"Set number of worker processors for async procpool according to input to {numprocs}")
         elif -1.0e-6 < numprocs < 1.0e-6:
-            self.logger.info(f"Set number of worker processors for async procpool automatically (2*#procs) to {numprocs}")
             numprocs = multiprocessing.cpu_count() * 2
+            self.logger.info(f"Set number of worker processors for async procpool automatically (2*#procs) to {numprocs}")
         else:
             numprocs_in = numprocs
             numprocs = max(int(multiprocessing.cpu_count() * abs(numprocs)), 1)
@@ -1578,7 +1621,7 @@ class MainController(DefConfigMixin):
     def _load_all(self, plugins):
         """load all plugins from config string. returns tuple ([list of loaded instances],allOk)"""
         pluglist = []
-        config_re = re.compile("""^(?P<structured_name>[a-zA-Z0-9._-]+)(?:\((?P<config_override>[a-zA-Z0-9._-]+)\))?$""")
+        config_re = re.compile(r"^(?P<structured_name>[a-zA-Z0-9._-]+)(?:\((?P<config_override>[a-zA-Z0-9._-]+)\))?$")
         allOK = True
         for plug in plugins:
             if plug == "":
