@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-#   Copyright 2009-2021 Oli Schacher, Fumail Project
+#   Copyright 2009-2022 Oli Schacher, Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -95,13 +95,13 @@ class ControlServer(object):
             pass
 
     def serve(self):
-        threading.currentThread().name = 'ControlServer Thread'
+        threading.current_thread().name = 'ControlServer Thread'
         controller = self.controller
 
         self.logger.info('Control/Info Server running on port %s' % self.port)
         while self.stayalive:
+            engine = None
             try:
-                engine = None
                 self.logger.debug('Waiting for connection...')
                 nsd = self._socket.accept()
                 if not self.stayalive:
@@ -1006,8 +1006,8 @@ Block:\t\t${blockedcount}
         return newlist
 
     @staticmethod
-    def buildfilter(input_to_string=objgraph._long_typename, dont_startwith=[],
-                    must_startwith=[], dont_contain=[], must_contain=[], lowercase=True):
+    def buildfilter(input_to_string=objgraph._long_typename, dont_startwith=(),
+                    must_startwith=(), dont_contain=(), must_contain=(), lowercase=True):
         """
         Build a filter that can be passed to the obgraph filter keyword
 
@@ -1095,13 +1095,12 @@ class CrashStore(object):
             exc_info = sys.exc_info()
 
         if thread is None:
-            thread = threading.currentThread()
-
-        name = thread.getName()
+            thread = threading.current_thread()
+        
         info = ""
         if hasattr(thread, 'threadinfo'):
             info = thread.threadinfo
-        desc = "%s (%s)" % (name, info)
+        desc = "%s (%s)" % (thread.name, info)
 
         maxtracebacks = 10
         CrashStore.exceptions.append((exc_info, time.time(), desc),)
