@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-#   Copyright 2009-2021 Fumail Project
+#   Copyright 2009-2022 Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,15 +14,27 @@
 # limitations under the License.
 #
 #
-#
+
+class _DummyExc(Exception):
+    pass
+
 
 import ipaddress
 
 STATUS = "not loaded"
+_NXDOMAIN = _DummyExc
+_TIMEOUT = _DummyExc
+_PYDNSEXC = _DummyExc
+_NONAMESERVER = _DummyExc
+_NOANSWER = _DummyExc
 try:
-    from dns import resolver
+    from dns import resolver, exception
     HAVE_DNSPYTHON=True
     STATUS = "available"
+    _NXDOMAIN = resolver.NXDOMAIN
+    _TIMEOUT = exception.Timeout
+    _NONAMESERVER = resolver.NoNameservers
+    _NOANSWER = resolver.NoAnswer
 except ImportError:
     resolver = None
     HAVE_DNSPYTHON=False
@@ -34,6 +46,7 @@ if not HAVE_DNSPYTHON:
         HAVE_PYDNS=True
         DNS.DiscoverNameServers()
         STATUS = "available"
+        _PYDNSEXC = DNS.Base.ServerError
     except ImportError:
         DNS = None
         STATUS = "DNS not installed"
@@ -60,20 +73,54 @@ QTYPE_TLSA = 'TLSA'
 
 
 
-def lookup(hostname, qtype=QTYPE_A):
+class FuNXDOMAIN(Exception):
+    pass
+
+class FuTIMEOUT(Exception):
+    pass
+
+class FuSERVFAIL(Exception):
+    pass
+
+class FuNoNameserver(Exception):
+    pass
+
+class FuNoAnswer(Exception):
+    pass
+
+
+def lookup(hostname, qtype=QTYPE_A, reraise=False):
     try:
         if HAVE_DNSPYTHON:
             arecs = []
-            arequest = resolver.query(hostname, qtype)
+            arequest = resolver.resolve(hostname, qtype)
             for rec in arequest:
                 arecs.append(rec.to_text())
             return arecs
 
         elif HAVE_PYDNS:
             return DNS.dnslookup(hostname, qtype)
-
+    
+    except _NXDOMAIN:
+        if reraise:
+            raise FuNXDOMAIN
+    except _TIMEOUT:
+        if reraise:
+            raise FuTIMEOUT
+    except _NONAMESERVER:
+        if reraise:
+            raise FuNoNameserver
+    except _NOANSWER:
+        if reraise:
+            raise FuNoAnswer
+    except _PYDNSEXC as e:
+        if reraise and 'NXDOMAIN' in e.message:
+            raise FuNXDOMAIN
+        if reraise and 'SERVFAIL' in e.message:
+            raise FuSERVFAIL
     except Exception:
-        return None
+        if reraise:
+            raise
 
     return None
 
@@ -83,7 +130,7 @@ def mxlookup(domain):
     try:
         if HAVE_DNSPYTHON:
             mxrecs = []
-            mxrequest = resolver.query(domain, QTYPE_MX)
+            mxrequest = resolver.resolve(domain, QTYPE_MX)
             for rec in mxrequest:
                 mxrecs.append(rec.to_text())
             mxrecs.sort()  # automatically sorts by priority
@@ -106,7 +153,7 @@ def mxlookup(domain):
 
 
 
-def revlookup(ip):
+def revlookup(ip, reraise=False):
     ipaddr = ipaddress.ip_address(ip)
     revip = ipaddr.reverse_pointer
-    return lookup(revip, qtype=QTYPE_PTR)
+    return lookup(revip, qtype=QTYPE_PTR, reraise=reraise)
