@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-#   Copyright 2009-2021 Fumail Project
+#   Copyright 2009-2022 Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -35,6 +35,7 @@ except ImportError:
     ENABLED = False
     StrictRedis = object
     redis = None
+    REDIS2 = False
     REDIS33X = None
 
 
@@ -89,7 +90,7 @@ class RedisKeepAlive(StrictRedis):
 
 
 class RedisPooledConn(object):
-    def __init__(self, redis_url=None, **args):
+    def __init__(self, redis_url:str=None, **args):
         self.logger = logging.getLogger('fuglu.RedisPooledConn')
         if 'password' in list(args.keys()) and args['password']:
             self.logger.warning(f'deprecated redis password config - include it in redis_conn URL')
@@ -108,14 +109,14 @@ class RedisPooledConn(object):
             self.pool = redis.ConnectionPool(host=host, port=port, db=int(db), **args)
 
 
-    def get_conn(self):
+    def get_conn(self) -> RedisKeepAlive:
         args = {}
         if self._pinginterval > 0:
             args['pinginterval'] = self._pinginterval
         return RedisKeepAlive(connection_pool=self.pool, **args)
     
     
-    def check_connection(self):
+    def check_connection(self) -> bool:
         if self.pool is None:
             return False
         else:
@@ -126,13 +127,13 @@ class RedisPooledConn(object):
 
 class ExpiringCounter(object):
     
-    def __init__(self, redis_pool:RedisPooledConn=None, ttl=0, maxcount=0):
+    def __init__(self, redis_pool:RedisPooledConn=None, ttl:int=0, maxcount:int=0):
         self.redis_pool = redis_pool or RedisPooledConn()
         self.ttl = ttl
         self.maxcount = maxcount
     
     
-    def _to_int(self, value, default:int=0):
+    def _to_int(self, value, default:int=0) -> int:
         """
         Convert to integer if possible
 
@@ -184,7 +185,7 @@ class ExpiringCounter(object):
             return 0
     
     
-    def get_count(self, key:str, cleanup:bool=True):
+    def get_count(self, key:str, cleanup:bool=True) -> int:
         """
         Get value. This is the sum of the count values within the ttl value stored in the class.
         Args:
@@ -214,7 +215,7 @@ class ExpiringCounter(object):
         return count
     
     
-    def cleanup(self):
+    def cleanup(self) -> None:
         """
         Remove stale entries from redis database
         """
