@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-#   Copyright 2009-2021 Fumail Project
+#   Copyright 2009-2022 Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -845,21 +845,29 @@ class Mailattachment(Cachelimits):
         if self.parent_archives:
             location += element_of + element_of.join(["{" + obj().filename + "}" for obj in self.parent_archives])
         return location
-    
-    
-    def get_mangled_filename(self, prefix='', maxchars=15):
+
+
+    @smart_cached_memberfunc(inputs=["filename"])
+    def get_mangled_filename(self, prefix: str = '', maxchars: int = 15, keepending: bool = False):
         first=self.filename.split('.')[0]
         #replace spaces with underscore
-        first = re.sub('\s', '_', first)
+        first = re.sub(r'\s', '_', first)
         #remove all non alnum chars
         first=''.join([x for x in first if x in string.ascii_letters or x in string.digits])
         if first.strip()=='' or first.strip()=='filename': #renattach renames to 'filename'
             first='upn-%s' % random.randint(1000,9999)
         elif len(first) < 4:
             first = 'sh-%s-%s' % (random.randint(1000,9999), first)
-        #strip to a machimum of maxchars characters
+        #strip to a maximum of maxchars characters
         first=first[:maxchars]
-        newfilename = '%s.%s' % (prefix, first)
+        newfilename = first
+        if prefix:
+            newfilename = f'{prefix}.{newfilename}'
+        if keepending and "." in self.filename:
+            ending = self.filename.rsplit(".", 1)[1]
+            if ending:
+                # if filename ends with a "." ending is empty
+                newfilename = f'{newfilename}.{ending}'
         return newfilename
     
     
