@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-#   Copyright 2009-2021 Fumail Project
+#   Copyright 2009-2022 Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -104,6 +104,8 @@ class ConfigWrapper(object):
 
 
 class DefConfigMixin(object):
+    requiredvars = {}
+    
     def __init__(self, config):
         self._config = ConfigWrapper(config, None)
         self._rawconfig = config
@@ -151,6 +153,10 @@ class SimpleTimeoutMixin(object):
 
 
 class ReturnOverrideMixin(object):
+    #config = None
+    #section = None
+    #_logger = None
+    
     def __init__(self, config, section: tp.Optional[str] = None):
         super().__init__(config, section=section)
         self._overrideaction = -1
@@ -210,9 +216,12 @@ class ReturnOverrideMixin(object):
                 plugin_msg = msg if msg else ""
                 override_return = actioncode_to_string(self.overrideaction)
                 override_msg = self._overridemessage if self._overridemessage else ""
-                self._logger().warning(f"{suspectid} overrideaction: "
-                                       f"plugin={plugin_return}/msg={plugin_msg}, "
-                                       f"override={override_return}/msg={override_msg}")
+                try:
+                    self._logger().warning(f"{suspectid} overrideaction: "
+                                           f"plugin={plugin_return}/msg={plugin_msg}, "
+                                           f"override={override_return}/msg={override_msg}")
+                except Exception:
+                    pass
                 ret = self.overrideaction
                 msg = self._overridemessage
 
