@@ -1,7 +1,22 @@
+# -*- coding: utf-8 -*-
+#   Copyright 2009-2022 Fumail Project
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+# http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
+#
+#
 import logging
 import time
-import asyncio
-import functools
 try:
     #from contextlib import AbstractAsyncContextManager
     from contextlib import AbstractContextManager
@@ -9,7 +24,7 @@ except ImportError:
     #AbstractAsyncContextManager = object
     AbstractContextManager = object
 import typing as tp
-from typing import Dict, Tuple, Union
+from typing import Tuple, Union
 from types import TracebackType
 
 import fuglu.connectors.milterconnector as sm
@@ -30,6 +45,7 @@ from fuglu.connectors.milterconnector import (
     DISCARD,
     TEMPFAIL,
     REJECT,
+    STR2RETCODE,
 )
 import fuglu.shared as fs
 
@@ -184,4 +200,17 @@ class BasicMilterPlugin(ReturnOverrideMixin, fs.BasicPlugin):
         """returns the logger for this plugin"""
         myclass = self.__class__.__name__
         loggername = "fuglu.mplugin.%s" % myclass
-        return logging.getLogger(loggername)
\ No newline at end of file
+        return logging.getLogger(loggername)
+    
+    
+    def _problemcode(self, configoption='problemaction'):
+        """
+        safely calculates action code based on problemaction config value
+        :return: action code
+        """
+        retcode = STR2RETCODE.get(self.config.get(self.section, configoption), self.config)
+        if retcode is not None:
+            return retcode
+        else:
+            # in case of invalid problem action
+            return CONTINUE
\ No newline at end of file
