@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-#   Copyright 2009-2021 Oli Schacher, Fumail Project
+#   Copyright 2009-2022 Oli Schacher, Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,4 +16,5 @@
 __all__ = ['archive', 'attachment', 'decision', 'clamav', 'sa', 'p_debug', 'p_blwl', 'a_logging', 'bacn',
            'p_skipper', 'p_fraction', 'vacation', 'antivirus', 'sssp', 'domainauth', 'script', 'a_statsd',
            'fuzor', 'rspamd', 'mailcopy', 'uriextract', 'call_ahead', 'delay', 'ivmsg', 'domainauth',
-           'tlspolicy', 'originpolicy', 'outpolicy', 'restrictions', ]
+           'tlspolicy', 'originpolicy', 'outpolicy', 'restrictions', 'geoip', 'messagesize', 'knownsubject',
+           'bacn',]
