@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-#   Copyright 2009-2021 Oli Schacher, Fumail Project
+#   Copyright 2009-2022 Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,11 +14,15 @@
 # limitations under the License.
 #
 #
-from fuglu.shared import AppenderPlugin, apply_template, get_outgoing_helo, FileList, actioncode_to_string, FuConfigParser
+from fuglu.shared import AppenderPlugin, apply_template, get_outgoing_helo, FileList, actioncode_to_string, FuConfigParser, Suspect
 from fuglu.stringencode import force_uString, force_bString
 from fuglu.mailattach import NoExtractInfo
 from fuglu.caching import smart_cached_memberfunc
+from fuglu.plugins.uriextract import EXCLUDE_DOMAIN, EXCLUDE_FQDN
+from fuglu.extensions.elastic import ElasticClient, ElasticException, lint_elastic
+from email.header import Header
 import datetime
+import time
 import re
 import os
 import json
@@ -26,18 +30,14 @@ import logging
 import urllib
 import urllib.parse
 import hashlib
-from domainmagic import extractor, tld
-from domainmagic.mailaddr import strip_batv, decode_srs, domain_from_mail, split_mail
-from domainmagic.validators import is_email
-from domainmagic.validators import is_url_tldcheck
-
-
+import typing as tp
 try:
-    import elasticsearch
-    HAVE_ELASTICSEARCH = True
+    from domainmagic import extractor, tld
+    from domainmagic.mailaddr import strip_batv, decode_srs, domain_from_mail, split_mail
+    from domainmagic.validators import is_email, is_url_tldcheck
+    HAVE_DOMAINMAGIC=True
 except ImportError:
-    elasticsearch = None
-    HAVE_ELASTICSEARCH = False
+    HAVE_DOMAINMAGIC=False
 
 
 FIELDMAP = {
@@ -45,8 +45,6 @@ FIELDMAP = {
     'injectqueueid': 'queueid',
     'Attachment.bounce.queueid': 'bounceqid'
 }
-LOG_EXCLUDE_FQDN=['www.w3.org', 'schemas.microsoft.com']
-LOG_EXCLUDE_DOMAIN=['avast.com']
 
 LOG_MAPPING = {
     "properties": {
@@ -56,6 +54,8 @@ LOG_MAPPING = {
         "blockedreport": {"type": "nested"},
         "uris": {"type": "nested"},
         "attachments": {"type": "nested"},
+        "client_ip": {"type": "ip"},
+        "size": {"type": "integer"},
     }
 }
 
@@ -142,15 +142,15 @@ class URILogger(object):
     def _get_domain_from_uri(self, uri, suspect=None):
         try:
             fqdn = extractor.domain_from_uri(uri)
-            if fqdn in LOG_EXCLUDE_FQDN:
-                return None
+            if fqdn in EXCLUDE_FQDN:
+                return None, None
         except Exception as e:
             # log error
             self.logger.error(f"{suspect.id if suspect else '<>'} msg: {str(e)} uri: {uri}")
-            return None
+            return None, None
         domain = self._get_domain_from_fqdn(fqdn, suspect)
-        if domain in LOG_EXCLUDE_DOMAIN:
-            return None
+        if domain in EXCLUDE_DOMAIN:
+            return None, None
         return fqdn, domain
 
 
@@ -161,7 +161,7 @@ class URILogger(object):
         for tag in tags:
             taguris = suspect.get_tag(tag, [])
             for uri in taguris:
-                if uri not in uris and is_url_tldcheck(uri, exclude_fqdn=LOG_EXCLUDE_FQDN, exclude_domain=LOG_EXCLUDE_DOMAIN):
+                if uri not in uris and is_url_tldcheck(uri, exclude_fqdn=EXCLUDE_FQDN, exclude_domain=EXCLUDE_DOMAIN):
                     uris[uri] = tag
     
         uris = self._remove_uris_in_rcpt_domain(uris, suspect.to_domain, suspect.id)
@@ -222,7 +222,8 @@ class AttLogger(object):
     def _skip_attachment_type(self, att_type):
         SKIPS = ['text/xml', 'text/json']
         for item in SKIPS:
-            if att_type == item or att_type.startswith(item):
+            # att_type might be None
+            if att_type and (att_type == item or att_type.startswith(item)):
                 return True
         return False
     
@@ -281,27 +282,30 @@ class ElasticBackend(object):
         self.elastic_connection = {}
     
     
-    def get_elastic_connection(self):
+    def get_elastic_connection(self, reload=False) -> ElasticClient:
         es = None
         elastic_uris = self.config.getlist(self.section,'elastic_uris')
+        verify_certs = self.config.getboolean(self.section, 'elastic_verify_certs')
         if elastic_uris:
             elastic_uri_string = ','.join(elastic_uris)
             try:
+                if reload:
+                    del self.elastic_connection[elastic_uri_string]
                 es = self.elastic_connection[elastic_uri_string]
             except KeyError:
-                es = elasticsearch.Elasticsearch(hosts=elastic_uris)
-                self.elastic_connection = {elastic_uri_string:es}
+                es = ElasticClient(hosts=elastic_uris, verify_certs=verify_certs, ssl_show_warn=False)
+                self.elastic_connection[elastic_uri_string] = es
         return es
     
     
-    def get_elastic_index(self, suspect):
+    def get_elastic_index(self, suspect:Suspect) -> str:
         indextmpl = self.config.get(self.section,'elastic_index')
         indexname = apply_template(indextmpl, suspect)
         return indexname
     
 
     @smart_cached_memberfunc(inputs=[])
-    def _set_index_mapping(self, indexname):
+    def _set_index_mapping(self, indexname:str) -> None:
         self.logger.info(f'checking mapping of index {indexname}')
         es = self.get_elastic_connection()
         exists = es.indices.exists(index=indexname)
@@ -316,7 +320,7 @@ class ElasticBackend(object):
             if need_put:
                 try:
                     r = es.indices.put_mapping(body=LOG_MAPPING, index=indexname)
-                except elasticsearch.exceptions.ElasticsearchException as e:
+                except ElasticException as e:
                     r = {'exc': e.__class__.__name__, 'msg':str(e)}
                 if r.get('acknowledged'):
                     self.logger.info(f'put new mapping to elastic index {indexname}')
@@ -328,7 +332,7 @@ class ElasticBackend(object):
             try:
                 log_mapping = {'mappings': LOG_MAPPING}
                 r = es.indices.create(indexname, body=log_mapping)
-            except elasticsearch.exceptions.ElasticsearchException as e:
+            except ElasticException as e:
                 r = {'exc': e.__class__.__name__, 'msg':str(e)}
             if r.get('acknowledged'):
                 self.logger.info(f'created new elastic index {indexname}')
@@ -336,19 +340,27 @@ class ElasticBackend(object):
                 self.logger.info(f'error creating new elastic index {indexname} : {str(r)}')
     
     
-    def log_to_elastic(self, suspect, logdata):
-        logdata['timestamp'] = datetime.datetime.utcfromtimestamp(suspect.timestamp).isoformat()
-        es = self.get_elastic_connection()
-        indexname = self.get_elastic_index(suspect)
-        self._set_index_mapping(indexname)
+    def log_to_elastic(self, suspect:Suspect, logdata:tp.Dict, reload=False, retry:int=3) -> tp.Dict[str, str]:
         result = {}
-        r = es.index(index=indexname, id=suspect.id, body=logdata)
-        for key in ['_id']:
-            try:
-                result[key] = r[key]
-            except KeyError:
-                self.logger.error('%s key %s not found in result %s' % (suspect.id, key, r))
-        self.logger.info('%s indexed in elastic: %s' % (suspect.id, r))
+        try:
+            logdata['timestamp'] = datetime.datetime.utcfromtimestamp(suspect.timestamp).isoformat()
+            es = self.get_elastic_connection(reload)
+            indexname = self.get_elastic_index(suspect)
+            self._set_index_mapping(indexname)
+            r = es.index(index=indexname, id=suspect.id, body=logdata)
+            for key in ['_id']:
+                try:
+                    result[key] = r[key]
+                except KeyError:
+                    self.logger.error('%s key %s not found in result %s' % (suspect.id, key, r))
+            self.logger.info('%s indexed in elastic: %s' % (suspect.id, r))
+        except Exception as e:
+            if retry>0:
+                self.logger.debug(f'{suspect.id} failed to index in elastic, retry={retry} reason={e.__class__.__name__}: {str(e)}')
+                time.sleep(0.2*(4-retry))
+                result = self.log_to_elastic(suspect, logdata, reload=True, retry=retry-1)
+            else:
+                raise
         return result
 
 
@@ -366,24 +378,31 @@ class ElasticLogger(AppenderPlugin):
                 'default': '',
                 'description':'comma separated list of ElasticSearch host definition (hostname, hostname:port, https://user:pass@hostname:port/)',
             },
+            'elastic_verify_certs': {
+                'default': 'True',
+                'description': 'verify server\'s SSL certificates',
+            },
             'elastic_index':{
                 'default': 'fuglulog-${date}',
                 'description':'Name of ElasticSearch index in which document will be stored. Template vars (e.g. ${to_domain} or ${date}) can be used.',
             },
             'log_headers':{
-                'default': 'from:addr,to:addr,reply-to:addr,subject,subject:hash:md5,message-id',
+                'default': 'from:addr,to:pseudo:md5,reply-to:addr,subject,subject:decode,subject:hash:md5,message-id',
                 'description:': """
                     Name of message headers to log and extract.
-                    Address headers support tags such as :addr or :name.
+                    Address headers (e.g. To, From, Reply-To) support tags such as :addr (address only), :pseudo:algo (address only, with hashed localpart), :name (name label only), :namedecode (name label only, decoded).
                     All header support tag hash:algo - algo must be one of hashlibs supported hash algorithms
+                    All headers support tag decode - try decode encoded headers
                     All header names will be prefixed hdr_ in elastic document, - and : will be converted to _"""
             },
             'log_values': {
-                'default': 'id,from_address,from_domain,from_localpart,queue_id,sasl_login,size,timestamp,to_address,to_domain,to_localpart',
-                'description:': 'Name of suspect attribute values to log and extract. Some values will be renamed.'
+                'default': 'id,from_address,from_domain,from_localpart,queue_id,sasl_login,size,timestamp,to_address,to_domain,to_localpart,to_address:pseudo:md5,to_localpart:pseudo:md5',
+                'description:': """
+                    Name of suspect attribute values to log and extract. Some values will be renamed.
+                    Email address like values support :pseudo:algo (hash localpart)"""
             },
             'log_tags':{
-                'default': 'injectqueueid,fuzor.digest,log.scanhost,log.decision,log.clienthelo,log.clientip,log.clienthostname,archived,Attachment.bounce.queueid,spf.result,dkim.result,arc.result,dmarc.result,log.real_from_address',
+                'default': 'injectqueueid,fuzor.digest,log.scanhost,log.decision,log.client_helo,log.client_ip,log.client_hostname,archived,Attachment.bounce.queueid,spf.result,dkim.result,arc.result,dmarc.result,log.real_from_address',
                 'description:': 'Name of message tags to log and extract. Some tags will be renamed.'
             },
             'log_raw_headers': {
@@ -408,7 +427,15 @@ class ElasticLogger(AppenderPlugin):
             },
             'log_attachment_hashes': {
                 'default': 'md5,sha1',
-                'description': 'log attachment checksums, specify any hash allowed by haslib: %s' % ','.join(hashlib.algorithms_available)
+                'description': 'log attachment checksums, specify any hash supported by hashlib: %s' % ','.join(hashlib.algorithms_available)
+            },
+            'log_header_as_env_sender': {
+                'default': 'X-Original-Sender, X-Mail-Args, Return-Path',
+                'description': 'override envelope sender with value from one of these headers (if set - first occurrence wins)'
+            },
+            'log_header_as_env_recipient': {
+                'default': 'X-Original-Recipient',
+                'description': 'override envelope recipient with value from one of these headers (if set - first occurrence wins)'
             },
             'extra_tld_file': {
                 'default':'',
@@ -424,7 +451,7 @@ class ElasticLogger(AppenderPlugin):
         self.elasticbackend = None
         
         
-    def _init_components(self):
+    def _init_components(self) -> None:
         if self.urilogger is None:
             self.urilogger = URILogger(self.config, self.section)
         if self.attlogger is None:
@@ -433,7 +460,7 @@ class ElasticLogger(AppenderPlugin):
             self.elasticbackend = ElasticBackend(self.config, self.section)
     
     
-    def _normalise_fieldname(self, field_name):
+    def _normalise_fieldname(self, field_name:str) -> str:
         if field_name.startswith('log.'):
             field_name = field_name[4:]
         else:
@@ -444,17 +471,51 @@ class ElasticLogger(AppenderPlugin):
         return field_name
     
     
-    def _normalise_header(self, suspect, header_name, header_tag, header_value):
+    def _get_hasher_from_tag(self, header_tag:str) -> tp.Tuple[tp.Union[None, tp.Callable], str]:
+        algo = header_tag.rsplit(':',1)[-1]
+        if algo in hashlib.algorithms_available:
+            hasher = getattr(hashlib, algo)
+        else:
+            hasher = None
+        return hasher, algo
+    
+    
+    def _pseudonymise(self, address:str, header_tag:str) -> str:
+        if address:
+            hasher, algo = self._get_hasher_from_tag(header_tag)
+            if '@' in address:
+                lhs, dom = split_mail(address)
+            else:
+                lhs = address
+                dom = None
+            if lhs is not None:
+                try:
+                    lhs = hasher(force_bString(lhs)).hexdigest()
+                    if dom is not None:
+                        return f'{lhs}@{dom}'
+                    else:
+                        return lhs
+                except TypeError:
+                    self.logger.error('failed to pseudonymise %s with lhs %s' % (address, lhs))
+        return address
+    
+    
+    def _normalise_header(self, suspect:Suspect, header_name:str, header_tag:str, header_value:str) -> str:
         header_name = header_name.lower()
         if header_tag is not None:
             header_tag = header_tag.lower()
             if header_tag.startswith('hash'):
-                _, algo = header_tag.split(':')
-                if algo in hashlib.algorithms_available:
-                    hasher = getattr(hashlib, algo)
-                    header_value = hasher(force_bString(header_value)).hexdigest()
+                hasher, algo = self._get_hasher_from_tag(header_tag)
+                if hasher is not None:
+                    try:
+                        header_value = hasher(force_bString(header_value)).hexdigest()
+                    except TypeError as e:
+                        self.logger.error(f"{suspect.id} hasher({str(hasher)}) problem ({str(e)}) for header tag:{header_tag} and value:{header_value} -> bytes:{force_bString(header_value)}")
+                        header_value = None
                 else:
-                    self.logger.info('%s unsupported hash algorithm %s' % (suspect.id, algo))
+                    self.logger.info('%s unsupported hash algorithm %s in header %s' % (suspect.id, algo, header_name))
+            elif header_tag == 'decode':
+                header_value = suspect.decode_msg_header(header_value)
             elif header_name in ['from', 'to', 'reply-to', 'sender']:
                 try:
                     parsed_header = suspect.parse_from_type_header(header=header_name, validate_mail=True)
@@ -464,10 +525,16 @@ class ElasticLogger(AppenderPlugin):
                             header_value = address
                         elif header_tag == 'name':
                             header_value = display
+                        elif header_tag == 'namedecode':
+                            header_value = suspect.decode_msg_header(display)
+                        elif header_tag.startswith('pseudo:'):
+                            header_value = self._pseudonymise(address, header_tag)
+                        else:
+                            self.logger.warning('%s invalid header tag %s in header %s' % (suspect.id, header_tag, header_name))
                 except Exception as e:
                     self.logger.warning('%s error extracting %s address: %s' % (suspect.id, header_name, str(e)))
                     header_value = None
-        if header_name == 'message_id': # make sure all message ids get logged with leading and traling <> for normalised search
+        if header_name == 'message_id': # make sure all message ids get logged with leading and trailing <> for normalised search
             if not header_value.startswith('<'):
                 header_value = f'<{header_value}'
             if not header_value.endswith('>'):
@@ -476,7 +543,7 @@ class ElasticLogger(AppenderPlugin):
         return header_value
     
     
-    def _get_suspect_header_data(self, suspect):
+    def _get_suspect_header_data(self, suspect:Suspect) -> tp.Dict[str, str]:
         logdata = {}
         msg_header_data = self.config.getlist(self.section, 'log_headers')
         msgrep = suspect.get_message_rep()
@@ -487,27 +554,76 @@ class ElasticLogger(AppenderPlugin):
                 header_tag = None
                 header_name = header_name_tag
             header_value = msgrep.get(header_name)
+            if isinstance(header_value, Header):
+                try:
+                    header_value = str(header_value)
+                except Exception:
+                    header_value = header_value.encode()
             if header_value:
                 header_value = self._normalise_header(suspect, header_name, header_tag, header_value)
-                header_name = self._normalise_fieldname(f'hdr_{header_name_tag}')
-                logdata[header_name] = header_value
+                if header_value:
+                    header_name = self._normalise_fieldname(f'hdr_{header_name_tag}')
+                    logdata[header_name] = header_value
         return logdata
     
     
-    def _get_suspect_fields(self, suspect):
+    def _get_env_override(self, suspect:Suspect, configoption) -> tp.Union[None, str]:
+        env_headers = self.config.getlist(self.section, configoption)
+        if env_headers:
+            msgrep = suspect.get_message_rep()
+            for header in env_headers:
+                value = msgrep.get(header)
+                if value and '@' in value:
+                    return value
+        return None
+    
+    
+    def _get_suspect_attr(self, suspect:Suspect, attr:str):
+        value = None
+        override = True
+        if attr in ['from_address', 'from_domain', 'from_localpart']:
+            value = self._get_env_override(suspect, 'log_header_as_env_sender')
+            if value and attr == 'from_domain':
+                value = domain_from_mail(value)
+            elif value and attr == 'from_localpart':
+                value = split_mail(value)[0]
+        elif attr in ['to_address', 'to_domain', 'to_localpart']:
+            value = self._get_env_override(suspect, 'log_header_as_env_recipient')
+            if value and attr == 'to_domain':
+                value = domain_from_mail(value)
+            elif value and attr == 'to_localpart':
+                value = split_mail(value)[0]
+        else:
+            override = False
+        
+        if not override or value is None:
+            try:
+                value = force_uString(getattr(suspect, attr))
+            except AttributeError:
+                self.logger.debug('%s no suspect attribute %s' % (suspect.id, attr))
+        return value
+    
+    
+    def _get_suspect_fields(self, suspect:Suspect) -> tp.Dict[str, str]:
         logdata = {}
         suspect_fields = self.config.getlist(self.section, 'log_values')
+        tagged_fields = {}
         for field in suspect_fields:
-            try:
-                value = getattr(suspect, field)
-                key = self._normalise_fieldname(FIELDMAP.get(field, field))
-                logdata[key] = force_uString(value)
-            except AttributeError:
-                self.logger.debug('%s no suspect attribute %s' % (suspect.id, field))
+            if ':' in field:
+                real_field, field_tag = field.split(':',1)
+                tagged_fields[real_field] = field_tag
+            else:
+                real_field = field
+                field_tag = None
+            value = self._get_suspect_attr(suspect, real_field)
+            if field_tag and field_tag.startswith('pseudo:') and value:
+                value = self._pseudonymise(value, field_tag)
+            key = self._normalise_fieldname(FIELDMAP.get(field, field))
+            logdata[key] = value
         return logdata
     
     
-    def _get_suspect_funcs(self, suspect):
+    def _get_suspect_funcs(self, suspect:Suspect) -> tp.Dict[str, str]:
         logdata = {}
         status = suspect.get_status()
         for func_name in status:
@@ -516,7 +632,7 @@ class ElasticLogger(AppenderPlugin):
         return logdata
     
     
-    def _get_suspect_tags(self, suspect):
+    def _get_suspect_tags(self, suspect:Suspect) -> tp.Dict[str, str]:
         logdata = {}
         suspect_tags = self.config.getlist(self.section, 'log_tags')
         for tag in suspect_tags:
@@ -527,7 +643,7 @@ class ElasticLogger(AppenderPlugin):
         return logdata
     
     
-    def _get_raw_headers(self, suspect, header_names):
+    def _get_raw_headers(self, suspect:Suspect, header_names:tp.List[str]) -> tp.List[tp.Dict[str, str]]:
         allhdr = header_names[0] == 'all'
         headerdata = []
         hdrline = 0
@@ -549,7 +665,7 @@ class ElasticLogger(AppenderPlugin):
         return headerdata
     
     
-    def _get_virus_reports(self, suspect):
+    def _get_virus_reports(self, suspect:Suspect) -> tp.List[tp.Dict[str, str]]:
         logdata = []
         engines = suspect.get_tag('virus', {})
         for engine in engines.keys():
@@ -561,10 +677,12 @@ class ElasticLogger(AppenderPlugin):
                     logitem['file'] = item
                     logitem['virus'] = virusreport[item]
                     logdata.append(logitem)
+            else:
+                self.logger.debug(f'{suspect.id} no virus report to log for engine {engine}')
         return logdata
     
     
-    def _get_blocked_reports(self, suspect):
+    def _get_blocked_reports(self, suspect:Suspect) -> tp.List[tp.Dict[str, str]]:
         logdata = []
         engines = suspect.get_tag('blocked', {})
         for engine in engines.keys():
@@ -576,27 +694,29 @@ class ElasticLogger(AppenderPlugin):
                     logitem['file'] = item
                     logitem['blockinfo'] = blockreport[item]
                     logdata.append(logitem)
+            else:
+                self.logger.debug(f'{suspect.id} no blocked report to log for engine {engine}')
         return logdata
     
     
-    def _get_spam_reports(self, suspect):
+    def _get_spam_reports(self, suspect:Suspect) -> tp.List[tp.Dict[str, str]]:
         logdata = []
         engines = suspect.get_tag('spam', {})
         for engine in engines.keys():
-            if engines[engine]:
-                logitem = {}
-                logitem['spam'] = suspect.get_tag('spam', {}).get(engine, False)
-                logitem['highspam'] = suspect.get_tag('highspam', {}).get(engine, False)
-                logitem['report'] = force_uString(suspect.get_tag(f'{engine}.report'))
-                for key in ['score', 'skipreason', 'stripped']:
-                    value = suspect.get_tag(f'{engine}.{key}')
-                    if value:
-                        logitem[key] = value
-                logdata.append(logitem)
+            logitem = {}
+            logitem['spam'] = suspect.get_tag('spam', {}).get(engine, False)
+            logitem['highspam'] = suspect.get_tag('highspam', {}).get(engine, False)
+            logitem['report'] = force_uString(suspect.get_tag(f'{engine}.report'))
+            for key in ['score', 'skipreason', 'stripped']:
+                value = suspect.get_tag(f'{engine}.{key}')
+                if value or isinstance(value, (float, int)):
+                    # log for integer/float also if value is 0 (which evaluates to False otherwise...)
+                    logitem[key] = value
+            logdata.append(logitem)
         return logdata
     
     
-    def _get_log_coredata(self, suspect):
+    def _get_log_coredata(self, suspect:Suspect) -> tp.Dict[str, str]:
         logdata = {}
         logdata.update(self._get_suspect_fields(suspect))
         logdata.update(self._get_suspect_funcs(suspect))
@@ -618,7 +738,7 @@ class ElasticLogger(AppenderPlugin):
             
         blockreport = self._get_blocked_reports(suspect)
         if blockreport:
-            logdata['blockedreport'] = virusreport
+            logdata['blockedreport'] = blockreport
         
         maxatt = self.config.getint(self.section, 'log_attachments')
         if maxatt > 0:
@@ -637,9 +757,13 @@ class ElasticLogger(AppenderPlugin):
         return logdata
     
     
-    def _log_to_file(self, suspect, logdata):
+    def _log_to_file(self, suspect:Suspect, logdata:tp.Dict) -> None:
         dirpath = self.config.get(self.section, 'fallback_logdir')
         if dirpath:
+            if not os.path.exists(dirpath):
+                self.logger.error(f'{suspect.id} inexisting log dump dir {dirpath} - logging to /tmp instead')
+                dirpath = '/tmp'
+            
             indexname = self.elasticbackend.get_elastic_index(suspect)
             filename = f'{suspect.id}.json'
             indexpath = os.path.join(dirpath, indexname)
@@ -648,20 +772,20 @@ class ElasticLogger(AppenderPlugin):
                 jsondata = json.dumps(logdata)
                 if not os.path.exists(indexpath):
                     os.mkdir(indexpath)
-                with open(filepath) as f:
+                with open(filepath, 'w') as f:
                     f.write(jsondata)
                 self.logger.info('%s dumped %s bytes of json data to %s' % (suspect.id, len(jsondata), filepath))
             except Exception as e:
                 self.logger.error('%s failed to dump json data to %s due to %s' % (suspect.id, filepath, str(e)))
     
     
-    def _normalise_address(self, address):
+    def _normalise_address(self, address:str) -> str:
         address = strip_batv(address)
         address = decode_srs(address)
         return address
     
     
-    def _add_tags(self, suspect, decision=None):
+    def _add_tags(self, suspect:Suspect, decision:tp.Union[None, str]=None) -> None:
         suspect.set_tag('log.scanhost', get_outgoing_helo(self.config))
         
         if decision is not None:
@@ -674,9 +798,12 @@ class ElasticLogger(AppenderPlugin):
             suspect.set_tag('log.client_ip', clientip)
             suspect.set_tag('log.client_hostname', clienthostname)
         
-        if suspect.from_address:
+        from_address = suspect.from_address
+        if not from_address:
+            from_address = self._get_env_override(suspect, 'log_header_as_env_sender')
+        if from_address:
             try:
-                real_from_address = self._normalise_address(suspect.from_address)
+                real_from_address = self._normalise_address(from_address)
                 real_from_localpart, real_from_domain = split_mail(real_from_address)
                 suspect.set_tag('log.real_from_address', real_from_address)
                 suspect.set_tag('log.real_from_domain', real_from_domain)
@@ -685,7 +812,10 @@ class ElasticLogger(AppenderPlugin):
                 self.logger.warning('%s could not normalise address %s error was %s' % (suspect.id, suspect.from_address, str(e)))
     
     
-    def process(self, suspect, decision):
+    def process(self, suspect:Suspect, decision:str) -> None:
+        if not HAVE_DOMAINMAGIC:
+            return
+        
         self._init_components()
         if self.elasticbackend.get_elastic_connection() is None:
             return
@@ -700,11 +830,12 @@ class ElasticLogger(AppenderPlugin):
     
     
     def lint(self):
-        if not HAVE_ELASTICSEARCH:
-            print('ERROR: elasticsearch library not available')
+        if not HAVE_DOMAINMAGIC:
+            print('ERROR: domainmagic library missing, this plugin will do nothing')
+            return False
+        
+        if not lint_elastic():
             return False
-    
-        print('INFO: Elasticsearch library version: %s' % elasticsearch.__versionstr__)
         
         self._init_components()
         es = self.elasticbackend.get_elastic_connection()
@@ -712,7 +843,7 @@ class ElasticLogger(AppenderPlugin):
             print('WARNING: elastic_uris not defined, this plugin will do nothing')
             return False
         elif not es.ping():
-            print('ERROR: failed to connect to elasticsearch')
+            print('ERROR: failed to connect to elasticsearch, connection info: %s' % str(es))
             return False
         
         dirpath = self.config.get(self.section, 'fallback_logdir')
@@ -732,10 +863,11 @@ class ElasticImport(object):
     def __init__(self, configfile):
         self.config = FuConfigParser()
         if os.path.exists(configfile):
-            self.config.read_file(configfile)
+            with open(configfile) as fp:
+                self.config.read_file(fp)
         else:
             print(f'ERROR: no such config {configfile}')
-        self.section = self.__class__.__name__
+        self.section = 'ElasticLogger'
         self.backend = ElasticBackend(self.config, 'ElasticLogger')
     
     
@@ -756,7 +888,9 @@ class ElasticImport(object):
         if not dirpath:
             print('fallback_logdir not set, nothing to do')
         else:
-            for indexname, _, filenames in os.walk(dirpath):
+            indexnames = [x[1] for x in os.walk(dirpath)][0]
+            for indexname in indexnames:
+                filenames = [x[2] for x in os.walk(os.path.join(dirpath, indexname))][0]
                 for filename in filenames:
                     if filename.endswith('.json'):
                         suspectid = filename.rsplit('.')[0]
@@ -771,10 +905,13 @@ class ElasticImport(object):
                         
     
     def _cleanup_dirs(self, dirpath):
-        for indexname, _, filenames in os.walk(dirpath):
+        indexnames = [x[1] for x in os.walk(dirpath)][0]
+        for indexname in indexnames:
+            abspath = os.path.join(dirpath, indexname)
+            filenames = [x[2] for x in os.walk(abspath)][0]
             if not filenames:
-                abspath = os.path.join(dirpath, indexname)
                 os.rmdir(abspath)
+                print(f'removed empty dir {abspath}')
     
 
 if __name__ == '__main__':
@@ -782,7 +919,7 @@ if __name__ == '__main__':
     import sys
     parser = argparse.ArgumentParser()
     parser.add_argument('--mapping', help='print elastic index mapping and quit', action='store_true')
-    parser.add_argument('--import', help='import json files stored in fallback_logdir. expects path to config file as argument', default='/etc/fuglu/conf.d/logging.conf')
+    parser.add_argument('--import', help='import json files stored in fallback_logdir. expects path to config file as argument. Config file must have a section [ElasticLogger]', default='/etc/fuglu/conf.d/logging.conf')
     parser.add_argument('--keepfiles', help='do not remove files that were imported successfully', action='store_true')
     args = parser.parse_args()
     importconfig = getattr(args, 'import')
