@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-#   Copyright 2009-2021 Oli Schacher, Fumail Project
+#   Copyright 2009-2022 Oli Schacher, Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -23,11 +23,13 @@ May be untested and/or deprecated.
 
 from fuglu.shared import AVScannerPlugin, DUNNO, string_to_actioncode, actioncode_to_string, Suspect, FuConfigParser
 from fuglu.stringencode import force_bString, force_uString
+from fuglu import get_version
 import socket
 import struct
 import re
 import os
 import sys
+import time
 
 
 # from : https://github.com/AlexeyDemidov/avsmtpd/blob/master/drweb.h
@@ -124,26 +126,6 @@ Tags:
         self.enginename = 'drweb'
     
     
-    def examine(self, suspect):
-        if self._check_too_big(suspect):
-            return DUNNO
-        
-        content = suspect.get_message_rep().as_string()
-        
-        for i in range(0, self.config.getint(self.section, 'retries')):
-            try:
-                viruses = self.scan_stream(content)
-                actioncode, message = self._virusreport(suspect, viruses)
-                return actioncode, message
-            except Exception as e:
-                self.logger.warning("Error encountered while contacting drweb (try %s of %s): %s" %
-                                    (i + 1, self.config.getint(self.section, 'retries'), str(e)))
-        self.logger.error("drweb scan failed after %s retries" %
-                          self.config.getint(self.section, 'retries'))
-        
-        return self._problemcode()
-    
-    
     def _parse_result(self, lines):
         dr = {}
         for line in lines:
@@ -374,7 +356,12 @@ Prerequisites: requires an ICAP capable antivirus engine somewhere in your netwo
             
             'service': {
                 'default': 'AVSCAN',
-                'description': 'ICAP Av scan service, usually AVSCAN (sophos, symantec)',
+                'description': 'ICAP Av scan service, usually AVSCAN (sophos, symantec, eset)',
+            },
+            
+            'send_fakeheaders': {
+                'default': 'False',
+                'description': 'send ICAP fake response headers (must be False for eset)',
             },
             
             'enginename': {
@@ -390,24 +377,6 @@ Prerequisites: requires an ICAP capable antivirus engine somewhere in your netwo
         return "ICAP AV"
     
     
-    def examine(self, suspect):
-        if self._check_too_big(suspect):
-            return DUNNO
-        
-        content = suspect.get_source()
-        
-        for i in range(0, self.config.getint(self.section, 'retries')):
-            try:
-                viruses = self.scan_stream(content, suspect.id)
-                actioncode, message = self._virusreport(suspect, viruses)
-                return actioncode, message
-            except Exception as e:
-                self.logger.warning("Error encountered while contacting ICAP server (try %s of %s): %s" % (
-                    i + 1, self.config.getint(self.section, 'retries'), str(e)))
-        self.logger.error("ICAP scan failed after %s retries" % self.config.getint(self.section, 'retries'))
-        
-        return self._problemcode()
-    
     
     def scan_stream(self, content, suspectid='(NA)'):
         """
@@ -423,69 +392,84 @@ Prerequisites: requires an ICAP capable antivirus engine somewhere in your netwo
         s = self.__init_socket__()
         dr = {}
         
+        content = force_uString(content)
         CRLF = "\r\n"
         host = self.config.get(self.section, 'host')
         port = self.config.get(self.section, 'port')
         service = self.config.get(self.section, 'service')
+        send_fakeheaders = self.config.getboolean(self.section, 'send_fakeheaders')
         buflen = len(content)
-        
-        # in theory, these fake headers are optional according to the ICAP errata
-        # and sophos docs
-        # but better be safe than sorry
-        
-        fakerequestheader = "GET http://localhost/message.eml HTTP/1.1" + CRLF
-        fakerequestheader += "Host: localhost" + CRLF
-        fakerequestheader += CRLF
-        fakereqlen = len(fakerequestheader)
-        
-        fakeresponseheader = "HTTP/1.1 200 OK" + CRLF
-        fakeresponseheader += "Content-Type: message/rfc822" + CRLF
-        fakeresponseheader += "Content-Length: " + str(buflen) + CRLF
-        fakeresponseheader += CRLF
-        fakeresplen = len(fakeresponseheader)
-        
+
         bodyparthexlen = hex(buflen)[2:]
         bodypart = bodyparthexlen + CRLF
         bodypart += content + CRLF
         bodypart += "0" + CRLF
         
-        hdrstart = 0
-        responsestart = fakereqlen
-        bodystart = fakereqlen + fakeresplen
+        if send_fakeheaders:
+            fakerequestheader = "GET http://localhost/message.eml HTTP/1.1%s" + CRLF
+            fakerequestheader += "Host: localhost" + CRLF
+            fakerequestheader += CRLF
+            fakereqlen = len(fakerequestheader)
+            
+            fakeresponseheader = "HTTP/1.1 200 OK" + CRLF
+            fakeresponseheader += "Content-Length: " + str(buflen) + CRLF
+            fakeresponseheader += CRLF
+            fakeresplen = len(fakeresponseheader)
+            
+            hdrstart = 0
+            responsestart = fakereqlen
+            bodystart = fakereqlen + fakeresplen
         
         # now that we know the length of the fake request/response, we can
         # build the ICAP header
         icapheader = ""
-        icapheader += "RESPMOD icap://%s:%s/%s ICAP/1.0 %s" % (
-            host, port, service, CRLF)
+        icapheader += "RESPMOD icap://%s:%s/%s ICAP/1.0%s" % (host, port, service, CRLF)
         icapheader += "Host: " + host + CRLF
+        icapheader += "User-Agent: Fuglu/%s%s" % (get_version().strip(), CRLF)
         icapheader += "Allow: 204" + CRLF
-        icapheader += "Encapsulated: req-hdr=%s, res-hdr=%s, res-body=%s%s" % (
-            hdrstart, responsestart, bodystart, CRLF)
+        if send_fakeheaders:
+            icapheader += "Encapsulated: req-hdr=%s, res-hdr=%s, res-body=%s%s" % (hdrstart, responsestart, bodystart, CRLF)
+        else:
+            icapheader += "Encapsulated: res-body=0" + CRLF
         icapheader += CRLF
         
-        everything = icapheader + fakerequestheader + \
-                     fakeresponseheader + bodypart + CRLF
+        if send_fakeheaders:
+            everything = icapheader + fakerequestheader + fakeresponseheader + bodypart + CRLF
+        else:
+            everything = icapheader + bodypart + CRLF
+        #print(f"sending: {everything}")
         s.sendall(force_bString(everything))
-        result = s.recv(20000)
+        result = force_uString(s.recv(20000))
         s.close()
+        #print(f"result is: {result}")
         
-        sheader = b"X-Violations-Found:"
-        if sheader.lower() in result.lower():
-            lines = result.split(b'\n')
-            lineidx = 0
-            for line in lines:
-                if sheader.lower() in line.lower():
-                    numfound = int(line[len(sheader):])
-                    # for each found virus, get 4 lines
-                    for vircount in range(numfound):
-                        infectedfile = lines[
-                            lineidx + vircount * 4 + 1].strip()
-                        infection = lines[lineidx + vircount * 4 + 2].strip()
-                        dr[infectedfile] = infection
-                    
+        sheaders = ['X-Infection-Found', "X-Violations-Found:", ]
+        for sheader in sheaders:
+            if sheader.lower() in result.lower():
+                lines = result.split('\n')
+                lineidx = 0
+                
+                for line in lines:
+                    if sheader.lower() in line.lower():
+                        if sheader == 'X-Infection-Found': # ESET
+                            # X-Infection-Found: Type=0; Resolution=0; Threat=Eicar;
+                            fields = [x.strip().split('=') for x in line.split(';')]
+                            for field in fields:
+                                if len(field)==2 and field[0].lower() == 'threat':
+                                    dr['stream'] = field[1]
+                                
+                        if sheader == 'X-Violations-Found:': # Sophos, Kaspersky?
+                            numfound = int(line[len(sheader):])
+                            # for each found virus, get 4 lines
+                            for vircount in range(numfound):
+                                infectedfile = lines[lineidx + vircount * 4 + 1].strip()
+                                infection = lines[lineidx + vircount * 4 + 2].strip()
+                                dr[infectedfile] = infection
+                        
+                        break
+                    lineidx += 1
+                if dr:
                     break
-                lineidx += 1
         
         if dr == {}:
             return None
@@ -493,41 +477,30 @@ Prerequisites: requires an ICAP capable antivirus engine somewhere in your netwo
             return dr
     
     
-    def __init_socket__(self):
-        unixsocket = False
+    def lint_options(self):
+        CRLF = "\r\n"
+        host = self.config.get(self.section, 'host')
+        port = self.config.get(self.section, 'port')
+        service = self.config.get(self.section, 'service')
         
-        try:
-            self.config.getint(self.section, 'port')
-        except ValueError:
-            unixsocket = True
+        icapheader = ''
+        icapheader += "OPTIONS icap://%s:%s/%s ICAP/1.0%s" % (host, port, service, CRLF)
+        icapheader += "Host: " + host + CRLF
+        icapheader += "User-Agent: Fuglu/%s%s" % (get_version().strip(), CRLF)
         
-        if unixsocket:
-            sock = self.config.get(self.section, 'port')
-            if not os.path.exists(sock):
-                raise Exception("unix socket %s not found" % sock)
-            s = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
-            s.settimeout(self.config.getint(self.section, 'timeout'))
-            try:
-                s.connect(sock)
-            except socket.error:
-                raise Exception(
-                    'Could not reach ICAP server using unix socket %s' % sock)
-        else:
-            host = self.config.get(self.section, 'host')
-            port = self.config.getint(self.section, 'port')
-            timeout = self.config.getfloat(self.section, 'timeout')
-            try:
-                s = socket.create_connection((host, port), timeout)
-            except socket.error:
-                raise Exception('Could not reach ICAP server using network (%s, %s)' % (host, port))
-        
-        return s
+        s = self.__init_socket__()
+        s.sendall(force_bString(icapheader))
+        result = s.recv(20000)
+        s.close()
+        print(force_uString(result))
     
     
     def lint(self):
         viract = self.config.get(self.section, 'virusaction')
         print("Virusaction: %s" % actioncode_to_string(string_to_actioncode(viract, self.config)))
-        allok = self.check_config() and self.lint_eicar()
+        allok = self.check_config()
+        #self.lint_options()
+        allok = allok and self.lint_eicar()
         return allok
 
 
@@ -596,7 +569,7 @@ Tags:
             },
         }
         
-        self.pattern = re.compile(b'^(\d)+ <(.+)> (.+)$')
+        self.pattern = re.compile(rb'^(\d)+ <(.+)> (.+)$')
         self.enginename = 'F-Prot'
     
     
@@ -703,8 +676,7 @@ Tags:
         s = self.__init_socket__()
         content = force_bString(content)
         buflen = len(content)
-        s.sendall(
-            force_bString('SCAN %s STREAM fu_stream SIZE %s' % (self.config.get(self.section, 'scanoptions'), buflen)))
+        s.sendall(force_bString('SCAN %s STREAM fu_stream SIZE %s' % (self.config.get(self.section, 'scanoptions'), buflen)))
         s.sendall(b'\n')
         self.logger.debug('%s Sending buffer (length=%s) to fpscand...' % (suspectid, buflen))
         s.sendall(content)
@@ -745,31 +717,8 @@ Tags:
         (handle, tempfilename) = tempfile.mkstemp(prefix='fuglu', dir=self.config.get('main', 'tempdir'))
         tempfilename = tempfilename
         
-        stream = """Date: Mon, 08 Sep 2008 17:33:54 +0200
-To: oli@unittests.fuglu.org
-From: oli@unittests.fuglu.org
-Subject: test eicar attachment
-X-Mailer: swaks v20061116.0 jetmore.org/john/code/#swaks
-MIME-Version: 1.0
-Content-Type: multipart/mixed; boundary="----=_MIME_BOUNDARY_000_12140"
-
-------=_MIME_BOUNDARY_000_12140
-Content-Type: text/plain
-
-Eicar test
-------=_MIME_BOUNDARY_000_12140
-Content-Type: application/octet-stream
-Content-Transfer-Encoding: BASE64
-Content-Disposition: attachment
-
-UEsDBAoAAAAAAGQ7WyUjS4psRgAAAEYAAAAJAAAAZWljYXIuY29tWDVPIVAlQEFQWzRcUFpYNTQo
-UF4pN0NDKTd9JEVJQ0FSLVNUQU5EQVJELUFOVElWSVJVUy1URVNULUZJTEUhJEgrSCoNClBLAQIU
-AAoAAAAAAGQ7WyUjS4psRgAAAEYAAAAJAAAAAAAAAAEAIAD/gQAAAABlaWNhci5jb21QSwUGAAAA
-AAEAAQA3AAAAbQAAAAAA
-
-------=_MIME_BOUNDARY_000_12140--"""
         with os.fdopen(handle, 'w+b') as fd:
-            fd.write(force_bString(stream))
+            fd.write(force_bString(self.eicar))
         
         try:
             viruses = self.scan_file(tempfilename)
@@ -794,7 +743,159 @@ AAEAAQA3AAAAbQAAAAAA
             print(e)
             return False
         
+
+
+class CyrenPlugin(AVScannerPlugin):
+    """ This plugin passes suspects to a Cyren Antivirus scan daemon (csamd)
+
+Prerequisites: Cyren Antivirus must be installed and running, not necessarily on the same box as fuglu though.
+    """
+    
+    def __init__(self, config, section=None):
+        AVScannerPlugin.__init__(self, config, section)
+        self.requiredvars = {
+            'host': {
+                'default': 'localhost',
+                'description': 'hostname where the ICAP server runs',
+            },
+            
+            'port': {
+                'default': '1344',
+                'description': "tcp port or path to unix socket",
+            },
+            
+            'timeout': {
+                'default': '10',
+                'description': 'socket timeout',
+            },
+            
+            'maxsize': {
+                'default': '22000000',
+                'description': "maximum message size, larger messages will not be scanned. ",
+            },
+            
+            'retries': {
+                'default': '3',
+                'description': 'how often should fuglu retry the connection before giving up',
+            },
+            
+            'virusaction': {
+                'default': 'DEFAULTVIRUSACTION',
+                'description': "action if infection is detected (DUNNO, REJECT, DELETE)",
+            },
+            
+            'problemaction': {
+                'default': 'DEFER',
+                'description': "action if there is a problem (DUNNO, DEFER)",
+            },
+            
+            'rejectmessage': {
+                'default': 'threat detected: ${virusname}',
+                'description': "reject message template if running in pre-queue mode and virusaction=REJECT",
+            },
+            
+            'enginename': {
+                'default': '',
+                'description': 'set custom engine name (defaults to %s)' % self.enginename,
+            },
+        }
+        self.enginename = 'cyren'
+        self.logger = self._logger()
+    
+    
+    def __str__(self):
+        return "Cyren AV"
+    
+    
+    def _parse_result(self, result, suspectid):
+        dr = {}
+        result = force_uString(result.strip())
+        self.logger.debug(f'{suspectid} cyren sez {result}')
+        #daemon_status, scan_status, obj, message = re.split('\s', result, 3)
+        fields = re.split('\s', result)
+        if len(fields) == 3:
+            daemon_status, scan_status, obj = fields
+            message = None
+        else:
+            daemon_status = fields[0]
+            scan_status = fields[1]
+            #obj = fields[2]
+            message = ' '.join(fields[3:])
         
+        if ':' in daemon_status:
+            daemon_status = daemon_status.rsplit(':')[-1]
+        
+        if daemon_status != 'OK' or scan_status == 'ERROR':
+            self.logger.warning(f'{suspectid} scanner errror, got {daemon_status} {scan_status}')
+            return None # or raise exception?
+        if scan_status != 'INFECTED':
+            self.logger.debug(f'{suspectid} message not infected, got {scan_status}')
+            return None
+        
+        dr['stream'] = message
+        return dr
+    
+    
+    def scan_stream(self, content, suspectid='(N/A)'):
+        """
+        Scans given byte buffer (file content). May raise an exception on errors.
+        :param content: file content as string
+        :param suspectid: suspect.id of currently processed suspect
+        :return: None if no virus is found, else a dict filename -> virusname
+        """
+        sock = self.__init_socket__()
+        remainingbytes = force_bString(content)
+        buflen = len(remainingbytes)
+        
+        sock.sendall(force_bString('INSTREAM %s\n' % buflen))
+        reply = sock.recv(20000)
+        if reply != b'OK SEND_DATA\n':
+            self.logger.warning(f'{suspectid} Cyren did not respond OK, got {force_uString(reply.strip())}')
+            return None
+        
+        maxtime = time.time() + self.config.getint(self.section, 'timeout')
+        default_chunk_size = min(buflen, 20480)
+        iChunk = 0
+        while len(remainingbytes) > 0:
+            if time.time() > maxtime:
+                raise TimeoutError('Timeout sending to AV daemon')
+            iChunk = iChunk + 1
+            chunklength = min(default_chunk_size, len(remainingbytes))
+            chunkdata = remainingbytes[:chunklength]
+            remainingbytes = remainingbytes[chunklength:]
+            packsize = struct.pack(b'!I', chunklength)
+            msg = packsize + chunkdata
+            sock.sendall(msg)
+        sock.sendall(struct.pack(b'!I', 0))
+        
+        resp = sock.recv(20000)
+        self._close_socket(sock)
+        dr = self._parse_result(resp, suspectid)
+        return dr
+    
+    
+    def lint_ping(self):
+        try:
+            sock = self.__init_socket__()
+        except Exception as e:
+            print("Could not contact csamd: %s" % (str(e)))
+            return False
+        sock.sendall(force_bString('PING\n'))
+        result = sock.recv(20000)
+        print("Got Pong: %s" % force_uString(result))
+        if result.strip() != b'PONG':
+            print("Invalid PONG: %s" % force_uString(result))
+        self._close_socket(sock)
+        return True
+    
+    
+    def lint(self):
+        allok = self.check_config() and self.lint_ping() and self.lint_eicar()
+        return allok
+
+
+
+
 if __name__ == '__main__':
     if 'drweb' in sys.argv:
-        drweb_main()
\ No newline at end of file
+        drweb_main()
