@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-#   Copyright 2009-2021 Oli Schacher, Fumail Project
+#   Copyright 2009-2022 Oli Schacher, Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,6 +16,7 @@
 #
 from fuglu.shared import ScannerPlugin, DUNNO, SuspectFilter, apply_template, get_outgoing_helo, Suspect
 from fuglu.stringencode import force_uString, force_bString
+from fuglu.extensions.elastic import ElasticClient, lint_elastic
 import os
 import shutil
 import pwd
@@ -30,16 +31,10 @@ from collections import OrderedDict
 try:
     from cassandra.cluster import Cluster
     from cassandra.policies import RoundRobinPolicy
+    from cassandra import __version__ as cassandraversion
     HAVE_CASSANDRA = True
 except ImportError:
     HAVE_CASSANDRA = False
-    
-try:
-    import elasticsearch
-    HAVE_ELASTICSEARCH = True
-except ImportError:
-    HAVE_ELASTICSEARCH = False
-
 
 
 class AbstractBackend(object):
@@ -244,6 +239,10 @@ class ElasticBackend(AbstractBackend):
             'default': '',
             'description':'comma separated list of ElasticSearch host definition (hostname, hostname:port, https://user:pass@hostname:port/)',
         },
+        'elastic_verify_certs': {
+            'default': 'True',
+            'description': 'verify server\'s SSL certificates',
+        },
         'elastic_index':{
             'default': 'fugluquar-${date}',
             'description':'Name of ElasticSearch index in which document will be stored. Template vars (e.g. ${to_domain} or ${date}) can be used.',
@@ -261,16 +260,19 @@ class ElasticBackend(AbstractBackend):
     elastic_connection = {}
 
 
-    def get_elastic_connection(self):
+    def get_elastic_connection(self, reload=False):
         es = None
         elastic_uris = self.config.getlist(self.section, 'elastic_uris')
+        verify_certs = self.config.getboolean(self.section, 'elastic_verify_certs')
         if elastic_uris:
             elastic_uri_string = ','.join(elastic_uris)
             try:
+                if reload:
+                    del self.elastic_connection[elastic_uri_string]
                 es = self.elastic_connection[elastic_uri_string]
             except KeyError:
-                es = elasticsearch.Elasticsearch(hosts=elastic_uris)
-                self.elastic_connection = {elastic_uri_string: es}
+                es = ElasticClient(hosts=elastic_uris, verify_certs=verify_certs, ssl_show_warn=False)
+                self.elastic_connection[elastic_uri_string] = es
         return es
     
     
@@ -280,43 +282,53 @@ class ElasticBackend(AbstractBackend):
         return indexname
     
     
-    def archive(self, suspect, fugluid_override=None):
-        original = self.config.getboolean(self.section, 'elastic_useoriginal')
-        content = self._get_msg_source(suspect, original)
-        doc = {
-            'fugluid': fugluid_override or suspect.id,
-            'fugluid_original': fugluid_override or suspect.id,
-            'content': force_uString(content),
-            'timestamp': datetime.datetime.utcfromtimestamp(suspect.timestamp),
-        }
-        
-        extrafields = self.config.getlist(self.section, 'elastic_extrafields')
-        for field in extrafields:
-            doc[field] = apply_template('${%s}' % field, suspect)
-        
-        es = self.get_elastic_connection()
-        indexname = self._get_elastic_index(suspect)
+    def archive(self, suspect, fugluid_override=None, reload=False, retry=3):
         result = {}
-        r = es.index(index=indexname, id=suspect.id, body=doc)
-        for key in ['_id']:
-            try:
-                result[key] = r[key]
-            except KeyError:
-                self.logger.error('%s key %s not found in result %s' % (suspect.id, key, r))
-        self.logger.info('%s indexed in elastic: %s' % (suspect.id, r))
+        try:
+            original = self.config.getboolean(self.section, 'elastic_useoriginal')
+            content = self._get_msg_source(suspect, original)
+            doc = {
+                'fugluid': fugluid_override or suspect.id,
+                'fugluid_original': suspect.id,
+                'content': force_uString(content),
+                'timestamp': datetime.datetime.utcfromtimestamp(suspect.timestamp),
+                'to_address': suspect.to_address,
+                'to_domain': suspect.to_domain,
+            }
+            
+            extrafields = self.config.getlist(self.section, 'elastic_extrafields')
+            for field in extrafields:
+                doc[field] = apply_template('${%s}' % field, suspect)
+            
+            es = self.get_elastic_connection(reload)
+            indexname = self._get_elastic_index(suspect)
+            r = es.index(index=indexname, id=suspect.id, body=doc)
+            for key in ['_id']:
+                try:
+                    result[key] = r[key]
+                except KeyError:
+                    self.logger.error('%s key %s not found in result %s' % (suspect.id, key, r))
+            self.logger.info('%s indexed in elastic: %s' % (suspect.id, r))
+        except Exception as e:
+            if retry>0:
+                self.logger.debug(f'{suspect.id} failed to index in elastic, retry={retry} reason={e.__class__.__name__}: {str(e)}')
+                time.sleep(0.2*(4-retry))
+                result = self.archive(suspect, fugluid_override, reload=True, retry=retry-1)
+            else:
+                raise
         return result
     
     
     def lint(self):
-        if not HAVE_ELASTICSEARCH:
-            print('ERROR: elasticsearch library not available')
+        if not lint_elastic():
             return False
         
-        print('INFO: Elasticsearch library version: %s' % elasticsearch.__versionstr__)
-        
         es = self.get_elastic_connection()
+        if es is None:
+            print('WARNING: elastic_uris not defined, this plugin will do nothing')
+            return False
         if not es.ping():
-            print('ERROR: failed to connect to elasticsearch')
+            print('ERROR: failed to connect to elasticsearch, connection info: %s' % str(es))
             return False
         
         return True
@@ -399,8 +411,7 @@ class CassandraBackend(AbstractBackend):
             print('ERROR: cassandra driver not available')
             return False
         
-        import cassandra
-        print("INFO: Cassandra Driver Version: %s" % cassandra.__version__)
+        print("INFO: Cassandra Driver Version: %s" % cassandraversion)
         
         try:
             self.get_cassandra_session().execute("SELECT * FROM quarantine WHERE fugluid='dummy'")
@@ -660,8 +671,8 @@ ARCHIVE_BACKENDS['cassandra'] = CassandraBackend
 
 
 class ArchivePlugin(ScannerPlugin):
-
-    """This plugins stores a copy of the message if it matches certain criteria (Suspect Filter). 
+    r"""
+This plugins stores a copy of the message if it matches certain criteria (Suspect Filter).
 You can use this if you want message archives for your domains, need a quarantine or to debug problems occuring only for certain recipients.
 The architecture allows to store data in various backends (databases), either simultaneously or using one backend as main and others as fallback in case the main backend is unavailable.
 Currently the following backends are supported:
@@ -768,7 +779,7 @@ Note: The first rule to match in a message is the only rule that will be applied
                 errors['preparation'] = 'could not find previous fugluid'
                 return result, errors
             elif suspect.check_id(fugluid_override):
-                errors['preparation'] = 'previous fugluid is not an md5 hash: %s' % fugluid_override
+                errors['preparation'] = 'previous fugluid is not valid: %s' % fugluid_override
                 return result, errors
         
         self._load_backends()
@@ -823,8 +834,11 @@ Note: The first rule to match in a message is the only rule that will be applied
                     result, errors = self.do_archive(suspect)
                     if result:
                         suspect.set_tag('archived', True)
+                        self.logger.debug('%s archived successfully: %s' % (suspect.id, result))
                     if errors:
                         self.logger.error('%s archiving errors in backend %s' % (suspect.id, ','.join(list(errors.keys()))))
+                        for key in errors:
+                            self.logger.debug('%s archiving errors in backend %s: %s' % (suspect.id, key, errors[key]))
                     suspect.set_tag('archived.result', result)
                     suspect.set_tag('archived.errors', errors)
                     
