@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-#   Copyright 2009-2021 Oli Schacher, Fumail Project
+#   Copyright 2009-2022 Oli Schacher, Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,6 +14,8 @@
 # limitations under the License.
 #
 #
+import traceback
+
 from fuglu.shared import ScannerPlugin, DELETE, DUNNO, string_to_actioncode, FileList, get_outgoing_helo
 from fuglu.bounce import Bounce
 from fuglu.stringencode import force_uString, force_bString
@@ -88,8 +90,7 @@ class RulesCache(object):
         if not hasattr(self, 'lock'):
             self.lock = Lock()
         if not hasattr(self, 'logger'):
-            self.logger = logging.getLogger(
-                'fuglu.plugin.FiletypePlugin.RulesCache')
+            self.logger = logging.getLogger('fuglu.plugin.FiletypePlugin.RulesCache')
         if not hasattr(self, 'lastreload'):
             self.lastreload = 0
         self.rulesdir = rulesdir
@@ -102,8 +103,7 @@ class RulesCache(object):
             self.logger.error('Invalid rule type requested: %s' % ruletype)
             return None
         if key not in self.rules[ruletype]:
-            self.logger.debug(
-                'Ruleset not found : [%s] [%s]' % (ruletype, key))
+            self.logger.debug('Ruleset not found : [%s] [%s]' % (ruletype, key))
             return None
         self.logger.debug('Ruleset found : [%s] [%s] ' % (ruletype, key))
 
@@ -233,8 +233,7 @@ class RulesCache(object):
                 continue
             tpl = line.split(None, 2)
             if len(tpl) != 3:
-                self.logger.debug(
-                    'Ignoring invalid line  (length %s): %s' % (len(tpl), line))
+                self.logger.debug('Ignoring invalid line  (length %s): %s' % (len(tpl), line))
                 continue
             (action, regex, description) = tpl
             action = action.lower()
@@ -249,7 +248,7 @@ class RulesCache(object):
 
 class FiletypePlugin(ScannerPlugin):
 
-    """This plugin checks message attachments. You can configure what filetypes or filenames are allowed to pass through fuglu. If a attachment is not allowed, the message is deleted and the sender receives a bounce error message. The plugin uses the '''file''' library to identify attachments, so even if a smart sender renames his executable to .txt, fuglu will detect it.
+    r"""This plugin checks message attachments. You can configure what filetypes or filenames are allowed to pass through fuglu. If a attachment is not allowed, the message is deleted and the sender receives a bounce error message. The plugin uses the '''file''' library to identify attachments, so even if a smart sender renames his executable to .txt, fuglu will detect it.
 
 Attachment rules can be defined globally, per domain or per user.
 
@@ -393,6 +392,31 @@ The other common template variables are available as well.
                 'default': 'SELECT action,regex,description FROM attachmentrules WHERE scope=:scope AND checktype=:checktype ORDER BY prio',
                 'description': "sql query to load rules from a db. #:scope will be replaced by the recipient address first, then by the recipient domain\n:check will be replaced 'filename','contenttype','archive-filename' or 'archive-contenttype'",
             },
+            
+            'config_section_filename': {
+                'default': '',
+                'description': 'load additional rules from filter config overrides databases (e.g. from fuglu.conf, yaml, rest api or sql backend)',
+            },
+    
+            'config_section_filetype': {
+                'default': '',
+                'description': 'load additional rules from filter config overrides databases (e.g. from fuglu.conf, yaml, rest api or sql backend)',
+            },
+    
+            'config_section_archivename': {
+                'default': '',
+                'description': 'load additional rules from filter config overrides databases (e.g. from fuglu.conf, yaml, rest api or sql backend)',
+            },
+    
+            'config_section_archivecryptoname': {
+                'default': '',
+                'description': 'load additional rules from filter config overrides databases (e.g. from fuglu.conf, yaml, rest api or sql backend)',
+            },
+    
+            'config_section_archivetype': {
+                'default': '',
+                'description': 'load additional rules from filter config overrides databases (e.g. from fuglu.conf, yaml, rest api or sql backend)',
+            },
 
             'checkarchivenames': {
                 'default': 'False',
@@ -428,6 +452,7 @@ The other common template variables are available as well.
         self.sendbounce = True
         self.checkarchivenames = False
         self.checkarchivecontent = False
+        self.runtimeconfig = None
 
 
         # copy dict with available extensions from Archivehandle
@@ -440,11 +465,9 @@ The other common template variables are available as well.
 
     def examine(self, suspect):
         if self.rulescache is None:
-            self.rulescache = RulesCache(
-                self.config.get(self.section, 'rulesdir'))
+            self.rulescache = RulesCache(self.config.get(self.section, 'rulesdir'))
 
-        self.blockedfiletemplate = self.config.get(
-            self.section, 'template_blockedfile')
+        self.blockedfiletemplate = self.config.get(self.section, 'template_blockedfile')
 
         runtimeconfig = DBConfig(self.config, suspect)
         self.checkarchivenames = runtimeconfig.getboolean(self.section, 'checkarchivenames')
@@ -484,8 +507,7 @@ The other common template variables are available as well.
         attachmentname = self.asciionly(attachmentname)
 
         if obj is None:
-            self.logger.warning(
-                "%s: message has unknown name or content-type attachment %s" % (suspect.id, attachmentname))
+            self.logger.warning("%s: message has unknown name or content-type attachment %s" % (suspect.id, attachmentname))
             return ATTACHMENT_DUNNO
 
         # remove non ascii chars
@@ -546,8 +568,7 @@ The other common template variables are available as well.
                     return ATTACHMENT_BLOCK
 
                 if action == 'delete':
-                    self.logger.info(
-                        '%s contains blocked attachment %s %s -- SILENT DELETE! --' % (suspect.id, displayname, asciirep))
+                    self.logger.info('%s contains blocked attachment %s %s -- SILENT DELETE! --' % (suspect.id, displayname, asciirep))
                     return ATTACHMENT_SILENTDELETE
 
                 if action == 'allow':
@@ -557,8 +578,7 @@ The other common template variables are available as well.
     
     def matchMultipleSets(self, setlist, obj, suspect, attachmentname=None):
         """run through multiple sets and return the first action which matches obj"""
-        self.logger.debug(
-            '%s Checking object %s against attachment rulesets' % (suspect.id, obj))
+        self.logger.debug('%s Checking object %s against attachment rulesets' % (suspect.id, obj))
         for ruleset in setlist:
             res = self.matchRules(ruleset, obj, suspect, attachmentname)
             if res != ATTACHMENT_DUNNO:
@@ -566,77 +586,166 @@ The other common template variables are available as well.
         return ATTACHMENT_DUNNO
     
     
+    def _load_rules(self, suspect):
+        user_names = []
+        user_ctypes = []
+        user_archive_names = []
+        user_archive_crypto_names = []
+        user_archive_ctypes = []
+        domain_names = []
+        domain_ctypes = []
+        domain_archive_names = []
+        domain_archive_crypto_names = []
+        domain_archive_ctypes = []
+        
+        for func in [self._load_rules_db, self._load_rules_conf, self._load_rules_file]:
+            data = func(suspect)
+            if data is None:
+                continue
+            
+            f_user_names, f_user_ctypes, f_user_archive_names, f_user_archive_crypto_names, f_user_archive_ctypes, \
+            f_domain_names, f_domain_ctypes, f_domain_archive_names, f_domain_archive_crypto_names, f_domain_archive_ctypes \
+                = data
+            if f_user_names is not None:
+                user_names.extend(f_user_names)
+            if f_user_ctypes is not None:
+                user_ctypes.extend(f_user_ctypes)
+            if f_user_archive_names is not None:
+                user_archive_names.extend(f_user_archive_names)
+            if f_user_archive_crypto_names is not None:
+                user_archive_crypto_names.extend(f_user_archive_crypto_names)
+            if f_user_archive_ctypes is not None:
+                user_archive_ctypes.extend(f_user_archive_ctypes)
+            if f_domain_names is not None:
+                domain_names.extend(f_domain_names)
+            if f_domain_ctypes is not None:
+                domain_ctypes.extend(f_domain_ctypes)
+            if f_domain_archive_names is not None:
+                domain_archive_names.extend(f_domain_archive_names)
+            if f_domain_archive_crypto_names is not None:
+                domain_archive_crypto_names.extend(f_domain_archive_crypto_names)
+            if f_domain_archive_ctypes is not None:
+                domain_archive_ctypes.extend(f_domain_archive_ctypes)
+        
+        return user_names, user_ctypes, user_archive_names, user_archive_crypto_names, user_archive_ctypes, \
+            domain_names, domain_ctypes, domain_archive_names, domain_archive_crypto_names, domain_archive_ctypes
+        
+        
+    def _load_rules_file(self, suspect):
+        self.logger.debug('%s Loading attachment rules from filesystem dir %s' % (suspect.id, self.config.get(self.section,'rulesdir')))
+        user_names = self.rulescache.getNAMERules(suspect.to_address)
+        user_ctypes = self.rulescache.getCTYPERules(suspect.to_address)
+        user_archive_names = self.rulescache.getARCHIVENAMERules(suspect.to_address)
+        user_archive_crypto_names = self.rulescache.getEcryptedARCHIVENAMERules(suspect.to_address)
+        user_archive_ctypes = self.rulescache.getARCHIVECTYPERules(suspect.to_address)
+
+        domain_names = self.rulescache.getNAMERules(suspect.to_domain)
+        domain_ctypes = self.rulescache.getCTYPERules(suspect.to_domain)
+        domain_archive_names = self.rulescache.getARCHIVENAMERules(suspect.to_domain)
+        domain_archive_crypto_names = self.rulescache.getEcryptedARCHIVENAMERules(suspect.to_domain)
+        domain_archive_ctypes = self.rulescache.getARCHIVECTYPERules(suspect.to_domain)
+        
+        return user_names, user_ctypes, user_archive_names, user_archive_crypto_names, user_archive_ctypes, \
+               domain_names, domain_ctypes, domain_archive_names, domain_archive_crypto_names, domain_archive_ctypes
+    
+    
+    def _load_rules_db(self, suspect):
+        dbconn = ''
+        if self.config.has_option(self.section, 'dbconnectstring'):
+            dbconn = self.config.get(self.section, 'dbconnectstring')
+    
+        if dbconn.strip() == '':
+            return None
+            
+        self.logger.debug('%s Loading attachment rules from database' % suspect.id)
+        query = self.config.get(self.section, 'query')
+        dbfile = DBFile(dbconn, query)
+        user_names = self.rulescache.get_rules_from_config_lines(
+            dbfile.getContent({'scope': suspect.to_address, 'checktype': FUATT_CHECKTYPE_FN}))
+        user_ctypes = self.rulescache.get_rules_from_config_lines(
+            dbfile.getContent({'scope': suspect.to_address, 'checktype': FUATT_CHECKTYPE_CT}))
+        user_archive_names = self.rulescache.get_rules_from_config_lines(
+            dbfile.getContent({'scope': suspect.to_address, 'checktype': FUATT_CHECKTYPE_ARCHIVE_FN}))
+        user_archive_crypto_names = self.rulescache.get_rules_from_config_lines(
+            dbfile.getContent({'scope': suspect.to_address, 'checktype': FUATT_CHECKTYPE_ARCHIVE_CRYPTO_FN}))
+        user_archive_ctypes = self.rulescache.get_rules_from_config_lines(
+            dbfile.getContent({'scope': suspect.to_address, 'checktype': FUATT_CHECKTYPE_ARCHIVE_CT}))
+        self.logger.debug(
+            '%s Found %s filename rules, %s content-type rules, %s archive filename rules, %s archive crypto rules, %s archive content rules for address %s' %
+            (suspect.id, len(user_names), len(user_ctypes), len(user_archive_names), len(user_archive_crypto_names), len(user_archive_ctypes),
+             suspect.to_address))
+    
+        domain_names = self.rulescache.get_rules_from_config_lines(
+            dbfile.getContent({'scope': suspect.to_domain, 'checktype': FUATT_CHECKTYPE_FN}))
+        domain_ctypes = self.rulescache.get_rules_from_config_lines(
+            dbfile.getContent({'scope': suspect.to_domain, 'checktype': FUATT_CHECKTYPE_CT}))
+        domain_archive_names = self.rulescache.get_rules_from_config_lines(
+            dbfile.getContent({'scope': suspect.to_domain, 'checktype': FUATT_CHECKTYPE_ARCHIVE_FN}))
+        domain_archive_crypto_names = self.rulescache.get_rules_from_config_lines(
+            dbfile.getContent({'scope': suspect.to_domain, 'checktype': FUATT_CHECKTYPE_ARCHIVE_CRYPTO_FN}))
+        domain_archive_ctypes = self.rulescache.get_rules_from_config_lines(
+            dbfile.getContent({'scope': suspect.to_domain, 'checktype': FUATT_CHECKTYPE_ARCHIVE_CT}))
+        self.logger.debug(
+            '%s Found %s filename rules, %s content-type rules, %s archive filename rules, %s archive crypto rules, %s archive content rules for domain %s' %
+            (suspect.id, len(domain_names), len(domain_ctypes), len(domain_archive_names), len(domain_archive_crypto_names),
+             len(domain_archive_ctypes), suspect.to_domain))
+
+        return user_names, user_ctypes, user_archive_names, user_archive_crypto_names, user_archive_ctypes, \
+               domain_names, domain_ctypes, domain_archive_names, domain_archive_crypto_names, domain_archive_ctypes
+    
+    
+    def _load_rules_conf(self, suspect):
+        user_names = []
+        user_ctypes = []
+        user_archive_names = []
+        user_archive_crypto_names = []
+        user_archive_ctypes = []
+        domain_names = []
+        domain_ctypes = []
+        domain_archive_names = []
+        domain_archive_crypto_names = []
+        domain_archive_ctypes = []
+        sections = {
+            'config_section_filename': user_names,
+            'config_section_filetype': user_ctypes,
+            'config_section_archivename': user_archive_names,
+            'config_section_archivecryptoname': user_archive_crypto_names,
+            'config_section_archivetype': user_archive_ctypes,
+        }
+        
+        if self.runtimeconfig is None or self.runtimeconfig.suspect != suspect:
+            self.runtimeconfig = DBConfig(self.config, suspect)
+        
+        for section in sections:
+            self.runtimeconfig.load_section(section)
+            options = self.runtimeconfig.get_cached_options(section)
+            for regex in options:
+                value = self.runtimeconfig.get(section, regex)
+                action, description = value.split(None, 1)
+                tp = (action, regex, description)
+                sections[section].append(tp)
+                
+        return user_names, user_ctypes, user_archive_names, user_archive_crypto_names, user_archive_ctypes, \
+               domain_names, domain_ctypes, domain_archive_names, domain_archive_crypto_names, domain_archive_ctypes
+
+
     def walk(self, suspect):
         """walks through a message and checks each attachment according to the rulefile specified in the config"""
 
         blockaction = self.config.get(self.section, 'blockaction')
         blockactioncode = string_to_actioncode(blockaction)
 
-        # try db rules first
-        self.rulescache.reloadifnecessary()
-        dbconn = ''
-        if self.config.has_option(self.section, 'dbconnectstring'):
-            dbconn = self.config.get(self.section, 'dbconnectstring')
-
-        if dbconn.strip() != '':
-            self.logger.debug('%s Loading attachment rules from database' % suspect.id)
-            query = self.config.get(self.section, 'query')
-            dbfile = DBFile(dbconn, query)
-            user_names = self.rulescache.get_rules_from_config_lines(
-                dbfile.getContent({'scope': suspect.to_address, 'checktype': FUATT_CHECKTYPE_FN}))
-            user_ctypes = self.rulescache.get_rules_from_config_lines(
-                dbfile.getContent({'scope': suspect.to_address, 'checktype': FUATT_CHECKTYPE_CT}))
-            user_archive_names = self.rulescache.get_rules_from_config_lines(
-                dbfile.getContent({'scope': suspect.to_address, 'checktype': FUATT_CHECKTYPE_ARCHIVE_FN}))
-            user_archive_crypto_names = self.rulescache.get_rules_from_config_lines(
-                dbfile.getContent({'scope': suspect.to_address, 'checktype': FUATT_CHECKTYPE_ARCHIVE_CRYPTO_FN}))
-            user_archive_ctypes = self.rulescache.get_rules_from_config_lines(
-                dbfile.getContent({'scope': suspect.to_address, 'checktype': FUATT_CHECKTYPE_ARCHIVE_CT}))
-            self.logger.debug('%s Found %s filename rules, %s content-type rules, %s archive filename rules, %s archive content rules for address %s' %
-                              (suspect.id, len(user_names), len(user_ctypes), len(user_archive_names), len(user_archive_ctypes), suspect.to_address))
-
-            domain_names = self.rulescache.get_rules_from_config_lines(
-                dbfile.getContent({'scope': suspect.to_domain, 'checktype': FUATT_CHECKTYPE_FN}))
-            domain_ctypes = self.rulescache.get_rules_from_config_lines(
-                dbfile.getContent({'scope': suspect.to_domain, 'checktype': FUATT_CHECKTYPE_CT}))
-            domain_archive_names = self.rulescache.get_rules_from_config_lines(
-                dbfile.getContent({'scope': suspect.to_domain, 'checktype': FUATT_CHECKTYPE_ARCHIVE_FN}))
-            domain_archive_crypto_names = self.rulescache.get_rules_from_config_lines(
-                dbfile.getContent({'scope': suspect.to_domain, 'checktype': FUATT_CHECKTYPE_ARCHIVE_CRYPTO_FN}))
-            domain_archive_ctypes = self.rulescache.get_rules_from_config_lines(
-                dbfile.getContent({'scope': suspect.to_domain, 'checktype': FUATT_CHECKTYPE_ARCHIVE_CT}))
-            self.logger.debug('%s Found %s filename rules, %s content-type rules, %s archive filename rules, %s archive content rules for domain %s' %
-                              (suspect.id, len(domain_names), len(domain_ctypes), len(domain_archive_names), len(domain_archive_ctypes), suspect.to_domain))
-        else:
-            self.logger.debug('%s Loading attachment rules from filesystem dir %s' % (suspect.id, self.config.get(self.section,'rulesdir')))
-            user_names = self.rulescache.getNAMERules(suspect.to_address)
-            user_ctypes = self.rulescache.getCTYPERules(suspect.to_address)
-            user_archive_names = self.rulescache.getARCHIVENAMERules(
-                suspect.to_address)
-            user_archive_crypto_names = self.rulescache.getEcryptedARCHIVENAMERules(
-                suspect.to_address)
-            user_archive_ctypes = self.rulescache.getARCHIVECTYPERules(
-                suspect.to_address)
-
-            domain_names = self.rulescache.getNAMERules(suspect.to_domain)
-            domain_ctypes = self.rulescache.getCTYPERules(suspect.to_domain)
-            domain_archive_names = self.rulescache.getARCHIVENAMERules(
-                suspect.to_domain)
-            domain_archive_crypto_names = self.rulescache.getEcryptedARCHIVENAMERules(
-                suspect.to_domain)
-            domain_archive_ctypes = self.rulescache.getARCHIVECTYPERules(
-                suspect.to_domain)
-
+        user_names, user_ctypes, user_archive_names, user_archive_crypto_names, user_archive_ctypes, \
+        domain_names, domain_ctypes, domain_archive_names, domain_archive_crypto_names, domain_archive_ctypes \
+            = self._load_rules(suspect)
+        
         # always get defaults from file
         default_names = self.rulescache.getNAMERules(FUATT_DEFAULT)
         default_ctypes = self.rulescache.getCTYPERules(FUATT_DEFAULT)
-        default_archive_names = self.rulescache.getARCHIVENAMERules(
-            FUATT_DEFAULT)
-        default_archive_crypto_names = self.rulescache.getEcryptedARCHIVENAMERules(
-            FUATT_DEFAULT)
-        default_archive_ctypes = self.rulescache.getARCHIVECTYPERules(
-            FUATT_DEFAULT)
-
+        default_archive_names = self.rulescache.getARCHIVENAMERules(FUATT_DEFAULT)
+        default_archive_crypto_names = self.rulescache.getEcryptedARCHIVENAMERules(FUATT_DEFAULT)
+        default_archive_ctypes = self.rulescache.getARCHIVECTYPERules(FUATT_DEFAULT)
+        
         # get mail attachment objects (only directly attached objects)
         for attObj in suspect.att_mgr.get_objectlist():
             contenttype_mime = attObj.contenttype_mime
@@ -655,42 +764,33 @@ The other common template variables are available as well.
 
             att_name = self.asciionly(att_name)
 
-            res = self.matchMultipleSets(
-                [user_names, domain_names, default_names], att_name, suspect, att_name)
+            res = self.matchMultipleSets([user_names, domain_names, default_names], att_name, suspect, att_name)
             if res == ATTACHMENT_SILENTDELETE:
-                self._debuginfo(
-                    suspect, "Attachment name=%s SILENT DELETE : blocked by name" % att_name)
+                self._debuginfo(suspect, "Attachment name=%s SILENT DELETE : blocked by name" % att_name)
                 return DELETE
             if res == ATTACHMENT_BLOCK:
-                self._debuginfo(
-                    suspect, "Attachment name=%s : blocked by name)" % att_name)
+                self._debuginfo(suspect, "Attachment name=%s : blocked by name)" % att_name)
                 message = suspect.tags['FiletypePlugin.errormessage']
                 return blockactioncode, message
 
             # go through content type rules
-            res = self.matchMultipleSets(
-                [user_ctypes, domain_ctypes, default_ctypes], contenttype_mime, suspect, att_name)
+            res = self.matchMultipleSets([user_ctypes, domain_ctypes, default_ctypes], contenttype_mime, suspect, att_name)
             if res == ATTACHMENT_SILENTDELETE:
-                self._debuginfo(
-                    suspect, "Attachment name=%s content-type=%s SILENT DELETE: blocked by mime content type (message source)" % (att_name, contenttype_mime))
+                self._debuginfo(suspect, "Attachment name=%s content-type=%s SILENT DELETE: blocked by mime content type (message source)" % (att_name, contenttype_mime))
                 return DELETE
             if res == ATTACHMENT_BLOCK:
-                self._debuginfo(
-                    suspect, "Attachment name=%s content-type=%s : blocked by mime content type (message source)" % (att_name, contenttype_mime))
+                self._debuginfo(suspect, "Attachment name=%s content-type=%s : blocked by mime content type (message source)" % (att_name, contenttype_mime))
                 message = suspect.tags['FiletypePlugin.errormessage']
                 return blockactioncode, message
 
             contenttype_magic = attObj.contenttype
             if contenttype_magic is not None:
-                res = self.matchMultipleSets(
-                    [user_ctypes, domain_ctypes, default_ctypes], contenttype_magic, suspect, att_name)
+                res = self.matchMultipleSets([user_ctypes, domain_ctypes, default_ctypes], contenttype_magic, suspect, att_name)
                 if res == ATTACHMENT_SILENTDELETE:
-                    self._debuginfo(
-                        suspect, "Attachment name=%s content-type=%s SILENT DELETE: blocked by mime content type (magic)" % (att_name, contenttype_magic))
+                    self._debuginfo(suspect, "Attachment name=%s content-type=%s SILENT DELETE: blocked by mime content type (magic)" % (att_name, contenttype_magic))
                     return DELETE
                 if res == ATTACHMENT_BLOCK:
-                    self._debuginfo(
-                        suspect, "Attachment name=%s content-type=%s : blocked by mime content type (magic)" % (att_name, contenttype_magic))
+                    self._debuginfo(suspect, "Attachment name=%s content-type=%s : blocked by mime content type (magic)" % (att_name, contenttype_magic))
                     message = suspect.tags['FiletypePlugin.errormessage']
                     return blockactioncode, message
 
@@ -721,8 +821,6 @@ The other common template variables are available as well.
                         archiveextractlevel = None
                         
                     try:
-
-
                         if self.checkarchivenames:
                             # here, check all the filenames, independent of how many files we would extract
                             # by the limits (att_mgr_default_maxnfiles, att_mgr_hard_maxnfiles)
@@ -734,28 +832,22 @@ The other common template variables are available as well.
                             self.logger.debug(f"{suspect.id} Is {att_name} password protected: {passwordprotected}")
                             ruleset = [user_archive_names, domain_archive_names, default_archive_names]
                             if passwordprotected:
-                                ruleset.extend([user_archive_crypto_names,
-                                                domain_archive_crypto_names,
-                                                default_archive_crypto_names])
+                                ruleset.extend([user_archive_crypto_names, domain_archive_crypto_names, default_archive_crypto_names])
 
                             for name in namelist:
                                 res = self.matchMultipleSets(ruleset, name, suspect, name)
                                 if res == ATTACHMENT_SILENTDELETE:
-                                    self._debuginfo(
-                                        suspect, "Blocked filename in archive %s SILENT DELETE" % att_name)
+                                    self._debuginfo( suspect, "Blocked filename in archive %s SILENT DELETE" % att_name)
                                     return DELETE
                                 if res == ATTACHMENT_BLOCK:
-                                    self._debuginfo(
-                                        suspect, "Blocked filename in archive %s" % att_name)
+                                    self._debuginfo(suspect, "Blocked filename in archive %s" % att_name)
                                     message = suspect.tags['FiletypePlugin.errormessage']
                                     return blockactioncode, message
 
                         if filetype_handler.available() and self.checkarchivecontent:
-
                             maxnfiles2extract = suspect.att_mgr.get_maxfilenum_extract(None)
                             nocheckinfo = NoExtractInfo()
-                            for archObj in attObj.get_objectlist(0, archiveextractlevel, archivecontentmaxsize,
-                                                                 maxnfiles2extract, noextractinfo=nocheckinfo):
+                            for archObj in attObj.get_objectlist(0, archiveextractlevel, archivecontentmaxsize, maxnfiles2extract, noextractinfo=nocheckinfo):
                                 safename = self.asciionly(archObj.filename)
                                 contenttype_magic = archObj.contenttype
 
@@ -786,12 +878,12 @@ The other common template variables are available as well.
 
                             for item in nocheckinfo.get_filtered():
                                 try:
-                                    self._debuginfo(suspect, 'Archive File not checked: reason: %s -> %s'
-                                                    % (item[0], item[1]))
+                                    self._debuginfo(suspect, 'Archive File not checked: reason: %s -> %s' % (item[0], item[1]))
                                 except Exception as e:
                                     self._debuginfo(suspect, 'Archive File not checked: %s' % str(e))
 
                     except Exception as e:
+                        self.logger.debug(traceback.format_exc())
                         self.logger.error("%s archive scanning failed in attachment %s: %s" % (suspect.id, att_name, str(e)))
         return DUNNO
     
@@ -964,7 +1056,7 @@ class FileHashCheck(ScannerPlugin):
     def __init__(self, config, section=None):
         ScannerPlugin.__init__(self, config, section)
         self.redis_pool = None
-        self.logger = logging.getLogger()
+        self.logger = self._logger()
         
         self.requiredvars = {
             'redis_conn': {
@@ -1304,7 +1396,7 @@ class FileHashRedis(object):
 class FileHashFeeder(FileHashCheck):
     def __init__(self,config,section=None):
         FileHashCheck.__init__(self,config,section)
-        self.logger=logging.getLogger()
+        self.logger = self._logger()
         
         requiredvars = {
             'prefix': {
