@@ -1,6 +1,6 @@
 #!/usr/bin/python
 # -*- coding: UTF-8 -*-
-#   Copyright 2012-2021 Fumail Project
+#   Copyright 2012-2022 Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,6 +16,7 @@
 #
 #
 #
+import configparser
 import sys
 #in case the tool is not installed system wide (development...)
 if __name__ =='__main__':
@@ -41,9 +42,9 @@ except ImportError:
 import fuglu.connectors.asyncmilterconnector as asm
 import fuglu.connectors.milterconnector as sm
 from fuglu.mshared import BMPRCPTMixin, BasicMilterPlugin
-from fuglu.shared import _SuspectTemplate, Cache, FuConfigParser
+from fuglu.shared import _SuspectTemplate, Cache, FuConfigParser, FileList, get_outgoing_helo
 from fuglu.stringencode import force_uString
-from fuglu.extensions.sql import SQL_EXTENSION_ENABLED, get_session, DBConfig
+from fuglu.extensions.sql import SQL_EXTENSION_ENABLED, get_session, DBConfig, RESTAPIError
 from fuglu.extensions.dnsquery import lookup, DNSQUERY_EXTENSION_ENABLED, mxlookup
 from fuglu.extensions.redisext import RedisPooledConn, ENABLED as REDIS_ENABLED, REDIS2
 from fuglu.scansession import TrackTimings
@@ -80,9 +81,9 @@ def get_config(fugluconfigfile=None, dconfdir=None):
 
 
 class BackendMixin(object):
-    def __init__(self, config, section):
-        self.config = config
-        self.section = section
+    config=None
+    section=None
+    def __init__(self):
         self.cache = None
         self.config_backend = None
     
@@ -160,6 +161,7 @@ CREATE TABLE `ca_addresscache` (
     """
     def __init__(self,config,section=None):
         super().__init__(config,section)
+        BackendMixin.__init__(self)
         self.logger = self._logger()
         self.relaycache = Cache()
         self.enabletimetracker = False
@@ -196,7 +198,7 @@ CREATE TABLE `ca_addresscache` (
             
             'redis_password': {
                 'default': '',
-                'description': 'password to connect to redis database. leave empty for no password',
+                'description': 'password to connect to redis database. leave empty for no password (deprecated, set password in redis_conn)',
             },
             
             'cleanupinterval':{
@@ -238,6 +240,11 @@ CREATE TABLE `ca_addresscache` (
                 'default': '',
                 'description': 'always accept recipients if they match this regex. leave empty to omit'
             },
+
+            'problemaction': {
+                'default': 'tempfail',
+                'description': "action if there is a problem (DUNNO, DEFER)",
+            },
             
             'enabled': {
                 'section': 'ca_default',
@@ -284,7 +291,7 @@ CREATE TABLE `ca_addresscache` (
             'server': {
                 'section': 'ca_default',
                 'default': 'mx:${domain}',
-                'description': 'how should we retrieve the next hop?',
+                'description': 'how should we retrieve the next hop? define sql, static, mx, tag',
             },
             
             'test_fallback': {
@@ -480,7 +487,7 @@ CREATE TABLE `ca_addresscache` (
         if not HAVE_DOMAINMAGIC:
             print('WARNING: domainmagic is not installed - some functionality will not be available')
         
-        if not self.checkConfig():
+        if not self.check_config():
             return False
 
         if self.config.get('ca_default', 'server').startswith('mx:') and not DNSQUERY_EXTENSION_ENABLED:
@@ -517,11 +524,12 @@ CREATE TABLE `ca_addresscache` (
             #print('Using default config timeout: %ss' % timeout)
         except Exception:
             print('WARNING: Could not get timeout value from config, using internal default of 10s')
-            
+        
         try:
             dbconnection = self.config.get(self.section, 'dbconnection')
-            conn=get_session(dbconnection)
-            conn.execute("SELECT 1")
+            if dbconnection:
+                conn=get_session(dbconnection)
+                conn.execute("SELECT 1")
         except Exception as e:
             print("ERROR: Failed to connect to SQL database: %s" % str(e))
             return False
@@ -616,7 +624,10 @@ CREATE TABLE `ca_addresscache` (
             self.logger.debug(f"{sess.id} Address {to_address} not in database or entry has expired")
         
         #load domain config
-        domainconfig = self.config_backend.get_domain_config_all()
+        try:
+            domainconfig = self.config_backend.get_domain_config_all()
+        except RESTAPIError:
+            return self._problemcode()
         timetracker.tracktime("domainconfig")
 
         if domainconfig is None:
@@ -625,7 +636,7 @@ CREATE TABLE `ca_addresscache` (
             return sm.CONTINUE
         
         #enabled?
-        test = SMTPTest(self.config, self.section, self.relaycache)
+        test = SMTPTest(self.config, self.section, self.relaycache, sess)
         servercachetime = test.get_domain_config_int(domain, 'test_server_interval', domainconfig)
 
         enabled = test.get_domain_config_bool(domain, 'enabled', domainconfig)
@@ -637,7 +648,11 @@ CREATE TABLE `ca_addresscache` (
         
         positive_directory_only = test.get_domain_config_bool(domain, 'positive_directory_only', domainconfig)
         testaddress=test.maketestaddress(domain, domainconfig)
-        testentry = self.cache.get_address(testaddress)
+        try:
+            testentry = self.cache.get_address(testaddress)
+        except Exception as e:
+            self.logger.warning(f'{sess.id} failed to get testaddress {testaddress} due to {e.__class__.__name__}: {str(e)}')
+            testentry = None
         if positive_directory_only and testentry:
             # we require the test user to be present in directory as a safety mesaure
             # if it is not there, db got flushed/corrupted/outdated and user should be accepted
@@ -687,15 +702,22 @@ CREATE TABLE `ca_addresscache` (
             starttime = time.time()
             for relay in test_relays:
                 self.logger.debug(f"{sess.id} testing relay {relay} for domain {domain}")
-                if self.cache.is_skiplisted(domain, relay):
-                    self.logger.info(f'{sess.id} {to_address}: server {relay} for domain {domain} is skiplisted for call-aheads, skipping')
-                    timetracker.report_plugintime(sess.id, str(self))
-                    return sm.CONTINUE
+                try:
+                    if self.cache.is_skiplisted(domain, relay):
+                        self.logger.info(f'{sess.id} {to_address}: server {relay} for domain {domain} is skiplisted for call-aheads, skipping')
+                        timetracker.report_plugintime(sess.id, str(self))
+                        return sm.CONTINUE
+                except Exception as e:
+                    self.logger.error(f'{sess.id} failed to query skiplist for {to_address} due to {e.__class__.__name__}: {str(e)}')
+                    return self._problemcode()
                 
                 #make sure we don't call-ahead ourself
                 if to_address==testaddress:
                     self.logger.error(f"{sess.id} call-ahead loop detected!")
-                    self.cache.skiplist(domain, relay, servercachetime, SMTPTestResult.STAGE_CONNECT, 'call-ahead loop detected')
+                    try:
+                        self.cache.skiplist(domain, relay, servercachetime, SMTPTestResult.STAGE_CONNECT, 'call-ahead loop detected')
+                    except Exception as e:
+                        self.logger.warning(f'{sess.id} failed to add skiplist entry for {domain} due to {e.__class__.__name__}: {str(e)}')
                     timetracker.report_plugintime(sess.id, str(self))
                     return sm.CONTINUE
                 
@@ -740,8 +762,11 @@ CREATE TABLE `ca_addresscache` (
                         servercachetime = min(servercachetime, interval)
             
             if relay is not None:
-                self.logger.warning(f'{sess.id} problem testing recipient verification support on server {relay} : {result.errormessage}. putting on skiplist.')
-                self.cache.skiplist(domain, relay, servercachetime, result.stage, result.errormessage)
+                try:
+                    self.cache.skiplist(domain, relay, servercachetime, result.stage, result.errormessage)
+                    self.logger.warning(f'{sess.id} problem testing recipient verification support on server {relay} : {result.errormessage}. putting on skiplist.')
+                except Exception as e:
+                    self.logger.warning(f'{sess.id} failed to add skiplist entry for {domain} due to {e.__class__.__name__}: {str(e)}')
             timetracker.tracktime("SMTPTestResult.Test_OK-fail")
             timetracker.report_plugintime(sess.id, str(self))
             return action, message
@@ -758,7 +783,10 @@ CREATE TABLE `ca_addresscache` (
                 recverificationsupport=False
             elif addrstate==SMTPTestResult.ADDRESS_DOES_NOT_EXIST:
                 recverificationsupport=True
-            self.cache.put_address(testaddress, servercachetime, not recverificationsupport, msg)
+            try:
+                self.cache.put_address(testaddress, servercachetime, not recverificationsupport, msg)
+            except Exception as e:
+                self.logger.warning(f'{sess.id} failed to add recipient {testaddress} in cache due to {e.__class__.__name__}: {str(e)}')
             timetracker.tracktime("need_server_test")
         else:
             recverificationsupport=True
@@ -777,7 +805,10 @@ CREATE TABLE `ca_addresscache` (
             if addrstate==SMTPTestResult.ADDRESS_TEMPFAIL:
                 self.logger.info(f'{sess.id} server {relay} for domain {domain}: '
                                  f'skiplisting for {servercachetime} seconds (tempfail: {msg})')
-                self.cache.skiplist(domain, relay, servercachetime, result.stage, 'tempfail: %s'%msg)
+                try:
+                    self.cache.skiplist(domain, relay, servercachetime, result.stage, 'tempfail: %s'%msg)
+                except Exception as e:
+                    self.logger.warning(f'{sess.id} failed to add skiplist entry for {domain} due to {e.__class__.__name__}: {str(e)}')
                 timetracker.tracktime("recverificationsupport-tempfail")
                 if test.get_domain_config_bool(domain, 'accept_on_tempfail', domainconfig):
                     timetracker.report_plugintime(sess.id, str(self), end=False)
@@ -787,7 +818,7 @@ CREATE TABLE `ca_addresscache` (
                     return sm.TEMPFAIL, msg
             
             defer_on_hostnotfound = test.get_domain_config_bool(domain, 'defer_on_hostnotfound', domainconfig)
-            if defer_on_hostnotfound and (msg.endswith('has no A records') or msg.endswith('host name could not be resolved')):
+            if defer_on_hostnotfound and msg.endswith(('has no A records', 'host name could not be resolved')):
                 return sm.TEMPFAIL, msg
             
             defer_on_relayaccessdenied = test.get_domain_config_bool(domain, 'defer_on_relayaccessdenied', domainconfig)
@@ -801,7 +832,10 @@ CREATE TABLE `ca_addresscache` (
                 
             if not (relayaccessdenied and defer_on_relayaccessdenied):
                 # don't cache if we defer
-                self.cache.put_address(to_address,cachetime,positive,msg)
+                try:
+                    self.cache.put_address(to_address,cachetime,positive,msg)
+                except Exception as e:
+                    self.logger.warning(f'{sess.id} failed to add recipient {testaddress} in cache due to {e.__class__.__name__}: {str(e)}')
                 neg = ""
                 if not positive:
                     neg = "negative "
@@ -824,11 +858,15 @@ CREATE TABLE `ca_addresscache` (
         else:
             self.logger.info(f'{sess.id} server {relay} for domain {domain}: '
                              f'skiplisting for {servercachetime} seconds ({blreason}) in stage {result.stage}')
-            self.cache.skiplist(domain, relay, servercachetime, result.stage, blreason)
+            try:
+                self.cache.skiplist(domain, relay, servercachetime, result.stage, blreason)
+            except Exception as e:
+                self.logger.warning(f'{sess.id} failed to add skiplist entry for {domain} due to {e.__class__.__name__}: {str(e)}')
             timetracker.tracktime("no-recverificationsupport")
         timetracker.report_plugintime(sess.id, str(self))
         return sm.CONTINUE
-
+    
+    
     def _get_stage_config(self, stage, test, domain, domainconfig, sessid: str = "<>"):
         try:
             interval = test.get_domain_config_int(domain, f'{stage}_fail_interval', domainconfig)
@@ -916,13 +954,57 @@ class SMTPTestResult:
         return desc
 
 
+class TargetList(FileList):
+    def _parse_lines(self, lines):
+        newcontent = {}
+        for line in lines:
+            line = self._apply_linefilters(line)
+            if line is not None:
+                try:
+                    domain, server = line.split()
+                    try:
+                        newcontent[domain].append(server)
+                    except KeyError:
+                        newcontent[domain] = [server]
+                except ValueError as e:
+                    self.logger.warning('invalid domain-target line: %s' % line)
+        return newcontent
+
+
+class TargetListSingleton(object):
+    """
+    Process singleton to store a default TargetList instance
+    """
+
+    instance = None
+    procPID = None
+
+    def __init__(self, *args, **kwargs):
+        pid =  os.getpid()
+        logger = logging.getLogger("%s.%s" % (__package__, self.__class__.__name__))
+        if pid == TargetListSingleton.procPID and TargetListSingleton.instance is not None:
+            logger.debug("Return existing %s Singleton for process with pid: %u" % (self.__class__.__name__, pid))
+        else:
+            if TargetListSingleton.instance is None:
+                logger.info("Create %s for process with pid: %u" % (self.__class__.__name__, pid))
+            elif TargetListSingleton.procPID != pid:
+                logger.warning("Replace %s (created by process %u) for process with pid: %u" % (self.__class__.__name__, TargetListSingleton.procPID, pid))
+
+            TargetListSingleton.instance = TargetList(*args, **kwargs)
+            TargetListSingleton.procPID  = pid
+
+    def __getattr__(self, name):
+        return getattr(TargetListSingleton.instance, name)
+
+
 class SMTPTest(object):
-    def __init__(self, config=None, section=None, relaycache=None):
+    def __init__(self, config=None, section=None, relaycache=None, miltersession=None):
         self.config = config
         self.section = section
         self.relaycache = relaycache
         self.logger=logging.getLogger('%s.smtptest' % __package__ if __package__ else "fuglu")
         self.be_verbose = False
+        self.miltersession = miltersession
 
     def is_ip(self, value):
         return RE_IPV4.match(value) or RE_IPV6.match(value)
@@ -943,7 +1025,7 @@ class SMTPTest(object):
 
     def get_domain_config(self, domain, key, domainconfig=None, templatedict=None):
         """Get configuration value for domain or default. Apply template string if templatedict is not None"""
-        defval=self.config.get('ca_default',key)
+        defval=self.config.get('ca_default', key, fallback='')
         
         theval=defval
         if domainconfig is None: #nothing from sql
@@ -975,7 +1057,7 @@ class SMTPTest(object):
         value = value.lower()
         if value in ["1", "yes", "true", "on"]:
             return True
-        if value in ["0", "no", "false", "off"]:
+        if value in [None, '', "0", "no", "false", "off"]:
             return False
         raise ValueError('not a boolean value: %s' % value)
 
@@ -1004,16 +1086,20 @@ class SMTPTest(object):
         elif tp == 'static':
             relays = [val, ]
         elif tp == 'txt':
+            targetlist_from_file = TargetList(lowercase=True, filename=val)
+            targetslist = targetlist_from_file.get_list()
             try:
-                with open(val) as fp:
-                    lines = fp.readlines()
-                for line in lines:
-                    fdomain, ftarget = line.split()
-                    if domain.lower() == fdomain.lower():
-                        relays = [ftarget, ]
-                        break
+                relays = targetslist.get(domain.lower(), [])
             except Exception as e:
-                self.logger.error(f"Txt lookup failed: {str(e)}")
+                self.logger.error(f"Error getting element for SMTPTest: domain->{domain}, in list {targetslist} -> {str(e)} -> serverconfig:{serverconfig}")
+                relays = []
+        elif tp == 'tag':
+            keylist = val.split(':')
+            value = self.miltersession.tags
+            for key in keylist:
+                value = value.get(key, {})
+                if isinstance(value, str):
+                    relays = [value]
         else:
             self.logger.error(f'unknown relay lookup type: {tp}')
     
@@ -1033,9 +1119,9 @@ class SMTPTest(object):
         
         if mailfrom is None:
             mailfrom=""
-
+        
         if helo is None:
-            heloinput=self.config.get('ca_default', 'helo_name')
+            heloinput=self.config.get('ca_default', 'helo_name', fallback='')
             if heloinput and heloinput.strip():
                 heloinput = heloinput.strip()
                 if heloinput.startswith('$') and heloinput[1:]:
@@ -1043,6 +1129,8 @@ class SMTPTest(object):
                     helo = os.getenv(heloinput[1:], default=None)
                 else:
                     helo = heloinput
+            else:
+                helo = get_outgoing_helo(self.config)
         if self.be_verbose:
             logging.getLogger('fuglu.SMTPTester').debug(f"helo is: {helo}")
 
@@ -1529,10 +1617,11 @@ class RedisCache(CallAheadCacheInterface):
             entries = self._multiget(names, ['address', 'positive'])
             delkeys = []
             for item in entries:
-                if positive == 'positive' and item[1] == 'True':
-                    delkeys.append(f'addr-{item["address"]}')
-                elif positive == 'negative' and item[1] == 'False':
-                    delkeys.append(f'addr-{item["address"]}')
+                addr = item[0].decode()
+                if positive == 'positive' and item[1] == b'True':
+                    delkeys.append(f'addr-{addr}')
+                elif positive == 'negative' and item[1] == b'False':
+                    delkeys.append(f'addr-{addr}')
             
         if delkeys:
             redisconn = self.redis_pool.get_conn()
@@ -1596,7 +1685,7 @@ class RedisCache(CallAheadCacheInterface):
         entries = self._multiget(names, ['positive'])
         poscount = negcount = 0
         for item in entries:
-            if force_uString(item[0]) == 'True':
+            if item[0] == b'True':
                 poscount += 1
             else:
                 negcount += 1
@@ -1866,29 +1955,7 @@ class ConfigBackendInterface(object):
 
 
 
-class DBConfigBackend(ConfigBackendInterface):
-    def set_rcpt(self, recipient):
-        self.config = DBConfig(self.config, None)
-        self.config.set_rcpt(recipient)
-        
-    def get_domain_config_value(self, key):
-        return self.config.get(self.section, key)
-    
-    def get_domain_config_all(self):
-        retval = {}
-        self.config.load_section(self.section)
-        data = self.config.sectioncache[self.section]
-        for row in data:
-            retval[row[0]] = row[1]
-        return retval
-
-
-
 class MySQLConfigBackend(ConfigBackendInterface):
-    def __init__(self, config, section):
-        ConfigBackendInterface.__init__(self, config, section)
-    
-    
     def get_domain_config_value(self, key):
         sc=None
         try:
@@ -1917,10 +1984,6 @@ class MySQLConfigBackend(ConfigBackendInterface):
 
 class ConfigFileBackend(ConfigBackendInterface):
     """Read domain overrides directly from config, using ca_<domain> sections"""
-    def __init__(self, config, section):
-        ConfigBackendInterface.__init__(self, config, section)
-    
-    
     def get_domain_config_value(self, key):
         if self.config.has_option('ca_%s' % self.domain, key):
             return self.config.get('ca_%s' % self.domain, key)
@@ -1930,15 +1993,37 @@ class ConfigFileBackend(ConfigBackendInterface):
     def get_domain_config_all(self):
         retval = {}
         section = 'ca_%s' % self.domain
-        for option in self.config.options(section):
-            retval[option]=self.config.get(section, option)
+        if not self.config.has_section(section):
+            section = 'ca_default'
+        if self.config.has_section(section):
+            for option in self.config.options(section):
+                retval[option]=self.config.get(section, option)
         return retval
+
+
+
+class DBConfigBackend(ConfigFileBackend):
+    def set_rcpt(self, recipient):
+        super().set_rcpt(recipient)
+        self.config = DBConfig(self.config, None)
+        self.config.set_rcpt(recipient)
+        
+    #def get_domain_config_value(self, key):
+    #    return self.config.get(self.section, key)
+    
+    #def get_domain_config_all(self):
+    #    retval = {}
+    #    self.config.load_section(self.section)
+    #    data = self.config.sectioncache[self.section]
+    #    for row in data:
+    #        retval[row[0]] = row[1]
+    #    return retval
         
 
 
 class SMTPTestCommandLineInterface(BackendMixin):
     def __init__(self):
-        super().__init__(None, None)
+        super().__init__()
         self.section = 'AddressCheck'
         
         self.commandlist={
@@ -2046,10 +2131,16 @@ class SMTPTestCommandLineInterface(BackendMixin):
             domainconfig=self.config_backend.get_domain_config_all()
             try:
                 timeout = test.get_domain_config_float(domain, 'timeout', domainconfig)
-            except (ValueError, TypeError):
+            except (ValueError, TypeError, KeyError, configparser.NoSectionError):
                 timeout = 10
-            use_tls = test.get_domain_config_bool(domain, 'use_tls', domainconfig)
-            smtp_port = test.get_domain_config_int(domain, 'smtp_port', domainconfig)
+            try:
+                use_tls = test.get_domain_config_bool(domain, 'use_tls', domainconfig)
+            except (ValueError, TypeError, KeyError, configparser.NoSectionError):
+                use_tls = True
+            try:
+                smtp_port = test.get_domain_config_int(domain, 'smtp_port', domainconfig)
+            except (ValueError, TypeError, KeyError, configparser.NoSectionError):
+                smtp_port = smtplib.SMTP_PORT
         except IOError as e:
             print(str(e))
             timeout = 10
@@ -2261,10 +2352,17 @@ class SMTPTestCommandLineInterface(BackendMixin):
         testaddress = test.maketestaddress(domain, domainconfig)
         try:
             timeout = test.get_domain_config_float(domain, 'timeout', domainconfig)
-        except (ValueError, TypeError):
+        except (ValueError, TypeError, KeyError, configparser.NoSectionError):
             timeout = 10
-        use_tls = test.get_domain_config_bool(domain, 'use_tls', domainconfig)
-        smtp_port = test.get_domain_config_int(domain, 'smtp_port', domainconfig)
+        try:
+            use_tls = test.get_domain_config_bool(domain, 'use_tls', domainconfig)
+        except (ValueError, TypeError, KeyError, configparser.NoSectionError):
+            use_tls = True
+        try:
+            smtp_port = test.get_domain_config_int(domain, 'smtp_port', domainconfig)
+        except (ValueError, TypeError, KeyError, configparser.NoSectionError):
+            smtp_port = smtplib.SMTP_PORT
+        
         result=test.smtptest(relay, [address,testaddress], mailfrom=sender, timeout=timeout, use_tls=use_tls, port=smtp_port)
         
         servercachetime = test.get_domain_config_int(domain, 'test_server_interval', domainconfig)
