@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-#   Copyright 2009-2021 Oli Schacher, Fumail Project
+#   Copyright 2009-2022 Oli Schacher, Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -133,8 +133,9 @@ Tags:
                 'description': 'set custom engine name (defaults to %s)' % self.enginename,
             },
         }
-        self.logger = self._logger()
         self.enginename = 'ClamAV'
+        # logger after engine so we can use the enginename in the logger...
+        self.logger = self._logger()
         self.skiplist = None
     
     
@@ -152,16 +153,20 @@ Tags:
         :return: (bool) True if hit should not be counted
         """
         self._init_skiplist()
-        if self.skiplist and suspect.is_welcomelisted():
+        if viruses and self.skiplist and suspect.is_welcomelisted():
             skiplist = self.skiplist.get_list()
-            virusnames = viruses.keys()
+            virusnames = list(viruses.values())
             if len(virusnames) == 1 and virusnames[0] in skiplist:
                 self.logger.info('%s got hit on %s but is skiplisted' % (suspect.id, ','.join(virusnames)))
                 return True
         return False
     
     def __str__(self):
-        return "Clam AV"
+        classname = self.__class__.__name__
+        if self.section == classname:
+            return "Clam AV"
+        else:
+            return f'Clam AV - {self.section}'
 
     def examine(self, suspect):
         if self._check_too_big(suspect):
@@ -288,8 +293,9 @@ Tags:
         pipelining = self.config.getboolean(self.section, 'pipelining')
         s = self.__init_socket__(oneshot=not pipelining)
         s.sendall(b'zINSTREAM\0')
-        default_chunk_size = 2048
         remainingbytes = force_bString(content)
+        default_chunk_size = min(len(remainingbytes), 20480)
+        maxtime = time.time() + self.config.getint(self.section, 'timeout')
 
         numChunksToSend = math.ceil(len(remainingbytes)/default_chunk_size)
         iChunk = 0
@@ -297,6 +303,8 @@ Tags:
         self.logger.debug('%s: sending message in %u chunks of size %u bytes' % (suspectid, numChunksToSend, default_chunk_size))
 
         while len(remainingbytes) > 0:
+            if time.time() > maxtime:
+                raise TimeoutError('Timeout sending to AV daemon')
             iChunk = iChunk + 1
             chunklength = min(default_chunk_size, len(remainingbytes))
             #self.logger.debug('sending chunk %u/%u' % (iChunk,numChunksToSend))
@@ -306,7 +314,7 @@ Tags:
             s.sendall(struct.pack(b'!L', chunklength))
             s.sendall(chunkdata)
         self.logger.debug('%s: sent chunk %u/%u, last number of bytes sent was %u' % (suspectid, iChunk, numChunksToSend, chunklength))
-        self.logger.debug('%s: All chunks send, send 0 - size to tell ClamAV the whole message has been sent' % suspectid)
+        self.logger.debug('%s: All chunks sent, sending 0 - size to tell ClamAV the whole message has been sent' % suspectid)
         s.sendall(struct.pack(b'!L', 0))
         self.logger.debug('%s: 0 has been sent, now wait for anser' % suspectid)
         dr = {}
@@ -397,15 +405,15 @@ Tags:
                 self.logger.warning("%s: Failed receive attempt %u/%u: %s" % (suspectID,failedAttempt, maxFailedAttempts, str(e)))
                 if failedAttempt == maxFailedAttempts:
                     raise ClamavTimeout(f"{suspectID}, Max failed received attempts({maxFailedAttempts}): {str(e)}")
-            else:
-                # Sometimes we get one socket error and after that we never get a proper answer to satisfy
-                # break or exception condition in try block, neither are there subsequent socket errors.
-                # This can lead to an endless loop where a worker remains forever in this while loop,
-                # rendering the worker useless and causing high cpu load. We thus add this additional
-                # loop termination condition.
-                runtime = time.time() - starttime
-                if runtime > readtimeout:
-                    raise ClamavTimeout(f"{suspectID}: Read timeout after {runtime:.2f}s > {readtimeout:.2f}s")
+            
+            # Sometimes we get one socket error and after that we never get a proper answer to satisfy
+            # break or exception condition in try block, neither are there subsequent socket errors.
+            # This can lead to an endless loop where a worker remains forever in this while loop,
+            # rendering the worker useless and causing high cpu load. We thus add this additional
+            # loop termination condition.
+            runtime = time.time() - starttime
+            if runtime > readtimeout:
+                raise ClamavTimeout(f"{suspectID}: Read timeout after {runtime:.2f}s > {readtimeout:.2f}s")
 
         return data[:-1]  # remove \0 at the end
     
