@@ -1,6 +1,6 @@
 #!/usr/bin/python3
 # -*- coding: utf-8 -*-
-#   Copyright 2009-2021 Oli Schacher, Fumail Project
+#   Copyright 2009-2022 Oli Schacher, Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -24,6 +24,7 @@ import re
 import os
 import fcntl
 from urllib.parse import urlparse
+from typing import Optional
 
 
 
@@ -382,7 +383,7 @@ class URIWriter(ScannerPlugin):
         self.requiredvars = {
             'path_template': {
                 'default': '/var/tmp/${urltype}-uris.txt',
-                'descritption': 'template string where to place uris of a certain category'
+                'description': 'template string where to place uris of a certain category'
             },
             
             'urltype_map_file': {
@@ -418,11 +419,15 @@ class URIWriter(ScannerPlugin):
     
     
     
-    def _exclude_uris(self, uris):
+    def _exclude_uris(self, uris, fugluid: str = ""):
         excludes = self.config.getlist(self.section, 'ignore_domains', separators=' ')
         newuris = []
         for uri in uris:
-            p = urlparse(uri)
+            try:
+                p = urlparse(uri)
+            except Exception as exc:
+                self.logger.debug(f"{fugluid} ignore uri due to parsing problem {str(exc)} for uri: \"{uri}\"")
+                continue
             for e in excludes:
                 if not p.netloc == e and not p.netloc.endswith(f'.{e}'):
                     newuris.append(uri)
@@ -481,7 +486,7 @@ class URIWriter(ScannerPlugin):
 
     def _run(self, suspect):
         uris = self._get_uris(suspect)
-        uris = self._exclude_uris(uris)
+        uris = self._exclude_uris(uris, fugluid=suspect.id)
         uris = list(set(uris))
         if not uris:
             self.logger.debug(f'{suspect.id} no URIs to write')
