@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-#   Copyright 2009-2021 Oli Schacher, Fumail Project
+#   Copyright 2009-2022 Oli Schacher, Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -131,21 +131,33 @@ class FilterDecision(ScannerPlugin):
     
     def _check_deliver_spam(self, suspect):
         deliver = True
-        if suspect.is_spam():
+        if suspect.is_virus():
+            deliver=False
+        elif suspect.is_blocklisted():
+            deliver=False
+        elif suspect.is_welcomelisted():
+            deliver=True
+        elif suspect.is_blocked():
+            deliver=False
+        elif suspect.is_spam():
             deliver = False
             if self._get_filtersetting(suspect, 'deliver_spam', False):
+                self.logger.debug('%s is spam, but delivering due to filtersetting' % suspect.id)
                 deliver = True
         elif suspect.is_highspam():
             deliver = False
             if self._get_filtersetting(suspect, 'deliver_highspam', False):
+                self.logger.debug('%s is highspam, but delivering due to filtersetting' % suspect.id)
                 deliver = True
         
-        if not suspect.is_ham() and not deliver and suspect.get_tag('archived', False):
+        if not deliver and not suspect.is_ham() and not suspect.get_tag('archived', False):
+            self.logger.debug('%s is ham=%s, but delivering because archived=%s' % (suspect.id, suspect.is_ham(), suspect.get_tag('archived')))
             deliver = True
         
-        if not suspect.is_ham() and deliver:
+        if deliver and not suspect.is_ham():
             spam_recipient = self._get_filtersetting(suspect, 'spam_recipient')
             if spam_recipient:
+                self.logger.debug('%s delivering to spam rcpt %s' % (suspect.id, spam_recipient))
                 suspect.to_address = spam_recipient
         return deliver
     
@@ -153,11 +165,11 @@ class FilterDecision(ScannerPlugin):
     def examine(self, suspect):
         deliver = self._check_deliver_spam(suspect)
         if not deliver:
-            if self.config.get(self.section, 'testmode'):
-                self.logger.info('%s testmode enabled. else would be deleting due to filterdecision')
+            if self.config.getboolean(self.section, 'testmode'):
+                self.logger.info('%s testmode enabled. else would be deleting due to filterdecision' % suspect.id)
                 return DUNNO
             else:
-                self.logger.info('%s deleting due to filterdecision')
+                self.logger.info('%s deleting due to filterdecision' % suspect.id)
                 return DELETE
         
         if not suspect.is_ham():
