@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-#   Copyright 2009-2021 Oli Schacher, Fumail Project
+#   Copyright 2009-2022 Oli Schacher, Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -28,17 +28,16 @@ import logging
 import os
 import re
 import time
+import traceback
 import typing as tp
 import fnmatch
 import socket
 
-from domainmagic.mailaddr import domain_from_mail
-
 import fuglu.connectors.asyncmilterconnector as asm
 import fuglu.connectors.milterconnector as sm
 
 from fuglu.shared import ScannerPlugin, apply_template, DUNNO, FileList, string_to_actioncode, get_default_cache, \
-    extract_domain, Suspect, Cache, _SuspectTemplate, actioncode_to_string, deprecated
+    extract_domain, Suspect, Cache, _SuspectTemplate, actioncode_to_string, deprecated, get_outgoing_helo
 from fuglu.mshared import BMPRCPTMixin, BasicMilterPlugin, EOM, retcode2milter
 from fuglu.extensions.sql import get_session, SQL_EXTENSION_ENABLED, get_domain_setting
 from fuglu.extensions.dnsquery import DNSQUERY_EXTENSION_ENABLED, lookup, QTYPE_TXT, QTYPE_MX
@@ -68,10 +67,15 @@ except ImportError:
 
 try:
     from domainmagic.tld import TLDMagic
+    from domainmagic.mailaddr import domain_from_mail
     DOMAINMAGIC_AVAILABLE = True
 except ImportError:
     DOMAINMAGIC_AVAILABLE = False
     TLDMagic = None
+    def domain_from_mail(address, **kwargs):
+        if address and "@" in address:
+            return address.rsplit("@", 1)[-1]
+        return address
 
 try:
     import dmarc
@@ -311,7 +315,7 @@ It is currently recommended to leave both header and body canonicalization as 'r
                     suspect.id, i, valid, authordomain, senderdomain))
             except DKIMException as de:
                 self.logger.warning("%s: DKIM validation failed: %s" % (suspect.id, str(de)))
-
+            
             suspect.set_tag("DKIMVerify.sigvalid", valid)
             suspect.set_tag("DKIMVerify.result", saval)
             suspect.set_tag("DKIMVerify.sigvalidauthor", authordomain)
@@ -324,8 +328,13 @@ It is currently recommended to leave both header and body canonicalization as 'r
         except NameError as e:
             self.logger.warning("%s: DKIM validation failed due to missing dependency: %s" % (suspect.id, str(e)))
             suspect.set_tag('DKIMVerify.skipreason', 'plugin error')
+        except TimeoutError as e:
+            self.logger.warning("%s: DKIM validation failed due to: %s" % (suspect.id, str(e)))
+            suspect.set_tag('DKIMVerify.skipreason', 'plugin error')
         except Exception as e:
-            self.logger.warning("%s: DKIM validation failed: %s" % (suspect.id, str(e)))
+            self.logger.warning("%s: DKIM validation failed: %s %s" % (suspect.id, e.__class__.__name__, str(e)))
+            import traceback
+            self.logger.debug(traceback.format_exc())
             suspect.set_tag('DKIMVerify.skipreason', 'plugin error')
 
         return DUNNO
@@ -555,6 +564,10 @@ class SPFPlugin(BMPRCPTMixin, ScannerPlugin, BasicMilterPlugin):
                 'default': '',
                 'description': '(milter) if this is non-empty, only sender domains in this file will be checked for SPF. define exceptions by prefixing with ! (e.g. example.com !foo.example.com). define TLD wildcards using * (e.g. example.*)',
             },
+            'ip_selective_spf_file': {
+                'default': '',
+                'description': '(milter) if this is non-empty, only sender IPs in this file will be checked for SPF. If IP also whitelisted, this is ignored: no check. This has precedence over domain_selective_spf_file',
+            },
             'selective_softfail': {
                 'default': 'False',
                 'description': '(milter) evaluate all senders for hard fails (unless listed in domain_whitelist_file) and only evaluate softfail for domains listed in domain_selective_spf_file',
@@ -595,6 +608,10 @@ class SPFPlugin(BMPRCPTMixin, ScannerPlugin, BasicMilterPlugin):
                 'default': '10',
                 'description': 'maximum number of lookups (RFC defaults to 10)',
             },
+            'max_lookup_time': {
+                'default': '20',
+                'description': 'maximum time per DNS lookup (RFC defaults to 20 seconds)',
+            },
             'hoster_mx_exception': {
                 'default': '.google.com .protection.outlook.com',
                 'description': 'always consider pass if mail is sent from servers specified, MX points to this server, and SPF record contains MX directive',
@@ -624,6 +641,7 @@ class SPFPlugin(BMPRCPTMixin, ScannerPlugin, BasicMilterPlugin):
 
         self.private_nets = [
             ipaddress.ip_network('10.0.0.0/8'),  # private network
+            ipaddress.ip_network('100.64.0.0/10'),  # cgnat
             ipaddress.ip_network('127.0.0.0/8'),  # localhost
             ipaddress.ip_network('169.254.0.0/16'),  # link local
             ipaddress.ip_network('172.16.0.0/12'),  # private network
@@ -638,6 +656,8 @@ class SPFPlugin(BMPRCPTMixin, ScannerPlugin, BasicMilterPlugin):
         self.selective_domain_loader = None
         self.domain_whitelist_loader = None
 
+        self.selective_ip_loader = None
+
     def __check_domain(self, domain, listentry, check_subdomain):
         listed = False
         if listentry == domain:
@@ -673,7 +693,10 @@ class SPFPlugin(BMPRCPTMixin, ScannerPlugin, BasicMilterPlugin):
 
         return listed
 
-    def check_this_domain(self, from_domain):
+    def check_this_domain(self, from_domain,
+                          sessid: str = "<sessionid>",
+                          logmsgs: bool = True
+                          ):
         do_check = self.check_cache.get_cache(from_domain)
         if do_check is not None:
             return do_check
@@ -685,7 +708,9 @@ class SPFPlugin(BMPRCPTMixin, ScannerPlugin, BasicMilterPlugin):
         if domain_whitelist_file != '' and os.path.exists(domain_whitelist_file):
             if self.domain_whitelist_loader is None:
                 self.domain_whitelist_loader = FileList(domain_whitelist_file, lowercase=True)
-            if not self._domain_in_list(from_domain, self.domain_whitelist_loader.get_list(), check_subdomain):
+            if self._domain_in_list(from_domain, self.domain_whitelist_loader.get_list(), check_subdomain):
+                if logmsgs:
+                    self.logger.debug(f"{sessid} domain in domain_whitelist_file -> set do_check to False")
                 do_check = False
 
         if do_check is None:
@@ -694,6 +719,8 @@ class SPFPlugin(BMPRCPTMixin, ScannerPlugin, BasicMilterPlugin):
                 if self.selective_domain_loader is None:
                     self.selective_domain_loader = FileList(selective_sender_domain_file, lowercase=True)
                 if self._domain_in_list(from_domain, self.selective_domain_loader.get_list(), check_subdomain):
+                    if logmsgs:
+                        self.logger.debug(f"{sessid} domain in domain_selective_spf_file -> set do_check to True")
                     do_check = True
 
         if do_check is None:
@@ -704,6 +731,8 @@ class SPFPlugin(BMPRCPTMixin, ScannerPlugin, BasicMilterPlugin):
             if dbconnection != '' and SQL_EXTENSION_ENABLED:
                 cache = get_default_cache()
                 if get_domain_setting(from_domain, dbconnection, sqlquery, cache, self.section, False, self.logger):
+                    if logmsgs:
+                        self.logger.debug(f"{sessid} domain has spf-dbsetting -> set do_check to True")
                     do_check = True
 
             elif dbconnection != '' and not SQL_EXTENSION_ENABLED:
@@ -716,7 +745,10 @@ class SPFPlugin(BMPRCPTMixin, ScannerPlugin, BasicMilterPlugin):
         return do_check
 
     def is_private_address(self, addr):
-        ipaddr = ipaddress.ip_address(addr)
+        try:
+            ipaddr = ipaddress.ip_address(addr)
+        except ValueError:
+            return False
         private = False
         for net in self.private_nets:
             if ipaddr in net:
@@ -725,6 +757,8 @@ class SPFPlugin(BMPRCPTMixin, ScannerPlugin, BasicMilterPlugin):
         return private
 
     def ip_whitelisted(self, addr):
+        if not addr:
+            return False
         if self.is_private_address(addr):
             return True
 
@@ -749,6 +783,24 @@ class SPFPlugin(BMPRCPTMixin, ScannerPlugin, BasicMilterPlugin):
                     return True
         return False
 
+    def ip_spf_check(self, addr):
+        try:
+            ip_selective_file = self.config.get(self.section, 'ip_selective_spf_file').strip()
+        except Exception:
+            ip_selective_file = ''
+
+        if ip_selective_file != '' and os.path.exists(ip_selective_file):
+            if self.selective_ip_loader is None:
+                self.selective_ip_loader = FileList(ip_selective_file, lowercase=True)
+
+            plainlist = self.selective_ip_loader.get_list()
+            self.ip_selective = [ipaddress.ip_network(x) for x in plainlist]
+            checkaddr = ipaddress.ip_address(addr)
+            for net in self.ip_selective:
+                if checkaddr in net:
+                    return True
+        return False
+
     def _hoster_mx_exception(self, hosters: tp.List[str], client_name: str, sender: str, sessid: str = "<sessid>") -> bool:
         client_name = force_uString(client_name).lower().rstrip('.')
         mxrec = None
@@ -783,8 +835,7 @@ class SPFPlugin(BMPRCPTMixin, ScannerPlugin, BasicMilterPlugin):
                                     self.logger.debug(f'{sessid} got spf {txt} for domain {domainname}')
                                     spfrec.append(txt)
                 except Exception as e:
-                    self.logger.info(
-                        f'{sessid} failed to lookup record for domain {domainname}: {e.__class__.__name__}: {str(e)}')
+                    self.logger.info(f'{sessid} failed to lookup record for domain {domainname}: {e.__class__.__name__}: {str(e)}')
 
                 if spfrec and len(spfrec) == 1 and re.search('\s\+?mx\s', spfrec[0]):
                     return True
@@ -793,7 +844,7 @@ class SPFPlugin(BMPRCPTMixin, ScannerPlugin, BasicMilterPlugin):
     def _in_hoster(self, helo: str, hosters: tp.List[str]) -> bool:
         is_hoster = False
         for hoster in hosters:
-            if helo.endswith(hoster):
+            if helo and helo.endswith(hoster):
                 is_hoster = True
                 break
         return is_hoster
@@ -808,7 +859,7 @@ class SPFPlugin(BMPRCPTMixin, ScannerPlugin, BasicMilterPlugin):
             return False
 
         if not suspect.from_address:
-            self.logger.debug('%s SPF Check skipped, bounce')
+            self.logger.debug(f'{suspect.id}: SPF Check skipped, bounce')
             suspect.set_tag('SPF.status', 'skipped')
             suspect.set_tag("SPF.explanation", 'skipped bounce')
             return False
@@ -816,7 +867,7 @@ class SPFPlugin(BMPRCPTMixin, ScannerPlugin, BasicMilterPlugin):
         self.skiplist.filename = self.config.get(self.section, 'skiplist')
         checkdomains = self.skiplist.get_list()
         if suspect.from_domain in checkdomains:
-            self.logger.debug('%s SPF Check skipped, sender domain skiplisted')
+            self.logger.debug(f'{suspect.id}: SPF Check skipped, sender domain skiplisted')
             suspect.set_tag('SPF.status', 'skipped')
             suspect.set_tag("SPF.explanation", 'sender domain skiplisted')
             return False
@@ -835,9 +886,11 @@ class SPFPlugin(BMPRCPTMixin, ScannerPlugin, BasicMilterPlugin):
             if logmsgs:
                 self.logger.error(f'{sessid} missing client_address({client_address is None}) or sender({sender is None})')
             return False
+        
         elif helo_name is None:
             if logmsgs:
                 self.logger.info(f'{sessid} missing helo_name')
+            #return False
 
         if self.ip_whitelisted(client_address):
             if logmsgs:
@@ -849,6 +902,15 @@ class SPFPlugin(BMPRCPTMixin, ScannerPlugin, BasicMilterPlugin):
                 self.logger.debug(f"{sessid} no sender - no SPF check")
             return False
 
+        if self.ip_spf_check(client_address):
+            if logmsgs:
+                self.logger.info(f"{sessid} client {client_address} is selected - activate SPF check")
+            selected_IP = True
+        else:
+            if logmsgs:
+                self.logger.debug(f"{sessid} client {client_address} is not selected - selcted_IP is set to False...")
+            selected_IP = False
+
         sender_domain = sm.MilterSession.extract_domain(sender)
         if sender_domain is None:
             if logmsgs:
@@ -856,7 +918,17 @@ class SPFPlugin(BMPRCPTMixin, ScannerPlugin, BasicMilterPlugin):
             return False
 
         sender_domain = sender_domain.lower()
-        check_domain = self.check_this_domain(sender_domain)
+
+        # If IP listed in ip_selective_spf file, we make the check, else with check if domain in domain_selective_spf
+        if selected_IP:
+            if logmsgs:
+                self.logger.debug(f"{sessid} client {client_address} selected_IP is true -> set check_domain to True")
+            check_domain = True
+        else:
+            check_domain = self.check_this_domain(sender_domain, sessid=sessid, logmsgs=logmsgs)
+            if logmsgs:
+                self.logger.debug(f"{sessid} client {client_address} selected_IP is false -> call check_this_domain -> returns {check_domain}")
+
 
         selective_softfail = self.config.getboolean(self.section, 'selective_softfail')
         if not check_domain and not selective_softfail:  # selective_softfail is False: check all and filter later
@@ -866,13 +938,14 @@ class SPFPlugin(BMPRCPTMixin, ScannerPlugin, BasicMilterPlugin):
 
         return check_domain
 
-    def _spf_lookup(self, ip, from_address, helo, maxlookups, retries=3):
-        spf.MAX_LOOKUP = maxlookups
-        result, explanation = spf.check2(ip, from_address, helo)
+    def _spf_lookup(self, ip, from_address, helo_name, retries=3):
+        if helo_name is None:
+            helo_name = ''
+        result, explanation = spf.check2(ip, from_address, helo_name)
         if result == 'temperror' and retries > 0:
             time.sleep(self.config.getint(self.section, 'temperror_sleep'))
             retries -= 1
-            result, explanation = self._spf_lookup(ip, from_address, helo, maxlookups, retries=retries)
+            result, explanation = self._spf_lookup(ip, from_address, helo_name, retries=retries)
         return result, explanation
 
     def run_all_spf_tests(self,
@@ -884,22 +957,30 @@ class SPFPlugin(BMPRCPTMixin, ScannerPlugin, BasicMilterPlugin):
                           catch_exceptions: bool = True,
                           ) -> tp.Tuple[str, str]:
         """run spf check and apply hoster exceptions"""
-
+        
+        result = 'none'
+        explanation = ''
         try:
             retries = self.config.getint(self.section, 'temperror_retries')
             maxlookups = self.config.getint(self.section, 'max_lookups')
-            result, explanation = self._spf_lookup(client_address, sender, helo_name, maxlookups, retries=retries)
+            spf.MAX_LOOKUP = maxlookups
+            maxlookuptime = self.config.getint(self.section, 'max_lookup_time')
+            spf.MAX_PER_LOOKUP_TIME = maxlookuptime
+            if client_address:
+                result, explanation = self._spf_lookup(client_address, sender, helo_name, retries=retries)
+                self.logger.debug(f"{sessid} lookup result={result}")
+            else:
+                self.logger.debug(f'{sessid} skipped SPF check for {sender} because client_address is empty')
         except Exception as e:
             if "SERVFAIL" in str(e):
                 # info level is enough
                 self.logger.info(f'{sessid} failed to check SPF for {sender} due to: {str(e)}')
             else:
                 self.logger.error(f'{sessid} failed to check SPF for {sender} due to: {str(e)}')
+                self.logger.debug(traceback.format_exc())
             if not catch_exceptions:
                 raise Exception(str(e)).with_traceback(e.__traceback__)
-            result = 'none'
-            explanation = ''
-
+        
         hoster_mx_exception = self.config.getlist(self.section, 'hoster_mx_exception', separators=' ', lower=True)
         if hoster_mx_exception and result in ['fail', 'softfail'] and helo_name and self._in_hoster(helo_name, hoster_mx_exception):
             if self._hoster_mx_exception(hosters=hoster_mx_exception, client_name=client_name, sender=sender, sessid=sessid):
@@ -908,8 +989,7 @@ class SPFPlugin(BMPRCPTMixin, ScannerPlugin, BasicMilterPlugin):
                 explanation = 'hoster mx permit'
 
         if result != 'none':
-            self.logger.info(f'{sessid} SPF client={client_address}, sender={sender}, '
-                             f'h={helo_name} result={result} : {explanation}')
+            self.logger.info(f'{sessid} SPF client={client_address}, sender={sender}, h={helo_name} result={result} : {explanation}')
         return result, explanation
 
     def process_spf_tests(self,
@@ -920,16 +1000,16 @@ class SPFPlugin(BMPRCPTMixin, ScannerPlugin, BasicMilterPlugin):
                           log: bool = True,
                           on_softfail_dunnotag: tp.Optional[str] = None,
                           on_fail_dunnotag: tp.Optional[str] = None,
-                          selective_softfail: tp.Optional[str] = None,
+                          selective_softfail: bool = False,
                           astring_on_fail: tp.Optional[str] = None,
                           astring_on_softfail: tp.Optional[str] = None,
                           astring_on_general: tp.Optional[str] = None,
                           ) -> str:
-
+        
         action = DUNNO
         if result == 'fail' and (check_domain or selective_softfail):
-            # reject on hard fail if domain is listed in domain_selective_spf_file
-            # or selective_softfail is enabled
+            # reject on hard fail if domain is in selective hardfail list or
+            # "selective softfail (==always fail on hard fail)" is enabled
             action = string_to_actioncode(
                 astring_on_fail if astring_on_fail else self.config.get(self.section, 'on_fail')
             )
@@ -937,15 +1017,13 @@ class SPFPlugin(BMPRCPTMixin, ScannerPlugin, BasicMilterPlugin):
             if on_fail_dunnotag:
                 if action != DUNNO:
                     if log:
-                        self.logger.info(f"{sessid} Change from fail action "
-                                         f"to DUNNO/CONTINUE due to WL:{on_fail_dunnotag}")
+                        self.logger.info(f"{sessid} Change from fail action to DUNNO/CONTINUE due to WL:{on_fail_dunnotag}")
                     action = DUNNO
                 else:
                     if log:
-                        self.logger.debug(f"{sessid} Keep fail action "
-                                          f"as DUNNO/CONTINUE with WL:{on_fail_dunnotag}")
+                        self.logger.debug(f"{sessid} Keep fail action as DUNNO/CONTINUE with WL:{on_fail_dunnotag}")
 
-        elif result == 'softfail' and check_domain:
+        elif result == 'softfail' and (check_domain or not selective_softfail):
             # reject on soft fail if domain is listed in domain_selective_spf_file
             action = string_to_actioncode(
                 astring_on_softfail if astring_on_softfail else self.config.get(self.section, 'on_softfail')
@@ -967,13 +1045,16 @@ class SPFPlugin(BMPRCPTMixin, ScannerPlugin, BasicMilterPlugin):
         elif result == 'softfail' and not check_domain:
             # only log soft fail if domain is not listed in domain_selective_spf_file
             if log:
-                self.logger.info(f'{sessid} ignoring SPF check for {sender} evaluating to softfail')
+                self.logger.info(f'{sessid} ignoring SPF check for {sender} evaluating to {result}')
         elif result not in ['fail', 'softfail']:
             # custom action for none, neutral, pass, permerr
             if astring_on_general:
                 action = string_to_actioncode(astring_on_general)
             elif self.config.has_option(self.section, f'on_{result}'):
                 action = string_to_actioncode(self.config.get(self.section, f'on_{result}'))
+        else:
+            if log:
+                self.logger.debug(f'{sessid} ignoring SPF check for {sender} due to result={result} check_domain={check_domain} and selective_softfail={selective_softfail}')
         return action
 
     def _get_clientinfo(self, suspect):
@@ -1000,9 +1081,10 @@ class SPFPlugin(BMPRCPTMixin, ScannerPlugin, BasicMilterPlugin):
             clientip = helo_name = clienthostname = None
 
         if clientip is None:
+            suspect.set_tag('SPF.status', 'skipped')
             self.logger.debug(f"{suspect.id} Could not extract clientip, skipping tests")
             return DUNNO
-
+        
         clientip = force_uString(clientip)
         helo_name = force_uString(helo_name)
         clienthostname = force_uString(clienthostname)
@@ -1016,8 +1098,9 @@ class SPFPlugin(BMPRCPTMixin, ScannerPlugin, BasicMilterPlugin):
             check_domain_milter = False
 
         if not (check_domain or check_domain_milter):
+            self.logger.debug(f"{suspect.id} SPF checks disabled: check_domain={check_domain} check_domain_milter={check_domain_milter}")
             return DUNNO
-
+        
         action = DUNNO
         message = None
         try:
@@ -1025,6 +1108,10 @@ class SPFPlugin(BMPRCPTMixin, ScannerPlugin, BasicMilterPlugin):
             result, explanation = \
                 self.run_all_spf_tests(sender=sender, helo_name=helo_name, client_address=clientip,
                                        client_name=clienthostname, sessid=suspect.id, catch_exceptions=False)
+            
+            #self.logger.debug(f"{suspect.id} sender={sender} helo_name={helo_name} client_address={clientip} client_name={clienthostname}")
+            #self.logger.debug(f"{suspect.id} result={result} explanation={explanation} check_domain={check_domain} check_domain_milter={check_domain_milter}")
+            
             # set tags and SA temp header
             if check_domain:
                 suspect.set_tag("SPF.status", result)
@@ -1054,7 +1141,7 @@ class SPFPlugin(BMPRCPTMixin, ScannerPlugin, BasicMilterPlugin):
             suspect.set_tag('SPF.status', 'skipped')
             suspect.set_tag("SPF.explanation", str(e))
             self.logger.warning('%s SPF check failed for %s due to %s' % (suspect.id, suspect.from_domain, str(e)))
-
+        
         return action, message
 
     def examine_rcpt(self, sess: tp.Union[asm.MilterSession, sm.MilterSession], recipient: bytes) -> tp.Union[bytes, tp.Tuple[bytes, str]]:
@@ -1087,6 +1174,7 @@ class SPFPlugin(BMPRCPTMixin, ScannerPlugin, BasicMilterPlugin):
         templ_dict = sess.get_templ_dict()
         templ_dict['result'] = result
         templ_dict['explanation'] = explanation
+        templ_dict['client_address'] = client_address
         message = template.safe_substitute(templ_dict)
 
         # convert to milter return code
@@ -1116,10 +1204,11 @@ class SPFPlugin(BMPRCPTMixin, ScannerPlugin, BasicMilterPlugin):
             print(fc.strcolor("ERROR: ", "red"), "pyspf or pydns module not installed - this plugin will do nothing")
             lint_ok = False
 
-        if not self.checkConfig():
+        if not self.check_config():
             print(fc.strcolor("ERROR: ", "red"), "Error checking config")
             lint_ok = False
-
+        
+        import configparser
         for result in ["fail", "softfail", "none", "neutral", "pass", "permerr"]:
             try:
                 configaction = self.config.get(self.section, f'on_{result}')
@@ -1129,8 +1218,11 @@ class SPFPlugin(BMPRCPTMixin, ScannerPlugin, BasicMilterPlugin):
                     if not action:
                         lint_ok = False
                         print(fc.strcolor("ERROR: ", "red"), f"'on_{result}'-Action value {configaction}' not in allowed choices: 'DUNNO', 'DEFER', 'REJECT'")
-            except Exception:
+            except configparser.NoOptionError:
                 pass
+            except Exception as e:
+                lint_ok = False
+                print(fc.strcolor("ERROR: ", "red"), f'failed to evaluate check result due to {e.__class__.__name__}: {str(e)}')
 
         domain_whitelist_file = self.config.get(self.section, 'domain_whitelist_file').strip()
         if domain_whitelist_file != '' and not os.path.exists(domain_whitelist_file):
@@ -1173,9 +1265,6 @@ class SPFPlugin(BMPRCPTMixin, ScannerPlugin, BasicMilterPlugin):
 
         return lint_ok
 
-    def __str__(self):
-        return "SPF Check"
-
 
 class DMARCPlugin(ScannerPlugin):
     """
@@ -1537,7 +1626,7 @@ class SpearPhishPlugin(ScannerPlugin):
     @deprecated
     def get_domain_setting(self, domain, dbconnection, sqlquery, cache, cachename, default_value=None, logger=None):
         if logger is None:
-            logger = logging.getLogger()
+            logger = logging.getLogger('fuglu.plugins.SpearPhishPlugin.sql')
 
         cachekey = '%s-%s' % (cachename, domain)
         cached = cache.get_cache(cachekey)
@@ -1737,7 +1826,7 @@ class SenderRewriteScheme(ScannerPlugin):
 
     def get_sql_setting(self, domain, dbconnection, sqlquery, cache, cachename, default_value=None, logger=None):
         if logger is None:
-            logger = logging.getLogger()
+            logger = logging.getLogger('fuglu.plugins.domainauth.SenderRewriteScheme.sql')
 
         cachekey = '%s-%s' % (cachename, domain)
         cached = cache.get_cache(cachekey)
@@ -1904,12 +1993,13 @@ def get_host_ipaddr(inhostname: tp.Optional[str] = None) -> tp.Optional[str]:
     """
     dummyhost = "255.255.255.254"
     defaultip = '0.0.0.0'
+    ipexclude = re.compile('^(127|0|10|192\.168|172\.(1[6-9]|[2-3][0-9]|4[0-1])|169\.254|100\.(6[4-9]|[7-9][0-9]|1[0-1][0-9]|12[0-7]))\.')
 
     try:
         hostname = inhostname if inhostname else socket.gethostname()
-        ipguess = [ip for ip in socket.gethostbyname_ex(hostname)[2] if not ip.startswith("127.")]
-    except socket.gaierror:
-        # name does not resolve
+        ipguess = [ip for ip in socket.gethostbyname_ex(hostname)[2] if not ipexclude.match(ip)]
+    except (socket.gaierror, UnicodeError):
+        # name does not resolve or hostname is empty
         ipguess = []
 
     if not ipguess and not inhostname:
@@ -1941,6 +2031,7 @@ class SPFOut(SPFPlugin):
         self._myip = None
         self._myhostname = None
         self._myhelo = None
+        
 
     def _init_ipnames(self):
         self._myip = None
@@ -1951,7 +2042,9 @@ class SPFOut(SPFPlugin):
             self.logger.debug(f"Got IP from env var as: {inip}")
 
         inhost = self.config.get(self.section, "hostname")
-        if inhost and inhost.startswith("$"):
+        if not inhost:
+            inhost = get_outgoing_helo(self.config)
+        elif inhost and inhost.startswith("$"):
             inhost = os.environ[inhost[1:]]
             self.logger.debug(f"Got hostname from env var as: {inhost}")
 
@@ -1964,13 +2057,15 @@ class SPFOut(SPFPlugin):
                 else:
                     # get ip from host
                     self._myip = get_host_ipaddr(inhostname=inhost)
-            except Exception:
-                pass
+                    self.logger.debug('detected local IP address %s from hostname %s' % (self._myip, inhost))
+            except Exception as e:
+                self.logger.debug(f'failed to detect host information due to {e.__class__.__name__}: {str(e)}')
 
         if self._myip is None:
             self._myip = get_host_ipaddr()
             self._myhostname = socket.gethostname()
             self._myhostname = self._myhostname
+            self.logger.debug('detected local IP address %s' % self._myip)
 
     def _get_clientinfo(self, suspect):
         if self._myip is None:
