@@ -1,6 +1,6 @@
 #!/usr/bin/python
 # -*- coding: UTF-8 -*-
-#   Copyright 2009-2021 Oli Schacher, Fumail Project
+#   Copyright 2009-2022 Oli Schacher, Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -43,58 +43,59 @@ CONN_EXC = tuple(CONN_EXC)
 
 
 class FuzorMixin(object):
+    requiredvars = {
+        'backend': {
+            'default': 'redis',
+            'description': 'storage backend to be used. currently supported: redis (check/report), kafka (report only)'
+        },
+        
+        'redis_conn': {
+            'default': 'redis://localhost:6379/0',
+            'description': 'redis config: redis://host:port/db',
+        },
+        'redis_password': {
+            'default': '',
+            'description': 'password to connect to redis database. leave empty for no password. deprecated, set password in redis_conn url',
+        },
+        'redis_ttl': {
+            'default': '604800',
+            'description': 'hash redis ttl in seconds',
+        },
+        
+        'kafkahosts': {
+            'default': '',
+            'description:': 'kafka bootstrap hosts: host1:port host2:port'
+        },
+        'kafkatopic': {
+            'default': 'fuzorhash',
+            'description': 'name of kafka topic'
+        },
+        'kafkausername': {
+            'default': '',
+            'description:': 'kafka sasl user name for this producer'
+        },
+        'kafkapassword': {
+            'default': '',
+            'description': 'kafka sasl password for this producer'
+        },
+        
+        'maxsize': {
+            'default': '600000',
+            'description':
+                'maxsize in bytes, larger messages will be skipped'
+        },
+        'stripoversize': {
+            'default': 'False',
+            'description':
+                'Remove attachments and reduce text to "maxsize" so large mails can be processed'
+        },
+        'timeout': {
+            'default': '2',
+            'description': 'timeout in seconds'
+        },
+    }
+    
     def __init__(self):
-        self.requiredvars = {
-            'backend': {
-                'default': 'redis',
-                'description': 'storage backend to be used. currently supported: redis (check/report), kafka (report only)'
-            },
-            
-            'redis_conn': {
-                'default': 'redis://localhost:6379/0',
-                'description': 'redis config: redis://host:port/db',
-            },
-            'redis_password': {
-                'default': '',
-                'description': 'password to connect to redis database. leave empty for no password. deprecated, set password in redis_conn url',
-            },
-            'redis_ttl': {
-                'default': '604800',
-                'description': 'hash redis ttl in seconds',
-            },
-            
-            'kafkahosts': {
-                'default': '',
-                'description:': 'kafka bootstrap hosts: host1:port host2:port'
-            },
-            'kafkatopic': {
-                'default': 'fuzorhash',
-                'description': 'name of kafka topic'
-            },
-            'kafkausername': {
-                'default': '',
-                'description:': 'kafka sasl user name for this producer'
-            },
-            'kafkapassword': {
-                'default': '',
-                'description': 'kafka sasl password for this producer'
-            },
-            
-            'maxsize': {
-                'default': '600000',
-                'description':
-                    'maxsize in bytes, larger messages will be skipped'
-            },
-            'stripoversize': {
-                'default': 'False',
-                'description':
-                    'Remove attachments and reduce text to "maxsize" so large mails can be processed'
-            },
-            'timeout': {
-                'default': '2',
-                'description': 'timeout in seconds'
-            },
-        }
         self.backend = None
         
         
@@ -106,6 +107,8 @@ class FuzorMixin(object):
                 self._init_backend_redis()
             elif backend == 'kafka':
                 self._init_backend_kafka()
+            else:
+                raise BackendError('backend=%s, error=invalid backend' % backend)
         except CONN_EXC as e:
             raise BackendError('backend=%s, error=%s' % (backend, str(e)))
         
@@ -284,6 +287,7 @@ class FuzorReport(ScannerPlugin, FuzorMixin):
     def __init__(self, config, section=None):
         ScannerPlugin.__init__(self, config, section)
         FuzorMixin.__init__(self)
+        self.requiredvars = FuzorMixin.requiredvars
         self.logger = self._logger()
     
     
@@ -311,6 +315,7 @@ class FuzorReportAppender(AppenderPlugin, FuzorMixin):
     def __init__(self, config, section=None):
         AppenderPlugin.__init__(self, config, section)
         FuzorMixin.__init__(self)
+        self.requiredvars = FuzorMixin.requiredvars
         self.logger = self._logger()
     
     
@@ -338,6 +343,7 @@ class FuzorCheck(ScannerPlugin, FuzorMixin):
         ScannerPlugin.__init__(self, config, section)
         FuzorMixin.__init__(self)
         self.logger = self._logger()
+        self.requiredvars = FuzorMixin.requiredvars
         self.requiredvars.update({
             'headername': {
                 'default': 'X-FuZor',
