@@ -1,4 +1,20 @@
-# -*- coding: UTF-8 -*-
+# -*- coding: utf-8 -*-
+#   Copyright 2009-2022 Fumail Project
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+# http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
+#
+#
 import re
 import os
 from fuglu.shared import ScannerPlugin, AppenderPlugin, DUNNO, Suspect, FileList, get_outgoing_helo
@@ -16,12 +32,13 @@ try:
     KAFKA_AVAILABLE=True
 except ImportError:
     KAFKA_AVAILABLE=False
-    
-    
-    
+
+
+
 class KnownSubjectMixin(object):
     config = None
     section = None
+    logger = None
     requiredvars = {
         'redis_conn': {
             'default': '',
@@ -157,12 +174,12 @@ class KnownSubjectMixin(object):
             for k, v in repl_map: # remove to_addr,  to_addr lhs and domain name from subject
                 s = s.replace(k, v)
 
-        s = re.sub('^((?:re|fwd?|aw|tr|sv)\W?:?\s?)+', '',  s) # strip re/fwd prefix
+        s = re.sub('^((?:re|fwd?|aw|tr|sv|rv)\W?:?\s?)+', '',  s) # strip re/fwd prefix
         s = re.sub("(?:^|\s|\b|\w)([0-9'.,-]{2,16})(?:\s|\b|\w|$)", 'N', s) # replace all number groups
         s = re.sub('[0-9]', 'X', s) # replace individual numbers
-        s = re.sub('\W', '', s, re.UNICODE) # remove all whitespaces and punctuations
-        s = re.sub('[^\x00-\x7F]','U', s) # replace non-ascii chars (note: unconfuse has already replaced special
-                                           #                               chars we care by an ascii representation
+        s = re.sub('[^\x00-\x7F]','U', s)  # replace non-ascii chars (note: unconfuse has already replaced special
+                                           #                                chars we care by an ascii representation)
+        s = re.sub('\W', '', s, re.UNICODE)  # remove all whitespaces and punctuations
         s = s[:32] # shorten to a maximum of 32 chars
         return s
     
@@ -255,7 +272,8 @@ class KnownSubject(ScannerPlugin, KnownSubjectMixin):
         ScannerPlugin.__init__(self,config,section)
         KnownSubjectMixin.__init__(self)
         self.logger = self._logger()
-        
+        self.requiredvars = KnownSubjectMixin.requiredvars
+    
     
     
     def examine(self,suspect):
