@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-#   Copyright 2009-2021 Fumail Project
+#   Copyright 2009-2022 Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -22,12 +22,14 @@ import time
 import asyncio
 import re
 from domainmagic.mailaddr import split_mail
+from domainmagic.validators import is_ipv6
 from fuglu.shared import ScannerPlugin, DUNNO, DELETE, SuspectFilter, AppenderPlugin, Suspect, get_outgoing_helo, apply_template, FileList
 from fuglu.stringencode import force_uString
 from fuglu.bounce import FugluSMTPClient, FugluAioSMTPClient, HAVE_AIOSMTP, SMTPException, Bounce
 from urllib.parse import urlparse
 from email.mime.multipart import MIMEMultipart
 from email.mime.text import MIMEText
+from fuglu.plugins.ratelimit.helperfuncs import get_ptr
 
 
 #TODO: reuse imap connections
@@ -310,7 +312,7 @@ class FeedList(FileList):
             for val in values[1:]:
                 k,v = val.split('=', 1)
                 k = k.lower()
-                if k in ['port', 'timeout']:
+                if k in ['port', 'timeout', 'retry']:
                     v = int(v)
                 elif k in ['tls', 'xclient']:
                     v = v.lower() in ['yes', 'true', '1', 'on']
@@ -395,7 +397,7 @@ class MailFeed(AppenderPlugin):
                 'default': '${confdir}/mailfeeds.txt',
                 'description': """file with feed targets.
                 format: target.server[;opt=val;opt=val] domain1 domain2 or * for all domains
-                possible options: port=587;tls=True;xclient=True;user=user;pass=pass;from=<>;to=*;timeout=30
+                possible options: port=587;tls=True;xclient=True;user=user;pass=pass;from=<>;to=*;timeout=30;retry=0
                 options tls, xclient, and from will override the global defaults defined in plugin config""",
             },
             
@@ -451,6 +453,23 @@ class MailFeed(AppenderPlugin):
         clientinfo = suspect.get_client_info(self.config)
         if clientinfo is not None:
             clienthelo, clientip, clienthostname = clientinfo
+            
+            if not clientip:
+                clientip = '[UNAVAILABLE]'
+            elif is_ipv6(clientip):
+                clientip = f'IPV6:{clientip}'
+            elif clienthostname == 'unknown': # we have a clientip but no fcrdns client hostname
+                clienthostname = suspect.get_tag('xclient-ptr')
+                if clienthostname is None:
+                    clienthostname = get_ptr(clientip, suspect)
+                    if clienthostname == 'unknown':
+                        clienthostname = '[TEMPUNAVAIL]'
+                    elif clienthostname == 'nxdomain':
+                        clienthostname = '[UNAVAILABLE]'
+                    suspect.set_tag('xclient-ptr', clienthostname)
+            if not clienthelo:
+                clienthelo = '[UNAVAILABLE]'
+                
             args['HELO'] = clienthelo
             args['ADDR'] = clientip
             args['NAME'] = clienthostname
@@ -478,10 +497,19 @@ class MailFeed(AppenderPlugin):
         # prepend header with original recipient
         original_recipient_header = self.config.get(self.section, 'original_recipient_header')
         if original_recipient_header:
-            msg_buffer = Suspect.prepend_header_to_source(original_recipient_header, suspect.from_address, msg_buffer)
+            msg_buffer = Suspect.prepend_header_to_source(original_recipient_header, suspect.to_address, msg_buffer)
         return msg_buffer
     
     
+    def _get_errcode_from_exc(self, exc, to_address):
+        errcode = 0
+        if isinstance(exc.args[0], dict):
+            smtperr = exc.args[0].get(to_address)
+            if smtperr:
+                errcode = smtperr[0]
+        return errcode
+    
+    
     def _send_sync(self, suspect: Suspect, target_host: str, target_args: dict, from_address: str, to_address: str, retry: int = 3):
         try:
             port = target_args.get('port', smtplib.SMTP_PORT)
@@ -503,6 +531,10 @@ class MailFeed(AppenderPlugin):
                     if xclient_args:
                         xclient_resp = smtp_server.xclient(xclient_args)
                         self._log_response('%s sent xclient args %s to %s' % (suspect.id, xclient_args, target_host), xclient_resp)
+                        xclient_helo = xclient_args.get('HELO')
+                        if xclient_helo and xclient_helo != '[UNAVAILABLE]':
+                            ehlo_resp = smtp_server.ehlo(xclient_helo)
+                            self._log_response('%s sent xclient ehlo %s to %s' % (suspect.id, xclient_helo, target_host), ehlo_resp)
                 except smtplib.SMTPNotSupportedError as e:
                     self.logger.error('%s xclient failed with target server %s due to: %s' % (suspect.id, target_host, str(e)))
             if use_auth:
@@ -521,8 +553,9 @@ class MailFeed(AppenderPlugin):
             except Exception as e:
                 self.logger.debug('%s error sending quit to %s: %s' % (suspect.id, target_host, str(e)))
         except Exception as e:
-            self.logger.error('%s failed to forward to %s due to %s' % (suspect.id, target_host, str(e)))
-            if retry > 0:
+            #self.logger.error('%s failed to forward to %s due to %s' % (suspect.id, target_host, str(e)))
+            errcode = self._get_errcode_from_exc(e, to_address)
+            if retry > 0 and not (500 < errcode < 599):
                 time.sleep((4-retry)/2)
                 fails = self._send_sync(suspect, target_host, target_args, from_address, to_address, retry=retry-1)
             else:
@@ -547,6 +580,10 @@ class MailFeed(AppenderPlugin):
                     if xclient_args:
                         xclient_resp = await smtp_server.xclient(xclient_args)
                         self._log_response('%s sent xclient args %s to %s' % (suspect.id, xclient_args, target_host), xclient_resp)
+                        xclient_helo = xclient_args.get('HELO')
+                        if xclient_helo and xclient_helo != '[UNAVAILABLE]':
+                            ehlo_resp = await smtp_server.ehlo(xclient_helo)
+                            self._log_response('%s sent xclient ehlo %s to %s' % (suspect.id, xclient_helo, target_host), ehlo_resp)
                 except SMTPException as e:
                     self.logger.error('%s xclient failed with target server %s due to: %s' % (suspect.id, target_host, str(e)))
             if use_auth:
@@ -575,7 +612,7 @@ class MailFeed(AppenderPlugin):
         return fails
     
     
-    def _send_mail(self, target_host: str, target_args: dict, suspect: Suspect, retry: int = 3):
+    def _send_mail(self, target_host: str, target_args: dict, suspect: Suspect):
         exceptions = target_args.get('exc', [])
         if suspect.to_domain in exceptions:
             self.logger.debug(f'{suspect.id} skipping sending to {target_host}')
@@ -598,6 +635,7 @@ class MailFeed(AppenderPlugin):
         if to_domain == get_outgoing_helo(self.config):
             to_address = f'{to_user}@{target_host}'
         
+        retry = target_args.get('retry', 3)
         disable_aiosmtp = self.config.getboolean('performance', 'disable_aiosmtp')
         if HAVE_AIOSMTP and not disable_aiosmtp:
             if self.event_loop is None:
@@ -606,14 +644,15 @@ class MailFeed(AppenderPlugin):
                 except AttributeError:
                     # python 3.6
                     self.event_loop = asyncio.get_event_loop()
-            fails = self.event_loop.run_until_complete(self._send_async(suspect, target_host, target_args, from_address, to_address, retry=retry-1))
+            fails = self.event_loop.run_until_complete(self._send_async(suspect, target_host, target_args, from_address, to_address, retry=retry))
         else:
-            fails = self._send_sync(suspect, target_host, target_args, from_address, to_address)
+            fails = self._send_sync(suspect, target_host, target_args, from_address, to_address, retry=retry)
         
         if not fails:
             self.logger.debug(f'{suspect.id} for {to_address} forwarded to {target_host}')
         else:
-            self.logger.error(f'{suspect.id} delivery failures to {target_host} for {fails}')
+            for rcpt in fails:
+                self.logger.error(f'{suspect.id} delivery failures to {target_host} for {rcpt} due to {fails[rcpt]}')
     
     
     def lint(self):
