@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-#   Copyright 2012-2021 Fumail Project
+#   Copyright 2012-2022 Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -26,7 +26,6 @@ import re
 from email import message_from_bytes
 from email.mime.multipart import MIMEMultipart
 from email.mime.text import MIMEText
-from domainmagic.mailaddr import strip_batv, decode_srs, email_normalise_ebl
 from fuglu.shared import strip_address, extract_domain, FileList, ScannerPlugin, DUNNO, apply_template, \
     string_to_actioncode, actioncode_to_string, get_outgoing_helo
 from fuglu.bounce import Bounce
@@ -39,6 +38,15 @@ import fuglu.connectors.asyncmilterconnector as asm
 import fuglu.connectors.milterconnector as sm
 from fuglu.logtools import createPIDinfo
 from .fuzor import FuzorMixin, FuzorDigest
+
+try:
+    from domainmagic.mailaddr import strip_batv, decode_srs, email_normalise_ebl
+    DOMAINMAGIC_AVAILABLE = True
+except ImportError:
+    def email_normalise_ebl(address, logger=None):
+        return address.lower()
+    DOMAINMAGIC_AVAILABLE=False
+    
 try:
     from pfqd.qstore import QStore
     from pfqd.qtools import queue_size, COL_FROM_DOM, COL_FROM_ADDR
@@ -269,8 +277,11 @@ class SenderDomainRules(BMPRCPTMixin, BasicMilterPlugin):
 
             sender = force_uString(sess.sender)
             if sender is not None:
-                from_address = strip_batv(strip_address(sender))
-                from_address = decode_srs(from_address)
+                if DOMAINMAGIC_AVAILABLE:
+                    from_address = strip_batv(strip_address(sender))
+                    from_address = decode_srs(from_address)
+                else:
+                    from_address = sender
                 from_domain = extract_domain(from_address)
             else:
                 from_address = None
@@ -703,19 +714,21 @@ class FuzorRateLimit(FuzorMixin, ScannerPlugin):
     is below threshold again.
     """
     def __init__(self, config, section=None):
-        ScannerPlugin.__init__(self, config, section)
         FuzorMixin.__init__(self)
+        ScannerPlugin.__init__(self, config, section)
         self.logger = self._logger()
-        
+        self.requiredvars = FuzorMixin.requiredvars
         self.requiredvars.update({
             'threshold': {
                 'default': '100',
                 'description': 'alert threshold'
             },
-            'sender_exception_file': {
+            'sender_exceptions_file': {
+                'default': '',
                 'description': 'file with senders that have a free pass to bulk'
             },
-            'alert_exception_file': {
+            'alert_exceptions_file': {
+                'default': '',
                 'description': 'file with senders that do not trigger an alert when bulking (but still get rate limited)'
             },
             'demomode': {
@@ -775,32 +788,44 @@ class FuzorRateLimit(FuzorMixin, ScannerPlugin):
             print('ERROR: config error')
             return False
         
-        sender_exception_file = self.config.get(self.section, 'sender_exception_file')
-        if not os.path.exists(sender_exception_file):
-            print('ERROR: cannot find sender_exception_file %s' % sender_exception_file)
+        if not FuzorMixin.lint(self):
+            return False
+        
+        sender_exception_file = self.config.get(self.section, 'sender_exceptions_file')
+        if sender_exception_file and not os.path.exists(sender_exception_file):
+            print('ERROR: cannot find sender_exceptions_file %s' % sender_exception_file)
             return False
         
         alert_exception_file = self.config.get(self.section, 'alert_exception_file')
-        if not os.path.exists(alert_exception_file):
-            print('ERROR: cannot find alert_exception_file %s' % alert_exception_file)
+        if alert_exception_file and not os.path.exists(alert_exception_file):
+            print('ERROR: cannot find alert_exceptions_file %s' % alert_exception_file)
             return False
         
         self._init_lists()
-        if self.sender_exceptions is None:
+        if sender_exception_file and self.sender_exceptions is None:
             print('ERROR: failed to initialise sender_exceptions')
             return False
         
+        try:
+            self._init_backend()
+            if self.backend is None:
+                print('ERROR: backend not initiated')
+                return False
+        except Exception as e:
+            print('ERROR failed to init backend: %s' % str(e))
+            return False
+        
         return True
     
     
     def _init_lists(self):
         if self.sender_exceptions is None:
-            sender_exception_file = self.config.get(self.section, 'sender_exception_file')
+            sender_exception_file = self.config.get(self.section, 'sender_exceptions_file')
             if sender_exception_file:
                 self.sender_exceptions = FileList(sender_exception_file, additional_filters=email_normalise_ebl)
         
         if self.alert_exceptions is None:
-            alert_exception_file = self.config.get(self.section, 'alert_exception_file')
+            alert_exception_file = self.config.get(self.section, 'alert_exceptions_file')
             if alert_exception_file:
                 self.alert_exceptions = FileList(alert_exception_file, additional_filters=email_normalise_ebl)
     
@@ -813,7 +838,7 @@ class FuzorRateLimit(FuzorMixin, ScannerPlugin):
             return DUNNO
         
         self._init_lists()
-        if email_normalise_ebl(suspect.from_address) in set(self.sender_exceptions.get_list()):
+        if self.sender_exceptions and email_normalise_ebl(suspect.from_address) in set(self.sender_exceptions.get_list()):
             return DUNNO
         
         digest, count = suspect.get_tag('FuZor', (None, 0))
@@ -845,6 +870,10 @@ class FuzorRateLimit(FuzorMixin, ScannerPlugin):
                     self.logger.error('failed to connect to redis server: %s' % str(e))
                     return DUNNO
                 
+                if self.backend is None:
+                    self.logger.warning('failed to initialise fuzor backend')
+                    return DUNNO
+                
                 attempts = 2
                 while attempts:
                     attempts -= 1
@@ -889,10 +918,12 @@ class FuzorRateLimit(FuzorMixin, ScannerPlugin):
                     # send mail if not already sent
                     # apply reject action
                     # ------
-                    alert_exception_list = set(self.alert_exceptions.get_list())
-                    if not email_normalise_ebl(suspect.from_address) in alert_exception_list and \
-                            not suspect.from_domain in alert_exception_list:
+                    if not self.alert_exceptions:
                         self._check_and_send_mail(suspect, digest, count)
+                    else:
+                        alert_exception_list = set(self.alert_exceptions.get_list())
+                        if not email_normalise_ebl(suspect.from_address) in alert_exception_list and not suspect.from_domain in alert_exception_list:
+                            self._check_and_send_mail(suspect, digest, count)
                     
                     actioncode = string_to_actioncode(self.config.get(self.section, 'actioncode'), self.config)
                     message = apply_template(self.config.get(self.section, 'rejectmessage'), suspect, {})
