@@ -1,4 +1,18 @@
 # -*- coding: UTF-8 -*-
+#   Copyright 2009-2022 Fumail Project
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+# http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
 # based on the ratelimit plugin in the postomaat project (https://gitlab.com/fumail/postomaat)
 # developed by @ledgr
 
@@ -9,13 +23,13 @@ import typing as tp
 import ipaddress as ia
 import asyncio
 import copy
+import time
 try:
     _ = tp.OrderedDict
 except AttributeError:
     # python 3.6 doesn't have a type for the OrderedDict, use normal Dict for typing
     tp.OrderedDict = tp.Dict
 
-import yaml
 from collections import OrderedDict
 
 from fuglu.shared import (
@@ -40,6 +54,12 @@ from .strategies import AVAILABLE_STRATEGIES, AVAILABLE_BACKENDS
 from .strategies.backendint import BackendInterface
 from .dynfunction import FunctionWrapperInt, MultipleFunctionsWrapper
 
+try:
+    import yaml
+    HAVE_YAML = True
+except ImportError:
+    yaml = None
+    HAVE_YAML = False
 
 ALLSTATES = list(BasicMilterPlugin.ALL_STATES.keys()) + [EOM]
 BE_VERBOSE = False # enable for debugging
@@ -90,13 +110,20 @@ class CounterInt(object):
 
         self.invert = bool(setupdict.get("invert"))
         self._lint = setupdict.get('lint', [])
-
+        
+        loggername = "%s.limiter.%s" % (__package__ if __package__ else "fuglu", self.__class__.__name__)
+        self.logger = logging.getLogger(loggername)
+    
+    
     def lint(self) -> int:
         """Lint counter rules if defined"""
 
         from fuglu.funkyconsole import FunkyConsole
         fc = FunkyConsole()
-
+        
+        if not HAVE_YAML:
+            print(fc.strcolor("ERROR", "red"), 'yaml module not found, this plugin will not do anything')
+            return False
 
         try:
             from mock import patch, MagicMock
@@ -104,8 +131,9 @@ class CounterInt(object):
             try:
                 from unittest.mock import patch, MagicMock
             except (ModuleNotFoundError, AttributeError):
-                print("LINT \"RateLimitPlugin -> Limiter -> Counter\" requires python package 'mock' or 'unittest")
-                return 1
+                print(fc.strcolor('WARNING', 'yellow'), f"LINT \"RateLimitPlugin -> Limiter -> Counter\" requires python package 'mock' or 'unittest' for proper linting capabilities")
+                MagicMock = None
+                patch = None
 
         try:
             from collections import defaultdict
@@ -113,7 +141,7 @@ class CounterInt(object):
             print("LINT \"RateLimitPlugin -> Limiter -> Counter\" requires python package 'collections'")
             return 1
 
-        if not self.fieldfunction:
+        if not self.fieldfunction and MagicMock is not None:
             # if there's no fieldfuncion, the attribute
             # should exist in the Session/Suspect object
             for tClass in self.targetSuspects:
@@ -159,39 +187,40 @@ class CounterInt(object):
                 err_count += 1
                 continue
 
-            with patch.object(self, '_get_field', return_value=input):
-                if templatedict is not None:
-                    d = templatedict
-                else:
-                    d = defaultdict(lambda: input)
-                with patch.object(self, 'get_suspect_dict', return_value=d):
-                    out_field = self.get_field(suspect=None)
-                    out_key = self.get_key(suspect=None)
-                    out_examine = self.examine(suspect=None)
+            if patch is not None:
+                with patch.object(self, '_get_field', return_value=input):
+                    if templatedict is not None:
+                        d = templatedict
+                    else:
+                        d = defaultdict(lambda: input)
+                    with patch.object(self, 'get_suspect_dict', return_value=d):
+                        out_field = self.get_field(suspect=None)
+                        out_key = self.get_key(suspect=None)
+                        out_examine = self.examine(suspect=None)
 
-            if fieldout is not None:
-                print("   ",
-                      fc.strcolor('PASS', 'green') if fieldout == out_field else fc.strcolor('FAIL', 'red'),
-                      f" Lint fieldout ({self.fieldname}): input={input}, field={out_field}, expected={fieldout}"
-                      )
-                if fieldout != out_field:
-                    err_count += 1
+                if fieldout is not None:
+                    print("   ",
+                          fc.strcolor('PASS', 'green') if fieldout == out_field else fc.strcolor('FAIL', 'red'),
+                          f" Lint fieldout ({self.fieldname}): input={input}, field={out_field}, expected={fieldout}"
+                          )
+                    if fieldout != out_field:
+                        err_count += 1
 
-            if keyout is not None:
-                print("   ",
-                      fc.strcolor('PASS', 'green') if keyout == out_key else fc.strcolor('FAIL', 'red'),
-                      f" Lint keyout ({self.fieldname}): input={input}, field={out_field}, key={out_key}, expected={keyout}"
-                      )
-                if keyout != out_key:
-                    err_count += 1
+                if keyout is not None:
+                    print("   ",
+                          fc.strcolor('PASS', 'green') if keyout == out_key else fc.strcolor('FAIL', 'red'),
+                          f" Lint keyout ({self.fieldname}): input={input}, field={out_field}, key={out_key}, expected={keyout}"
+                          )
+                    if keyout != out_key:
+                        err_count += 1
 
-            if examine is not None:
-                print("   ",
-                      fc.strcolor('PASS', 'green') if examine == out_examine else fc.strcolor('FAIL', 'red'),
-                      f" Lint examine ({self.fieldname}): input={input}, field={out_field}, examine={out_examine}, expected={examine}"
-                      )
-                if examine != out_examine:
-                    err_count += 1
+                if examine is not None:
+                    print("   ",
+                          fc.strcolor('PASS', 'green') if examine == out_examine else fc.strcolor('FAIL', 'red'),
+                          f" Lint examine ({self.fieldname}): input={input}, field={out_field}, examine={out_examine}, expected={examine}"
+                          )
+                    if examine != out_examine:
+                        err_count += 1
         return err_count
 
     def _invert(self, res: bool):
@@ -245,14 +274,18 @@ class CounterInt(object):
         elif self.ipmatch:
             # If there is one (or several) network ranges defined,
             # only count if ip is in net
-
-            # create ip
-            ip = ia.ip_address(field)
-
-            # check if ip is in net
-            for net in self.ipmatch:
-                if ip in net:
-                    return self._invert(True)
+            
+            try:
+                # create ip
+                ip = ia.ip_address(field)
+            except ValueError as e:
+                self.logger.error(str(e))
+            else:
+                # check if ip is in net
+                for net in self.ipmatch:
+                    if ip in net:
+                        return self._invert(True)
+            
             return self._invert(False)
         else:
             # default is True, always count
@@ -365,9 +398,8 @@ class Limiter(object):
             self.bounceaction = SKIP if action == "skip" else string_to_actioncode(bounceaction)
         else:
             self.bounceaction = self.action
-
-        myclass = self.__class__.__name__
-        loggername = "%s.limiter.%s" % (__package__ if __package__ else "fuglu", myclass)
+        
+        loggername = "%s.limiter.%s" % (__package__ if __package__ else "fuglu", self.__class__.__name__)
         self.logger = logging.getLogger(loggername)
 
         self.counters = OrderedDict()
@@ -551,6 +583,11 @@ class RateLimitPlugin(BMPConnectMixin, BMPHeloMixin, BMPMailFromMixin,
                 'default': ','.join(BasicMilterPlugin.ALL_STATES.keys()),
                 'description': f'comma/space separated list of milter states this plugin should be '
                                f'applied ({",".join(BasicMilterPlugin.ALL_STATES.keys())})'
+            },
+            
+            'timeout': {
+                'default': "10",
+                'description': 'Processing timeout, abort further lookups when exceeded'
             }
 
         }
@@ -646,7 +683,8 @@ class RateLimitPlugin(BMPConnectMixin, BMPHeloMixin, BMPMailFromMixin,
             if not catch_exceptions:
                 raise Exception(str(e)).with_traceback(e.__traceback__)
         return limiterdict
-
+    
+    
     def examine(self, suspect: Suspect):
         try:
             return asyncio.run(self.core(suspect=suspect, state=EOM))
@@ -654,10 +692,17 @@ class RateLimitPlugin(BMPConnectMixin, BMPHeloMixin, BMPMailFromMixin,
             self.logger.warning(f"EOM asyncio-run(EOM) plugin got error {str(e)} -> retrying")
 
         return asyncio.run(self.core(suspect=suspect, state=EOM))
-
+    
+    
     async def core(self, suspect: tp.Union[Suspect, asm.MilterSession, sm.MilterSession], state: str = EOM) -> tp.Union[int, tp.Tuple[int, str]]:
         """Main routine examining suspect, returncode from config if one limiter is exceeded"""
+        if not HAVE_YAML:
+            return DUNNO
+        
         self.logger.debug(f"{suspect.id} -> state={state} -> limiters to check {[lname for lname,l in self.limiters.items() if state in l.states and l.number ]}")
+        timeout = self.config.getfloat(self.section, 'timeout')
+        starttime= time.time()
+        errors = 0
         for lname, limiter in self.limiters.items():
             if BE_VERBOSE:
                 self.logger.debug(f"{suspect.id} state={state} check limiter \"{lname}\"")
@@ -694,6 +739,8 @@ class RateLimitPlugin(BMPConnectMixin, BMPHeloMixin, BMPMailFromMixin,
                     (allow, count) = await curbackend.check_allowed(eventname, limiter.number, timespan, increment)
                 else:
                     (allow, count) = curbackend.check_allowed(eventname, limiter.number, timespan, increment)
+                if count < 0:
+                    errors += 1
                 self.logger.debug(f'{suspect.id} {state}-Limiter event {eventname} (allow={allow},DUNNO={limiter.action == DUNNO}),SKIP={limiter.action == SKIP} count: {count}')
 
                 limiteraction = limiter.action if suspect.from_address else limiter.bounceaction
@@ -710,6 +757,7 @@ class RateLimitPlugin(BMPConnectMixin, BMPHeloMixin, BMPMailFromMixin,
                             template = _SuspectTemplate(limiter.message)
                         except Exception as e:
                             self.logger.debug(f'{suspect.id} {state}-Limiter: exception=\"{str(e)}\"', exc_info=e)
+                            template = _SuspectTemplate('')
 
                         try:
                             # python 3.9
@@ -734,7 +782,16 @@ class RateLimitPlugin(BMPConnectMixin, BMPHeloMixin, BMPMailFromMixin,
                 self.logger.error(f'{suspect.id} (state={state}) Failed to run limiter backend for strategy '
                                   f'"{limiter.strategy}" eventname {eventname} error {error}'
                                   )
-                self.logger.debug("{suspect.id} (state={state}) error traceback...", exc_info=ex)
+                self.logger.debug(f"{suspect.id} (state={state}) error traceback...", exc_info=ex)
+            
+            timediff = time.time() - starttime
+            if timediff > timeout:
+                self.logger.warning(f'{suspect.id} timeout exceeded after %.2fs, aborting further lookups, errors={errors}' % timediff)
+                break
+            elif errors > 3:
+                self.logger.warning(f'{suspect.id} error count exceeded after %.2fs, aborting further lookups, errors={errors}' % timediff)
+                break
+            
         return DUNNO
 
     @property
