@@ -1,10 +1,24 @@
 # -*- coding: UTF-8 -*-
+# -*- coding: UTF-8 -*-
+#   Copyright 2009-2022 Fumail Project
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+# http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
 # based on the ratelimit plugin in the postomaat project (https://gitlab.com/fumail/postomaat)
 # developed by @ledgr
 import time
 import math
 from collections import defaultdict
-from hashlib import md5
 from datetime import timedelta
 
 from .backendint import BackendInterface
@@ -15,8 +29,9 @@ try:
     import asyncio
     import aioredis
     AIOREDIS_AVAILABLE = 1
+    aioConnectionError = aioredis.exceptions.ConnectionError
 except ImportError:
-    pass
+    aioConnectionError = ConnectionError
 
 STRATEGY = 'sliding-window'
 BACKENDS = defaultdict(dict)
@@ -185,7 +200,6 @@ if REDIS_AVAILABLE:
                 event[b'mitigate'] = newval
 
                 self.logger.debug(f"{eventname} set mitigate flag, retry_after={retry_after}")
-                print(f"{eventname} set mitigate flag, retry_after={retry_after}")
                 self.update(eventname, event)
                 return False, count
 
@@ -216,14 +230,15 @@ if AIOREDIS_AVAILABLE:
                         # python 3.6
                         loop = asyncio.get_event_loop()
                     self.logger.debug(f"Got running loop...")
-                    self._redis = await aioredis.create_redis_pool(self.config, loop=loop)
+                    self._redis = await aioredis.create_redis_pool(self.config, loop=loop, timeout=3)
                 else:
                     # version >= 2
-                    self._pool = aioredis.BlockingConnectionPool().from_url(url=self.config)
+                    self._pool = aioredis.BlockingConnectionPool(timeout=3).from_url(url=self.config)
                     self._redis = await aioredis.StrictRedis(connection_pool=self._pool)
             return self._redis
 
         async def add(self, eventname, ttl=0):
+            success = False
             event_data = {
                 b'mitigate': 0,
                 b'bucket0': 0,
@@ -231,40 +246,111 @@ if AIOREDIS_AVAILABLE:
                 b'last_bucket': self.present_bucket,
                 b'bucket_start_ts': self.now
             }
-            r = await self.redis
-            try:
-                await r.hmset_dict(eventname, event_data)
-            except AttributeError:
-                # aioredis >= 2.0
-                await r.hset(eventname, mapping=event_data)
-            if isinstance(ttl, float):
-                ttl = timedelta(seconds=ttl)
-            if isinstance(ttl, timedelta):
-                ttl = int(ttl.seconds)
-            await r.expire(eventname, ttl)
+            attempts = 2
+            while attempts:
+                attempts -= 1
+                try:
+                    r = await self.redis
+                    try:
+                        await r.hmset_dict(eventname, event_data)
+                    except AttributeError:
+                        # aioredis >= 2.0
+                        await r.hset(eventname, mapping=event_data)
+                    if isinstance(ttl, float):
+                        ttl = timedelta(seconds=ttl)
+                    if isinstance(ttl, timedelta):
+                        ttl = int(ttl.seconds)
+                    await r.expire(eventname, ttl)
+                    success = True
+                except (ConnectionError, aioConnectionError) as e:
+                    self._redis = None
+                    if attempts:
+                        self.logger.warning(f"Connection error in 'add' - retry {str(e)}")
+                    else:
+                        self.logger.error(f"Connection error in 'add' {str(e)}")
+                except Exception as e:
+                    self._redis = None
+                    if attempts:
+                        self.logger.warning(f"Connection error in 'add' - retry: {str(e)}")
+                    else:
+                        self.logger.error(f"Connection error in 'add': {str(e)}", exc_info=e)
+            return success
 
         async def get_event(self, eventname):
-            r = await self.redis
-            return await r.hgetall(eventname)
+            event = None
+            success = False
+            attempts = 2
+            while attempts:
+                attempts -= 1
+                try:
+                    r = await self.redis
+                    event = await r.hgetall(eventname)
+                    success = True
+                except (ConnectionError, aioConnectionError) as e:
+                    self._redis = None
+                    if attempts:
+                        self.logger.warning(f"Connection error in 'get_event' - retry {str(e)}")
+                    else:
+                        self.logger.error(f"Connection error in 'get_event' {str(e)}")
+                except Exception as e:
+                    self._redis = None
+                    if attempts:
+                        self.logger.warning(f"Connection error in 'get_event' - retry: {str(e)}")
+                    else:
+                        self.logger.error(f"Connection error in 'get_event': {str(e)}", exc_info=e)
+            return event, success
 
         async def update(self, eventname, event_data):
-            r = await self.redis
-            try:
-                await r.hmset_dict(eventname, event_data)
-            except AttributeError:
-                # aioredis >= 2.0
-                await r.hset(eventname, mapping=event_data)
+            attempts = 2
+            while attempts:
+                attempts -= 1
+                try:
+                    r = await self.redis
+                    try:
+                        await r.hmset_dict(eventname, event_data)
+                    except AttributeError:
+                        # aioredis >= 2.0
+                        await r.hset(eventname, mapping=event_data)
+                except (ConnectionError, aioConnectionError) as e:
+                    self._redis = None
+                    if attempts:
+                        self.logger.warning(f"Connection error in 'update' - retry {str(e)}")
+                    else:
+                        self.logger.error(f"Connection error in 'update' {str(e)}")
+                except Exception as e:
+                    self._redis = None
+                    if attempts:
+                        self.logger.warning(f"Connection error in 'update' - retry: {str(e)}")
+                    else:
+                        self.logger.error(f"Connection error in 'update': {str(e)}", exc_info=e)
 
         async def set_mitigate(self, eventname, retry_after):
             newval = float(self.now) + float(retry_after)
             self.logger.debug(f"Setting mitigate to: {newval}")
-            r = await self.redis
-            event_data = {b"mitigate": newval}
-            try:
-                await r.hmset_dict(eventname, event_data)
-            except AttributeError:
-                # aioredis >= 2.0
-                await r.hset(eventname, mapping=event_data)
+
+            attempts = 2
+            while attempts:
+                attempts -= 1
+                try:
+                    r = await self.redis
+                    event_data = {b"mitigate": newval}
+                    try:
+                        await r.hmset_dict(eventname, event_data)
+                    except AttributeError:
+                        # aioredis >= 2.0
+                        await r.hset(eventname, mapping=event_data)
+                except (ConnectionError, aioConnectionError) as e:
+                    self._redis = None
+                    if attempts:
+                        self.logger.warning(f"Connection error in 'set_mitigate' - retry {str(e)}")
+                    else:
+                        self.logger.error(f"Connection error in 'set_mitigate' {str(e)}")
+                except Exception as e:
+                    self._redis = None
+                    if attempts:
+                        self.logger.warning(f"Connection error in 'set_mitigate' - retry: {str(e)}")
+                    else:
+                        self.logger.error(f"Connection error in 'set_mitigate': {str(e)}", exc_info=e)
 
         def get_buckets(self, timespan):
             """get time buckets where counters are saved
@@ -309,10 +395,19 @@ if AIOREDIS_AVAILABLE:
             self.present_bucket, self.past_bucket = self.get_buckets(timespan)
             count = -1 # not calculated yet or mitigation is on
 
-            event = await self.get_event(eventname)
+            event, success = await self.get_event(eventname)
+            if success and (not event or not b'bucket_start_ts' in event):
+                success = await self.add(eventname, ttl=timespan * 3)
+                if success:
+                    event, success = await self.get_event(eventname)
+
             if not event:
-                await self.add(eventname, ttl=timespan * 3)
-                event = await self.get_event(eventname)
+                self.logger.warning(f'{eventname} failed to get event, bailing out')
+                return True, count
+            
+            if not b'bucket_start_ts' in event:
+                self.logger.warning(f'{eventname} event is missing bucket_start_ts, bailing out')
+                return True, count
 
             # we are ahead of both bucket timespans
             # so the counters are irrelevant and must be reset
@@ -322,8 +417,10 @@ if AIOREDIS_AVAILABLE:
             if self.present_bucket != event[b'last_bucket']:
                 self.change_bucket(event)
                 self.reset_bucket(event, self.present_bucket)
-                if isinstance(timespan, float):
+                if isinstance(timespan, (int, float)):
                     timespan_timedelta = timedelta(seconds=timespan)
+                else:
+                    timespan_timedelta = timedelta(seconds=0)
                 if isinstance(timespan_timedelta, timedelta):
                     timespan_timedelta = int(timespan_timedelta.seconds)
                 await (await self.redis).expire(eventname, timespan_timedelta * 3)
