@@ -1,4 +1,4 @@
-#   Copyright 2009-2021 Fumail Project
+#   Copyright 2009-2022 Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -24,13 +24,15 @@ except AttributeError:
     tp.OrderedDict = tp.Dict
 
 import re
-import yaml
 import logging
 from collections import defaultdict
 try:
     from mock import MagicMock
 except ModuleNotFoundError:
-    from unittest.mock import MagicMock
+    try:
+        from unittest.mock import MagicMock
+    except ModuleNotFoundError:
+        MagicMock = None
 
 from domainmagic.rbl import RBLLookup
 from domainmagic.tld import TLDMagic
@@ -51,7 +53,14 @@ from fuglu.mshared import (
 from fuglu.stringencode import force_uString
 from fuglu.plugins.ratelimit.dynfunction import MultipleFunctionsWrapper
 from fuglu.plugins.ratelimit.main import TemplateFunctionWrapper
-from fuglu.extensions.dnsquery import revlookup
+from fuglu.plugins.ratelimit.helperfuncs import get_ptr as get_ptr_helper
+
+try:
+    import yaml
+    HAVE_YAML = True
+except ImportError:
+    yaml = None
+    HAVE_YAML = False
 
 BE_VERBOSE = False
 regexrule = re.compile(r"^/(?P<regex>.{1,200}[^\\])\/(?P<flag>[a-z])?[ \t]{1,150}((?P<astring>[A-Z]{1,8})|(?P<acode>[0-9]{3}))([ \t]{1,150}(?P<msg>.{1,100}))?$")
@@ -59,7 +68,10 @@ ifregex = re.compile(r"if /(?P<regex>.{1,200}[^\\])\/(?P<flag>[a-z])?[ \t]{0,150
 hashrule = re.compile(r"^(?P<key>\S{1,200})[ \t]{1,150}((?P<astring>[A-Z]{1,8})|(?P<acode>[0-9]{3}))([ \t]{1,150}(?P<msg>.{1,100}))?$")
 
 
-class CreateSubdomainListMixin:
+RESTRICTIONS_TAG = 'AccessRestrictionResults'
+
+
+class CreateSubdomainListMixin(object):
     def __init__(self, **kwargs):
         self.tldmagic = TLDMagic()
         super().__init__(**kwargs)
@@ -85,7 +97,7 @@ class CreateSubdomainListMixin:
         return checkstrings
 
 
-class MatchRestriction:
+class MatchRestriction(object):
     def __init__(self,
                  actionstring: str = "REJECT",
                  actioncode: tp.Optional[int] = None,
@@ -103,7 +115,7 @@ class MatchRestriction:
 
     def examine(self, inputstring: str) -> tp.Tuple:
         if self.match(inputstring):
-            return (self.actioncode if self.actioncode else self.milteraction, self.message)
+            return self.actioncode if self.actioncode else self.milteraction, self.message
         else:
             return ()
 
@@ -180,7 +192,6 @@ class RegexRestriction(MatchRestriction):
         else:
             raise ValueError("Either both regexstring and flags are array or both must be string")
 
-
     def match(self, inputstring: str) -> bool:
         if isinstance(self.regex, list):
             return all(rgx.search(inputstring) for rgx in self.regex)
@@ -227,7 +238,7 @@ class HashCore(MatchRestriction):
         if hashmatch:
             hashcore: HashCore
             hashmatch: str
-            return (self.actioncode if self.actioncode else self.milteraction, self.message, hashmatch)
+            return self.actioncode if self.actioncode else self.milteraction, self.message, hashmatch
         else:
             return ()
 
@@ -315,7 +326,7 @@ class HashTable(MatchRestriction):
             hashcore, hashmatch = match
             hashcore: HashCore
             hashmatch: str
-            return (hashcore.actioncode if hashcore.actioncode else hashcore.milteraction, hashcore.message, hashmatch)
+            return hashcore.actioncode if hashcore.actioncode else hashcore.milteraction, hashcore.message, hashmatch
         else:
             return ()
 
@@ -412,14 +423,13 @@ class RBLRestriction(MatchRestriction):
                     template = _SuspectTemplate(self.message)
                     output = {'output': msg}
                     msg = template.safe_substitute(output)
-                return (self.actioncode if self.actioncode else self.milteraction, msg)
-
+                return self.actioncode if self.actioncode else self.milteraction, msg
         return ()
 
     def _str_(self) -> str:
         return f"{self.__class__.__name__}({self.providertype}/{self.searchdomain})"
 
-class RestrictionSet:
+class RestrictionSet(object):
     # default variables to test in the stages if not given differently
     DEFAULT_STAGE_ATTRS = {
         sm.CONNECT: "addr",
@@ -512,19 +522,8 @@ class AccessRestrictions(BMPConnectMixin, BMPHeloMixin, BMPMailFromMixin, BMPRCP
     DELAY_REJ_STATE = 2
     DELAY_REJ_AFTER = 3
     DELAY_REJ_TAG = 'AccessRestrictions.delayedreject'
-
-    @staticmethod
-    def get_ptr(ipaddress: str) -> tp.Optional[str]:
-        """get ptr, return 'unknown' if none is found to match MilterSession implementation"""
-        try:
-            answers: tp.List[str] = revlookup(ipaddress)
-            answers = [a.rstrip('.') for a in answers if a and a.rstrip('.')] if answers else []
-            if len(answers):
-                return answers[0]
-        except Exception as e:
-            logger = logging.getLogger('fuglu.AccessRestrictions.get_ptr')
-            logger.error(f"got: {str(e)} querying PTR for: {ipaddress}")
-        return "unknown"
+    
+    get_ptr = get_ptr_helper # define here for easy monkey patching in unit tests
 
 
     @staticmethod
@@ -620,7 +619,7 @@ class AccessRestrictions(BMPConnectMixin, BMPHeloMixin, BMPMailFromMixin, BMPRCP
             else:
                 raise Exception(str(e)).with_traceback(e.__traceback__)
 
-        restrictions = configdict.get("restrictions")
+        restrictions = configdict.get("restrictions", {})
         if not restrictions:
             self.logger.warning(f"No restrictions to load defined in {filename}, plugin will not do anything")
             return restrictions_sets, setupdict
@@ -632,23 +631,19 @@ class AccessRestrictions(BMPConnectMixin, BMPHeloMixin, BMPMailFromMixin, BMPRCP
             else:
                 raise ValueError(f"No setup to load defined in {filename}!")
             return restrictions_sets, setupdict
-
-        try:
-            force_state = self.config.get(self.section, 'force_state')
-        except Exception as e:
-            self.logger.debug(f"Got error trying to read 'force_state' from config: {str(e)}")
-        else:
-            if force_state:
-                oldsetup = setup
-                setup = {force_state: []}
-                for v in oldsetup.values():
-                    # vis an array of dicts defining where a restriction is applied
-                    # -> check if dict is already in array, add only if new because they are now all
-                    #     in the same state
-                    for definition in v:
-                        if definition not in setup[force_state]:
-                            setup[force_state].append(definition)
-                self.logger.debug(f"Forced setup states to force_state={force_state}")
+        
+        force_state = self.config.get(self.section, 'force_state', fallback=None)
+        if force_state:
+            oldsetup = setup
+            setup = {force_state: []}
+            for v in oldsetup.values():
+                # vis an array of dicts defining where a restriction is applied
+                # -> check if dict is already in array, add only if new because they are now all
+                #     in the same state
+                for definition in v:
+                    if definition not in setup[force_state]:
+                        setup[force_state].append(definition)
+            self.logger.debug(f"Forced setup states to force_state={force_state}")
 
         if not isinstance(restrictions, list):
             self.logger.debug(f"Only one restriction, set as array")
@@ -712,9 +707,12 @@ class AccessRestrictions(BMPConnectMixin, BMPHeloMixin, BMPMailFromMixin, BMPRCP
                 elif state== asm.EOH:
                     inputs_possible = ["headers"]
                 else:
-                    inputs_possible = list(asm.MilterSession(MagicMock(), MagicMock()).get_templ_dict(setall=True).keys())
+                    if MagicMock:
+                        inputs_possible = list(asm.MilterSession(MagicMock(), MagicMock()).get_templ_dict(setall=True).keys())
+                    else:
+                        inputs_possible = None
 
-                if input is not None and input not in inputs_possible:
+                if input is not None and inputs_possible is not None and input not in inputs_possible:
                     if ignoreerrors:
                         self.logger.warning(f"State:{state} -> Restrictionset input:{input} not available: {inputs_possible} -> ignoring")
                         continue
@@ -791,12 +789,15 @@ class AccessRestrictions(BMPConnectMixin, BMPHeloMixin, BMPMailFromMixin, BMPRCP
              - run plugin(mode:REJECT_DIRECTLY)
         - delay_rejects is disabled, run plugin(mode:REJECT_DIRECTLY)
         """
+        if not HAVE_YAML:
+            return sm.CONTINUE
+        
         position = self.delay_rejects_statedict.get(state, AccessRestrictions.DELAY_REJ_AFTER)
         previous_reject_tag = sess.tags.get(self.DELAY_REJ_TAG)
 
         if previous_reject_tag:
             if position == AccessRestrictions.DELAY_REJ_BEFORE:
-                self.logger.debug(f"state:{state} -> reject flag is set, skip tests for a later reject")
+                self.logger.debug(f"{sess.id} state:{state} -> reject flag is set, skip tests for a later reject")
                 return sm.CONTINUE
             else:
                 # it's time to reject
@@ -856,6 +857,10 @@ class AccessRestrictions(BMPConnectMixin, BMPHeloMixin, BMPMailFromMixin, BMPRCP
                         else:
                             msg = None
                         self.logger.debug(f"Test:{rinput} with input:{checkstring} for Restriction({rname}): {res}")
+                        try:
+                            sess.tags[RESTRICTIONS_TAG][rname] = res
+                        except KeyError as e:
+                            sess.tags[RESTRICTIONS_TAG] = dict(rname = res)
 
                         # --- #
                         # hit #
@@ -915,6 +920,8 @@ class AccessRestrictions(BMPConnectMixin, BMPHeloMixin, BMPMailFromMixin, BMPRCP
         return self._examine_general(sess=sess, state=asm.EOH, inputstring=(sess.original_headers, ))
 
     def examine(self, suspect: Suspect):
+        if not HAVE_YAML:
+            return DUNNO
         msg = suspect.get_message_rep()
         # if there is a trigger header defined, check if it is set and run checks only if present
         #
@@ -948,6 +955,10 @@ class AccessRestrictions(BMPConnectMixin, BMPHeloMixin, BMPMailFromMixin, BMPRCP
             return True
 
         fc = FunkyConsole()
+        if not HAVE_YAML:
+            print(fc.strcolor("ERROR", "red"), 'yaml module not found, this plugin will not do anything')
+            return False
+        
         check_config = super().lint()
         if not check_config:
             print(fc.strcolor("ERROR - config check", "red"))
