@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*- 
-#   Copyright 2009-2021 Oli Schacher, Fumail Project
+#   Copyright 2009-2022 Oli Schacher, Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -329,10 +329,10 @@ Tags:
         if spamheader is None:
             self.logger.warning('%s Did not find Header %s in returned message from SA' % (suspect.id, spamheadername))
         else:
-            if re.match(r"""^YES""", spamheader.strip(), re.IGNORECASE) is not None:
+            if re.match(r'^YES', spamheader.strip(), re.IGNORECASE) is not None:
                 isspam = True
 
-            patt = re.compile('Score=([\-\d.]{1,10})', re.IGNORECASE)
+            patt = re.compile(r'Score=([\-\d.]{1,10})', re.IGNORECASE)
             m = patt.search(spamheader)
 
             if m is not None:
@@ -539,20 +539,26 @@ Tags:
             self.logger.debug('%s Message is not spam' % suspect.id)
             suspect.debug('Message is not spam')
         
-        ishighspam = False
         if spamscore is not None:
             suspect.tags['SAPlugin.spamscore'] = spamscore # deprecated
-            highspamlevel = runtimeconfig.getfloat(self.section, 'highspamlevel')
-            if spamscore >= highspamlevel:
-                ishighspam = True
-                configaction = string_to_actioncode(runtimeconfig.get(self.section, 'highspamaction'), self.config)
-                if configaction is not None:
-                    action = configaction
+        ishighspam = self._is_highspam(runtimeconfig, spamscore)
+        if ishighspam:
+            configaction = string_to_actioncode(runtimeconfig.get(self.section, 'highspamaction'), self.config)
+            if configaction is not None:
+                action = configaction
         
         self._spamreport(suspect, isspam, ishighspam, report, spamscore, enginename='SpamAssassin')
         return action, message
-
-
+    
+    
+    def _is_highspam(self, runtimeconfig, spamscore):
+        if spamscore is not None:
+            highspamlevel = runtimeconfig.getfloat(self.section, 'highspamlevel')
+            if spamscore >= highspamlevel:
+                return True
+        return False
+    
+    
     def __init_socket(self):
         unixsocket = False
 
@@ -958,7 +964,7 @@ Tags:
     
     
     def lint(self):
-        allok = self.check_config() and self.lint_ping()
+        allok = self.check_config() and self._lint_ping()
         if allok:
             databases = self._get_databases()
             allok = 1<=len(databases)<=2
