@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-#   Copyright 2012-2021 Fumail Project
+#   Copyright 2012-2022 Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -35,12 +35,13 @@ class EnforceTLS(BMPRCPTMixin, BasicMilterPlugin):
         self.logger = self._logger()
         self.selective_domain_loader = None
         self.requiredvars = {
-            'domainlist': {
+            'datasource': {
                 'default': '',
                 'description': """
                 if this is empty, all recipient domains will be forced to use TLS
                 txt:<filename> - get from simple textfile which lists one domain per line
-                sql:<statement> - get from sql database :domain will be replaced with the actual domain name. must return field enforce_inbound_tls
+                sql:<statement> - get from sql database :domain will be replaced with the actual domain name. must return one field containing boolean/int value
+                tag:filtersettings:enforce_tls - get from p_blwl FilterSettings backend tag
                 """,
             },
             'dbconnection': {
@@ -63,39 +64,46 @@ class EnforceTLS(BMPRCPTMixin, BasicMilterPlugin):
         }
     
     
-    def enforce_domain(self, to_domain: str) -> bool:
+    def enforce_domain(self, sess: tp.Union[asm.MilterSession, sm.MilterSession], to_domain: str) -> bool:
         dbconnection = self.config.get(self.section, 'dbconnection').strip()
-        domainlist = self.config.get(self.section, 'domainlist')
+        datasource = self.config.get(self.section, 'datasource')
         enforce = False
         
-        if domainlist.strip() == '':
+        if datasource.strip() == '':
             enforce = True
         
-        elif domainlist.startswith('txt:'):
-            domainfile = domainlist[4:]
+        elif datasource.startswith('txt:'):
+            domainfile = datasource[4:]
             if self.selective_domain_loader is None:
                 self.selective_domain_loader = FileList(domainfile, lowercase=True)
                 if to_domain in self.selective_domain_loader.get_list():
                     enforce = True
-
+        
         # use DBConfig instead of get_domain_setting
-        elif domainlist.startswith('sql:') and dbconnection != '':
+        elif SQL_EXTENSION_ENABLED and datasource.startswith('sql:') and dbconnection != '':
             cache = get_default_cache()
-            sqlquery = domainlist[4:]
+            sqlquery = datasource[4:]
             enforce = get_domain_setting(to_domain, dbconnection, sqlquery, cache, self.section, False, self.logger)
         
+        elif datasource.startswith('tag:'):
+            keylist = datasource.split(':')[1:]
+            value = sess.tags
+            for key in keylist:
+                value = value.get(key, {})
+                if isinstance(value, bool):
+                    enforce = value
+        
         return enforce
     
     
-    def examine_rcpt(self, sess: tp.Union[asm.MilterSession, sm.MilterSession], recipient: bytes) -> tp.Union[
-        bytes, tp.Tuple[bytes, str]]:
+    def examine_rcpt(self, sess: tp.Union[asm.MilterSession, sm.MilterSession], recipient: bytes) -> tp.Union[bytes, tp.Tuple[bytes, str]]:
         """Use asnyc routine because tls info needs async patched milter lib"""
         encryption_protocol = sess.tls_version
         if not recipient:
             return sm.CONTINUE
         
         recipient = force_uString(recipient)
-        enforce = self.enforce_domain(sm.MilterSession.extract_domain(recipient))
+        enforce = self.enforce_domain(sess, sm.MilterSession.extract_domain(recipient))
         
         action = sm.CONTINUE
         message = None
@@ -127,16 +135,16 @@ class EnforceTLS(BMPRCPTMixin, BasicMilterPlugin):
                   f"choices: 'continue', 'tempfail', 'reject'")
         
         if lint_ok:
-            domainlist = self.config.get(self.section, 'domainlist')
-            if domainlist.strip() == '':
+            datasource = self.config.get(self.section, 'datasource')
+            if datasource.strip() == '':
                 print('Enforcing TLS for all domains')
-            elif domainlist.startswith('txt:'):
-                domainfile = domainlist[4:]
+            elif datasource.startswith('txt:'):
+                domainfile = datasource[4:]
                 if not os.path.exists(domainfile):
                     print('Cannot find domain file %s' % domainfile)
                     lint_ok = False
-            elif domainlist.startswith('sql:'):
-                sqlquery = domainlist[4:]
+            elif datasource.startswith('sql:'):
+                sqlquery = datasource[4:]
                 if not sqlquery.lower().startswith('select '):
                     lint_ok = False
                     print('SQL statement must be a SELECT query')
@@ -150,6 +158,8 @@ class EnforceTLS(BMPRCPTMixin, BasicMilterPlugin):
                     except Exception as e:
                         lint_ok = False
                         print(str(e))
+            elif datasource.startswith('tag:'):
+                pass
             else:
                 lint_ok = False
                 print('Could not determine domain list backend type')
@@ -226,7 +236,7 @@ class TLSSender(BMPRCPTMixin, BasicMilterPlugin):
             template = _SuspectTemplate(self.config.get(self.section, 'messagetemplate'))
             message = template.safe_substitute(sess.get_templ_dict())
         self.logger.debug(f"{sess.id} sender: {sender}, domain: {sender_domain}, enforce: {enforce}, "
-                          f"encprot: {encryption_protocol}, action: {sm.RETCODE2STR[action]}")
+                          f"encprot: {force_uString(encryption_protocol)}, action: {sm.RETCODE2STR[action]}")
         return action, message
     
     
