@@ -1,4 +1,4 @@
-#   Copyright 2009-2021 Oli Schacher, Fumail Project
+#   Copyright 2009-2022 Oli Schacher, Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -146,7 +146,7 @@ class BasicTCPServer(object):
         # example: when sending SIGHUP which might recreate the processor pool or threads pool
         #          which would then still point to the wrong (old) memory location and is therefore not served anymore
 
-        threading.currentThread().name = '%s Server on Port %s' % (
+        threading.current_thread().name = '%s Server on Port %s' % (
             self.protohandlerclass.protoname, self.port)
 
         self.logger.info('%s Server running on port %s' %
@@ -175,6 +175,11 @@ class BasicTCPServer(object):
                                             self.controller.plugins, self.controller.appenders, self.port,
                                             self.controller.milterdict)
                     engine.handlesession()
+            except ConnectionAbortedError as e:
+                if self.stayalive:
+                   self.logger.warning('Connection aborted - break and setting stayalive to False')
+                   self.stayalive = False
+                break
             except Exception as e:
                 exc = traceback.format_exc()
                 self.logger.error('Exception in serve(): %s - %s' % (str(e), exc))
