@@ -1,4 +1,4 @@
-#   Copyright 2009-2021 Oli Schacher, Fumail Project #
+#   Copyright 2009-2022 Oli Schacher, Fumail Project #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
@@ -25,7 +25,7 @@ from functools import reduce
 import typing as tp
 
 from fuglu.stats import Statskeeper, StatDelta
-from fuglu.shared import DUNNO, ACCEPT, REJECT, DEFER, DELETE, Suspect
+from fuglu.shared import DUNNO, ACCEPT, REJECT, DEFER, DELETE
 from fuglu.connectors.check import HealthCheckSuspect
 from fuglu.debug import CrashStore
 from fuglu.stringencode import force_uString
@@ -266,7 +266,7 @@ class SessionHandler(TrackTimings):
 
         try:
             self.enabletimetracker = config.getboolean('main', 'scantimelogger')
-        except Exception as e:
+        except Exception:
             self.enabletimetracker = False
 
         self.logger.debug("enabletimetracker = %s" % self.enabletimetracker)
@@ -290,18 +290,19 @@ class SessionHandler(TrackTimings):
 
         keep_connection = True
         isuspect = 0  # in case connection is reused (milter) keep track of suspect for workerstate
-
+        message_prefix = 'undef'
+        
         while keep_connection:
+            suspect = None
             try:
                 isuspect += 1
-                suspect = None
                 # reset variables to make sure per-message variables are re-initialised in case of handling
                 # multiple messages in one session
                 self.resetvars()
                 
-                message_prefix = u"(#%u)" % isuspect  # identifier (receiving multiple suspects in one session)
+                message_prefix = "(#%u)" % isuspect  # identifier (receiving multiple suspects in one session)
 
-                self.set_workerstate(message_prefix+u'receiving message')
+                self.set_workerstate(message_prefix+'receiving message')
                 self.tracktime("SessionHandler-Setup")
                 suspect = self.protohandler.get_suspect(milterplugins=self.milterplugins)
                 if suspect is None:
@@ -315,7 +316,7 @@ class SessionHandler(TrackTimings):
                         pass
                     return
                 elif isinstance(suspect, HealthCheckSuspect):
-                    self.logger.debug(message_prefix+u'Health Check Suspect retrieved')
+                    self.logger.debug(message_prefix+'Health Check Suspect retrieved')
                     self.protohandler.healthcheck_reply()
                     self.resetvars()
                     return
@@ -324,11 +325,11 @@ class SessionHandler(TrackTimings):
                 self.stats.increase_counter_values(StatDelta(in_=1))
 
                 if len(suspect.recipients) != 1:
-                    self.logger.warning(message_prefix+u'Notice: Message from %s has %s recipients. Plugins supporting only one recipient will see: %s' % (
+                    self.logger.warning(message_prefix+'Notice: Message from %s has %s recipients. Plugins supporting only one recipient will see: %s' % (
                         suspect.from_address, len(suspect.recipients), suspect.to_address))
-                self.logger.debug(message_prefix+u"Message from %s to %s: %s bytes stored to %s" % (
+                self.logger.debug(message_prefix+"Message from %s to %s: %s bytes stored to %s" % (
                     suspect.from_address, suspect.to_address, suspect.size, suspect.tempfile))
-                self.set_workerstate(message_prefix+u"Handling message %s" % suspect)
+                self.set_workerstate(message_prefix+"Handling message %s" % suspect)
                 # store incoming port to tag, could be used to disable plugins
                 # based on port
                 try:
@@ -336,7 +337,7 @@ class SessionHandler(TrackTimings):
                     if port is not None:
                         suspect.tags['incomingport'] = port
                 except Exception as e:
-                    self.logger.warning(message_prefix+u'Could not get incoming port: %s' % str(e))
+                    self.logger.warning(message_prefix+'Could not get incoming port: %s' % str(e))
 
                 pluglist, applist = self.run_prependers(suspect)
 
@@ -374,7 +375,7 @@ class SessionHandler(TrackTimings):
                 # check if one of the plugins made a decision
                 result = self.action
 
-                self.set_workerstate(message_prefix+u"Finishing message %s" % suspect)
+                self.set_workerstate(message_prefix+"Finishing message %s" % suspect)
 
                 message_is_deferred = False
                 if result == ACCEPT or result == DUNNO:
@@ -398,11 +399,17 @@ class SessionHandler(TrackTimings):
                     retmesg = 'OK: (%s)' % suspect.id
                     if self.message is not None:
                         retmesg = self.message
+                    if suspect.id not in retmesg:
+                        # make sure fugluid is in return message
+                        retmesg = f"{retmesg} ({suspect.id})"
                     self.protohandler.discard(retmesg)
                 elif result == REJECT:
                     retmesg = "Rejected by content scanner"
                     if self.message is not None:
                         retmesg = self.message
+                    if suspect.id not in retmesg:
+                        # make sure fugluid is in return message
+                        retmesg = f"{retmesg} ({suspect.id})"
                     retmesg = "%s (%s)" % (retmesg, suspect.id)
                     self.protohandler.reject(retmesg)
                 elif result == DEFER:
@@ -426,17 +433,17 @@ class SessionHandler(TrackTimings):
                 if suspect.tempfilename() != "(buffer-only)":
                     try:
                         os.remove(suspect.tempfile)
-                        self.logger.debug(message_prefix+u'Removed tempfile %s' % (suspect.tempfile if suspect.tempfile else "(not available)"))
+                        self.logger.debug(message_prefix+'Removed tempfile %s' % (suspect.tempfile if suspect.tempfile else "(not available)"))
                         suspect.tempfile = None
                         self.tracktime("Remove-tempfile")
                     except OSError:
-                        self.logger.warning(message_prefix+u'Could not remove tempfile %s' % suspect.tempfile)
+                        self.logger.warning(message_prefix+'Could not remove tempfile %s' % suspect.tempfile)
             except KeyboardInterrupt:
                 sys.exit(0)
             except ValueError as e:
                 # Error in envelope send/receive address
                 try:
-                    self.logger.warning(message_prefix+u"Invalid send/receive address -> %s" % force_uString(e))
+                    self.logger.warning(message_prefix+"Invalid send/receive address -> %s" % force_uString(e))
                 except Exception:
                     pass
 
@@ -507,9 +514,8 @@ class SessionHandler(TrackTimings):
                     self.logger.debug(message_prefix+u'Remove suspect (current refs): %u' % sys.getrefcount(suspect))
                     suspectid = suspect.id
                     del suspect
-                except Exception as e:
+                except Exception:
                     suspectid = "unknown"
-                    pass
 
                 # ---------------#
                 # report timings #
@@ -667,7 +673,7 @@ class SessionHandler(TrackTimings):
             except Exception as e:
                 CrashStore.store_exception()
                 exc = traceback.format_exc()
-                self.logger.error('Plugin %s failed: %s' % (str(plugin), exc))
+                self.logger.error('%s Plugin %s failed: %s' % (suspect.id, str(plugin), exc))
                 suspect.debug('Plugin failed : %s . Please check fuglu log for more details' % e)
                 ptag = suspect.get_tag("processingerrors", defaultvalue=[])
                 ptag.append("Plugin %s failed: %s" % (str(plugin), str(e)))
