@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-#   Copyright 2009-2021 Fumail Project
+#   Copyright 2009-2022 Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -93,8 +93,8 @@ def try_decoding(b_inputstring, encodingGuess="utf-8", errors="strict"):
         if seconderrors and str(e).endswith('invalid continuation byte'):
             try:
                 u_outputstring = b_inputstring.decode(encodingGuess, errors=seconderrors)
-            except (UnicodeDecodeError, LookupError) as e:
-                pass
+            except (UnicodeDecodeError, LookupError) as f:
+                e = f
         if not u_outputstring:
             # if we get here we will also print either the chardet or trial&error decoding message anyway
             logger.debug("found non %s encoding or encoding not found (msg: %s), try to detect encoding"
@@ -158,10 +158,10 @@ def force_uString(inputstring, encodingGuess="utf-8", errors="strict", convert_n
         # expected to be converted to unicode
         
         logger = logging.getLogger("fuglu.force_uString")
-        if isinstance(inputstring, int):
+        if isinstance(inputstring, (int, float)):
             pass
         elif not isinstance(inputstring, (str, bytes)):
-            logger.debug("object is not string/unicode/bytes/int but %s" % str(type(inputstring)))
+            logger.debug("object is not string/bytes/int/float but %s" % str(type(inputstring)))
         else:
             logger.debug("decoding failed using guess %s for object of type %s with message %s"
                          % (encodingGuess, str(type(inputstring)), str(e)))
